#!/usr/bin/perl

#
# Defines
#

# Files to edit
$grub_file = "menu.lst";
$grub_dir = "/boot/grub";
$elilo_file = "elilo.conf";
$elilo_dir = "/etc";
	
# The parameters
	
$param1_name="udev_timeout";
$param1_value="600";
$param2_name="fc_rport_scan_timeout";
$param2_value="0";

# Patch files and directories

$udev_dir = "/etc/init.d";
$udev_patch = "boot.udev.patch";
$udev_reversepatch = "boot.udev.reversepatch";
$mkinitrd_dir = "/sbin";
$mkinitrd_patch = "mkinitrd.patch";
$mkinitrd_reversepatch = "mkinitrd.reversepatch";

# Make initrd utility
$mkinitrd = "./make_initrd";

# System version file
$versionfile = "/etc/SuSE-release";
$no_supported_msg = "These patches are not supported on this system\n";

# Other flags

$initrd_needs_rebuild = 0;
$uflag = 0;
$TRUE = 1;
$FALSE = 0;
$BAD = 1;

#
# Functions
#

# Name: ParseArgs
# Description: Parse all command line arguments and set the
#              appropriate flags
# In: @paramters - command line arguments
# Out: None
# Returns: None
sub ParseArgs {
        @parameters = @_;
        my $size = @parameters;
        my $i = 0;

        for ($i = 0; $i < $size; $i = $i + 1)
        {
                my $arg = $parameters[$i];

                if (($arg eq "-u") || ($arg eq "--uninstall")) {
                        $uflag = 1;
                }
                else {
                        print "$arg is an invalid flag\n";
                        exit ($BAD);
                }
        }
}

# Name: Diff
# Description: Runs diff on two files and returns whether there are any
#              differences
# In: f1 - File one
# Out: f2 - File two
# Returns: TRUE if the files are different, FALSE otherwise 
sub Diff {
	my $f1 = $_[0];
	my $f2 = $_[1];

	# Execute diff command but send output to /dev/null
	system ("diff $f1 $f2 1>/dev/null 2>/dev/null");

	# Need to shift right the return value by 8 to get the actual
	# return value of the process
	return $? >> 8;		
}

# Name: Copy
# Description:  Copies a file from one place to another
# In: f1 - source file
#     f2 - destination file
# Out: None
# Returns: None
sub Copy {
	my $source = $_[0];
	my $dest = $_[1];

	# Execute copy command
	system ("cp -f $source $dest");
}

# Name: Move
# Description: Moves a file from one place to another
# In: f1 - source file
#     f2 - destination file
# Out: None
# Returns: None
sub Move {
	my $source = $_[0];
	my $dest = $_[1];

	# Execute copy command
	system ("mv -f $source $dest");
}

# Name: Remove
# Description: Deletes a file
# In: f1 - File to delete
# Out: None
# Returns: None
sub Remove {
	my $f1 = $_[0];
	
	# Execute rm command
	system ("rm -f $f1");
}

# Name: ChangeKernelParameters
# Description: Adds kernel parameters to the boot loader configurtion file
# In: None
# Out: None
# Returns: None
sub ChangeKernelParameters {
	my $arch = `uname -m`;
	chomp ($arch);

	# Which boot loader we edit depends on the architecture

	if ($arch eq "ia64") {
		EditElilo();
	}
	else {
		EditGrub();
	}
}

# Name: RemoveKernelParameters
# Description: Remove kernel parameters to the boot loader configurtion file
# In: None
# Out: None
# Returns: None
sub RemoveKernelParameters {
	my $arch = `uname -m`;
	chomp ($arch);

	# Which boot loader we edit depends on the architecture

	if ($arch eq "ia64") {
		RemoveKernelParametersFromFile("$elilo_dir/$elilo_file", $FALSE);
	}
	else {
		RemoveKernelParametersFromFile("$grub_dir/$grub_file", $TRUE);
	}
}

# Name: RemoveKernelParametersFromFile
# Description: Removes the kernel parameters from the boot loader config file
# In: filename - Boot loader file
#     grub - Whether grub is the bootloader
# Out: None
# Retursn: None
sub RemoveKernelParametersFromFile {
        my $filename = $_[0];
	my $grub = $_[1];
	my $altered_line = $FALSE;

	# First we need to produce a new file so that we can compare it to 
	# the original file
		
	my $origfile = "$filename";
	my $newfile = "$filename.new";
	
	# Open the source and destination file

        open (SOURCE, $origfile) || die "Cannot open $origfile";
        open (DEST, ">$newfile") || die "Cannot open $newfile";

        # Loop through the whole file and look for "append" lines

        while (<SOURCE>) {
                my $line = $_;

		# Check to see if we have either parameter in this kernel line

		$altered_line = $FALSE;
		if (($line =~ m/$param1_name/) || ($line =~ m/$param2_name/)) {
			$altered_line = $TRUE;
		}
		
		# Apply regular expression filters to remove the parameters
		$line =~ s/\s?$param1_name=$param1_value\s?//g;
		$line =~ s/\s?$param2_name=$param2_value\s?//g;

		# If we are using grub and this is a kernel line we need
		# to add a newline

		if (($grub == $TRUE) && ($line =~ m/kernel/) && ($altered_line == $TRUE)) {
			$line = $line."\n";
		}
		
		# Print the line to the new file
		print DEST $line;
	}

        # Close 'em out

        close (SOURCE);
        close (DEST);
	
	# Check to see if we even need to edit the existing file
	my $rc = Diff($origfile, $newfile);
	
	if ($rc == $TRUE) {
        	# Copy to the new file into place
		
		print "Removing $param1_name=$param1_value and $param2_name=$param2_value from $origfile\n";
		Move ($newfile, $origfile);
	}
	else {
		Remove ($newfile);
	}
}

# Name: EditElilo
# Description: Adds parameters to the end of the append line in elilo.conf 
# In: None
# Out: None
# Returns: None
sub EditElilo {
        #
	# First we need to produce a new file so that we can compare it to 
	# the original file
		
	my $origfile = "$elilo_dir/$elilo_file";
	my $newfile = "$elilo_dir/$elilo_file.new";
	
	# Open the source and destination file

        open (SOURCE, $origfile) || die "Cannot open $origfile";
        open (DEST, ">$newfile") || die "Cannot open $newfile";

        # Loop through the whole file and look for "append" lines

        while (<SOURCE>) {
                my $line = $_;
                my $newkernelline = "";

                # If we fine an "append" line add, the needed parameters

                if ($line =~ m/ append /) {
                        $newkernelline = AddParametersElilo($line);
                }

                # If we changed the kernel line add it to the new file
                # otherwise copy the current line to the new file as is

                if ($newkernelline ne  "") {
                        print DEST $newkernelline;
                }
                else {
                        print DEST $line;
                }
        }

        # Close 'em out

        close (SOURCE);
        close (DEST);
	
	# Check to see if we even need to edit the existing file
	my $rc = Diff($origfile, $newfile);
	
	if ($rc == $TRUE) {
        	# Save the original file
		if (!(-e "$save_dir/$elilo_file.save") && (-d $save_dir)) {
                	print "Saving $origfile to $save_dir/$elilo_file.save\n";
                	Copy ("$origfile", "$save_dir/$elilo_file.save");
        	}

		# Now copy the patch
        	
		print "Adding $param1_name=$param1_value and $param2_name=$param2_value to $origfile\n";
		Move ($newfile, $origfile);
	}
	else {
		Remove ($newfile);
	}
}

# Name: AddParametersElilo
# Description: Adds new parameters to the append line
# In: Original "append" line
# Out: None
# Returns: New "append" line with new parameters added
sub AddParametersElilo {
	my $origline = $_[0];
	chomp ($origline);
	my $newline = "";
	my $newparameter1 = "";
	my $newparameter2 = "";
	my $param1_found = 0;
	my $param2_found = 0;

	# Extract the kernel parameters from the append string
	
	@tmp1 = split (/ = /, $origline); # Split at the equal sign
	$tmp2 = $tmp1[1]; # Extract the second field
	$tmp2 =~ s/"//g; # eliminate quotation marks
		
	# Finally cut the string into the different kernel parameters
	my @kernelparameters = split(/ /, $tmp2);
	
	# Loop through all the kernel parameters 
	foreach $parameter(@kernelparameters) {
		# Test to see if the current parameter is one of the ones that
		# we want to change.  If it is, change it's value and flag that
		# we found it
		
		$newparameter1 = TestAndChangeParameter1($parameter);				
		$newparameter2 = TestAndChangeParameter2($parameter);
		
		if ($newparameter1 ne "") {
			$newline = $newline." ".$newparameter1;
			$param1_found = 1;
		}
		elsif ($newparameter2 ne "") {
			$newline = $newline." ".$newparameter2;
			$param2_found = 1;
		}
		else {
			$newline = $newline." ".$parameter;
		}
	}

	# If we didn't find the parameters we want to change in the current
	# kernel command line append them at the end
	
	if ($param1_found == 0) {
		$newline = $newline." $param1_name=$param1_value";
	}
	
	if ($param2_found == 0) {
		$newline = $newline." $param2_name=$param2_value";
	}

	# Chop extra first space
	substr ($newline, 0, 1) = "";
	
	# Form the append line
	$newline = "    append = \"$newline\"\n";
	
	return $newline;
}

# Name: EditGrub 
# Description: Adds parameters to the end of the kernel line in menu.lst
# In: None
# Out: None
# Returns: None
sub EditGrub {

	my $origfile = "$grub_dir/$grub_file";
	my $newfile = "$grub_dir/$grub_file.new";
	
	# Open the source and destination file
	
	open (SOURCE, $origfile) || die "Cannot open $origfile";
	open (DEST, ">$newfile") || die "Cannot open $$newfile";

	# Loop through the whole file and look for "kernel" lines
	
	while (<SOURCE>) {
		my $line = $_;
		my $newkernelline = "";

		# If we fine a "kernel" line add the needed parameters
		
		if ($line =~ m/\skernel /) {
			$newkernelline = AddParametersGrub($line);
		}

		# If we changed the kernel line add it to the new file
		# otherwise copy the current line to the new file as is
		
		if ($newkernelline ne  "") {
			print DEST $newkernelline;
		}
		else {
			print DEST $line;
		}
	}

	# Close 'em out
	
	close (SOURCE);
	close (DEST);
	
	# Check to see if we even need to edit the existing file
	my $rc = Diff($origfile, $newfile);
	
	if ($rc == $TRUE) {
        	# Save the original file
		if (!(-e "$save_dir/$grub_file.save") && (-d $save_dir)) {
                	print "Saving $origfile to $save_dir/$grub_file.save\n";
                	Copy ("$origfile", "$save_dir/$grub_file.save");
        	}

		# Now copy the patch
        	
		print "Adding $param1_name=$param1_value and $param2_name=$param2_value to $origfile\n";
		Move ($newfile, $origfile);
	}
	else {
		Remove ($newfile);
	}
}

# Name: AddParametersGrub
# Description: Adds new parameters to the kernel line
# In: Original "kernel" line
# Out: None
# Returns: New "kernel" line with new parameters added
sub AddParametersGrub {
	my $origline = $_[0];
	chomp ($origline);
	my $newline = "";
	my @kernelparameters = split(/ /, $origline);
	my $newparameter1 = "";
	my $newparameter2 = "";
	my $param1_found = 0;
	my $param2_found = 0;

	# Loop through all the kernel parameters 
	foreach $parameter(@kernelparameters) {
		# Test to see if the current parameter is one of the ones that
		# we want to change.  If it is, change it's value and flag that
		# we found it
		
		$newparameter1 = TestAndChangeParameter1($parameter);				
		$newparameter2 = TestAndChangeParameter2($parameter);
		
		if ($newparameter1 ne "") {
			$newline = $newline." ".$newparameter1;
			$param1_found = 1;
		}
		elsif ($newparameter2 ne "") {
			$newline = $newline." ".$newparameter2;
			$param2_found = 1;
		}
		else {
			$newline = $newline." ".$parameter;
		}
	}

	# If we didn't find the parameters we want to change in the current
	# kernel command line append them at the end
	
	if ($param1_found == 0) {
		$newline = $newline." $param1_name=$param1_value";
	}
	
	if ($param2_found == 0) {
		$newline = $newline." $param2_name=$param2_value";
	}

	# Add newline and chomp off the first space
	
	$newline = $newline."\n";
	substr ($newline, 0, 1) = "";
	
	return $newline;
}

# Name: TestAndChangeParameter1
# Description: Tests the parameter passed to it to see if it is the parameter
#              we want to change and if it is, change it
# In: a - parameter to examine
# Out: None
# Returns: "" if this was not the parameter we're looking for, the new value
#          for the parameter otherwise
sub TestAndChangeParameter1 {
	my $a = $_[0];

	my @parameter = split (/=/, $a);
	my $name = $parameter[0];
	my $value = $parameter[1];

	if ($name ne $param1_name) {
		return "";
	}
	else {
		$value = $param1_value;
	}

	return "$name=$value";
}

# Name: TestAndChangeParameter2
# Description: Tests the parameter passed to it to see if it is the parameter
#              we want to change and if it is, change it
# In: a - parameter to examine
# Out: None
# Returns: "" if this was not the parameter we're looking for, the new value
#          for the parameter otherwise
sub TestAndChangeParameter2 {
	my $a = $_[0];

	my @parameter = split (/=/, $a);
	my $name = $parameter[0];
	my $value = $parameter[1];

	if ($name ne $param2_name) {
		return "";
	}
	else {
		$value = $param2_value;
	}

	return "$name=$value";
}

# Name: NeedToApplyPatch
# Description: Tests whether we need to apply a patch
# In: patchdir = Directory to apply patch in
#     patchfile = Name of patch 
# Out: None
# Returns: FALSE if patch has already been applied, TRUE if
#          the patch needs to be applied
sub NeedToApplyPatch {
        $patchdir = $_[0];
	$patchfile = $_[1];

	# Copy patchfile
	Copy ($patchfile, "$patchdir/$patchfile");

        # Test whether we need to apply the patch
	system("cd $patchdir; patch -f --dry-run -p0 < $patchfile 1>/dev/null 2>/dev/null");

        # 1 means the patch failed and we do not watch to try to apply the patch
        # whereas 0 means the dry run is successful so apply the patch
        my $rc =  $? >> 8;

	# Remove patchfile
	Remove ("$patchdir/$patchfile");

	if ($rc == 0) {
		return ($TRUE);
	}
	else {
		return ($FALSE);
	}
}
	

# Name: ApplyPatch
# Description: Generic patch apply function
# In: patchdir - Directory to apply patch in 
#     patchfile - Name of patch 
# Out: None
# Returns: None 
sub ApplyPatch {
	$patchdir = $_[0];
        $patchfile = $_[1];

	if (NeedToApplyPatch($patchdir, $patchfile) == $TRUE) {
	        # Copy patchfile
        	Copy ($patchfile, "$patchdir/$patchfile");

        	# Test whether we need to apply the patch
        	system("cd $patchdir; patch -p0 < $patchfile");
		
		# Remove patchfile
        	Remove ("$patchdir/$patchfile");
	}
	else {
        	print "$patchfile patch does not need to be applied\n";
	}
}
 
# Name: ApplyUdevPatch
# Description: Applies the boot.udev patch 
# In: None
# Out: None
# Returns: None
sub ApplyBootUdevPatch {
	if (NeedToApplyPatch($udev_dir, $udev_patch) == $TRUE) {
        	# Save the original file
		if (!(-e "$save_dir/$udev_file.save") && (-d $save_dir)) {
                	print "Saving $udev_file to $save_dir/$udev_file.save\n";
                	Copy ("$udev_dir/$udev_file", "$save_dir/$udev_file.save");
        	}

		# Acutall apply the patch
		ApplyPatch ($udev_dir, $udev_patch);

		# Mark that we need an initrd rebuild
                $initrd_needs_rebuild = 1;
	}
}

# Name: ApplyMkinitrdPatch
# Description: Applies the mkinitrd patch 
# In: None
# Out: None
# Returns: None
sub ApplyMkinitrdPatch {
        if (NeedToApplyPatch($mkinitrd_dir, $mkinitrd_patch) == $TRUE) {
                # Save the original file
                if (!(-e "$save_dir/$mkinitrd_file.save") && (-d $save_dir)) {
                        print "Saving $mkinitrd_file to $save_dir/$mkinitrd_file.save\n";
                        Copy ("$mkinitrd_dir/$mkinitrd_file", "$save_dir/$mkinitrd_file.save");
                }

                # Now copy the patch
		ApplyPatch ($mkinitrd_dir, $mkinitrd_patch);

                # Mark that we need an initrd rebuild
                $initrd_needs_rebuild = 1;
        }
}

# Name: ApplyBootUdevReversePatch
# Description: Removes the patch to boot.udev 
# In: None
# Out: None
# Returns: None
sub ApplyBootUdevReversePatch {
        if (NeedToApplyPatch($udev_dir, $udev_reversepatch) == $TRUE) {
                # Acutally apply the patch
                ApplyPatch ($udev_dir, $udev_reversepatch);

                # Mark that we need an initrd rebuild
                $initrd_needs_rebuild = 1;
        }
}

# Name: ApplyMkinitrdPatch
# Description: Removes the patch to mkinitrd 
# In: None
# Out: None
# Returns: None
sub ApplyMkinitrdReversePatch {
        if (NeedToApplyPatch($mkinitrd_dir, $mkinitrd_reversepatch) == $TRUE) {
                # Actually apply the patch 
                ApplyPatch ($mkinitrd_dir, $mkinitrd_reversepatch);

                # Mark that we need an initrd rebuild
                $initrd_needs_rebuild = 1;
        }
}

# Name: RebuildInitrd
# Description: Rebuilds the initrd
# In: None
# Out: None
# Returns: None
sub RebuildInitrd () {
	my $hpnoinitrd = $ENV{HPNOINITRD};

	if ($hpnoinitrd eq "y") {
		return;
	}

	if (-e $mkinitrd) {
		print "\n";
		system ($mkinitrd);
		print "\n";
	}
	else {
		print "\n$mkinitrd does not exist.  You will have to rebuild the initrd manually\n";
	}
}

# Name: CheckVersion
# Description: Checks to make sure that we are running SLES 10 SP 1
# In: None
# Out: None
# Returns: None
sub CheckVersion {
	if ((-e $versionfile)) {
		my $oslabel = `grep \"SUSE Linux Enterprise Server 10\" $versionfile`;
		my $patchlevel = `grep \"PATCHLEVEL = 1\" $versionfile`;

		if (($oslabel eq "") || ($patchlevel eq "")) {
			print $no_supported_msg;
			exit ($BAD);
		}	
	}
	else {
		print $no_supported_msg;
		exit ($BAD);
	}
}

# Name: Install
# Description: Installs patches and kernel parameter changes
# In: None
# Out: None
# Returns: None
sub Install {
	# Add kernel parameters to menu.lst as needed
        ChangeKernelParameters();

	# Apply boot.udev patch
	ApplyBootUdevPatch();

	# Apply mkinitrd patch
	ApplyMkinitrdPatch();

	# Rebuild the initrd

	if ($initrd_needs_rebuild == 1) {
        	RebuildInitrd();
	}
}

# Name: Uninstall
# Description: Uninstalls patches and kernel parameter changes
# In: None
# Out: None
# Returns: None
sub Uninstall {
	# Remove the kernel paramters from the boot loader config file
        RemoveKernelParameters();

	# Remove boot.udev patch
        ApplyBootUdevReversePatch();

        # Remove mkinitrd patch
        ApplyMkinitrdReversePatch();

        # Rebuild the initrd

        if ($initrd_needs_rebuild == 1) {
                RebuildInitrd();
        }
}

#
# Script Main
#

# Check to make sure we're running SLES 10 SP 1
CheckVersion();

# Read command line arguments

@arguments = @ARGV;
ParseArgs(@arguments);

if ($uflag == 1) {
	Uninstall();
}
else {
	Install();
}

