/*******************************************************************
 * This file is part of the Emulex Linux Device Driver for         *
 * Fibre Channel Host Bus Adapters.                                *
 * Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
 * EMULEX and SLI are trademarks of Emulex.                        *
 * www.emulex.com                                                  *
 *                                                                 *
 * This program is free software; you can redistribute it and/or   *
 * modify it under the terms of version 2 of the GNU General       *
 * Public License as published by the Free Software Foundation.    *
 * This program is distributed in the hope that it will be useful. *
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
 * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
 * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
 * more details, a copy of which can be found in the file COPYING  *
 * included with this package.                                     *
 *******************************************************************/

/* 
 * $Id: mpapi.h 1005 2006-01-27 17:27:22Z hdaschbach $
 */

#ifndef _H_MPAPI
#define _H_MPAPI

#ifdef __cplusplus
extern "C" {
#endif

/*
 * An identifier initialized to the value of a NULL identifier
 */

extern elxDiDeviceIdentifier mplNullIdentifier; /* = {0} */

                                             
/***************************************************************************
 *
 * mplInitialize
 *
 * Initialize the MultiPulse API Library. 
 * Return the version of the MultiPulse library.
 *
 * Parameters:
 *
 * Version        - points to a buffer to receive the MultiPulse version
 *
 * Length         - contains the number of bytes available in the buffer
 *                  pointed to by Version 
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the initialization is successful. Other status
 * indicative of the failure if the initialization is not successful.
 *
 * Comments:
 *
 * This function must be called to initialize the MultiPulse library before
 * any other MultiPulse library calls are made.
 *
 ***************************************************************************/

MPL_STATUS mplInitialize(char *Version, 
                         unsigned int Length);

                         
                         
                         
/***************************************************************************
 *
 * mplGetPortList
 *
 * Retreive a list of identifiers to be used to identify the different ports
 * available to the MultiPulse library.
 *
 * Parameters:
 *
 * PortList      - Points a GET_PORT_LIST structure for input and return values
 *                 If PortList->Count == 0, the current count of available
 *                 GET_PORT_LIST identifiers is returned.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if list of port identifiers in GET_PORT_LIST is valid. 
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_BUFFER_TOO_SMALL if the list to receive the identifiers is not
 *     large enough to contain all of the port identifiers available.
 *     Only PortList->Count identifiers are returned.
 *
 * Comments:
 *
 * Call this function to get the available port identifiers to be used in
 * subsequent MultiPulse API library calls.
 *
 * The Count member of GET_PORT_LIST structure must be initialized to the number
 * of elements in the Ports member array, or 0, before calling mplGetPortList.
 *
 ***************************************************************************/

MPL_STATUS mplGetPortList(PMPL_GET_PORT_LIST PortList);




/***************************************************************************
 *
 * mplGetRouteList
 *
 * Retreive a list of identifiers to be used to identify the different routes
 * available to the MultiPulse library.
 *
 * Parameters:
 *
 * RouteList    - Points a GET_ROUTE_LIST structure for input and return values
 *                If RouteList->Count == 0, the current count of available
 *                GET_ROUTE_LIST identifiers is returned.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if list of route identifiers in GET_ROUTE_LIST is valid. 
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_BUFFER_TOO_SMALL if the list to received the identifiers is not
 *    large enough to contain all of the route identifiers available
 *    Only RouteList->Count identifiers are returned.
 *
 * Comments:
 *
 * Call this function to get the available route identifiers to be used in
 * subsequent MultiPulse API library calls.
 *
 * The Count member of GET_ROUTE_LIST structure must be initialized to the number
 * of elements in the Ports member array, or 0, before calling mplGetRouteList.
 *
 ***************************************************************************/

MPL_STATUS mplGetRouteList(PMPL_GET_ROUTE_LIST RouteList);





/***************************************************************************
 *
 * mplGetPathList
 *
 * Retreive a list of identifiers to be used to identify the different paths
 * available to the MultiPulse library for the specified route. If no route is
 * specified, return a list of the unbound paths.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure for input and return values
 *                If PathList->Count == 0, the current count of available
 *                GET_PATH_LIST identifiers is returned.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if list of path identifiers in GET_PATH_LIST is valid. 
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_BUFFER_TOO_SMALL if the list to received the identifiers is not
 *    large enough to contain all of the path identifiers available
 *    Only PathList->Count identifiers are returned.
 *
 * Comments:
 *
 * Call this function to get the available path identifiers to be used in
 * subsequent MultiPulse API library calls.
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of elements in the Ports member array, or 0, before calling mplGetPathList.
 *
 * The Route member of the GET_PATH_LIST structure should contain a valid route
 * identifier returned from a previous call to mplGetRouteList. If the list of
 * free paths is desired, the Route member should be initialized to the value
 * of MplNullIdentifier.
 *
 ***************************************************************************/

MPL_STATUS mplGetPathList(PMPL_GET_PATH_LIST PathList);





/***************************************************************************
 *
 * mplCreateRoute
 *
 * Create a new route from a list of paths / define the behavior of the route.
 *
 * Parameters:
 *
 * PathList          - Points a GET_PATH_LIST structure indicating the
 *                     available paths for the route
 *
 * PathControl       - Points to a path control structure used to control how
 *                     the paths comprising the route will be controlled
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the route is created successfully.
 * Other error codes will indicate the reason for failure.
 *
 * Comments:
 *
 * The path list structure is created to describe the paths to the route.
 * The elements of the path list structure member Paths are obtained from
 * a call to mplGetPaths. The MPL_GET_PATH_LIST RouteId field is used to
 * specify the identifier used to identify the route. This must be unique on
 * the system it is defined. Creating two routes with the same route identifier
 * will lead to an unstable system. The MPL_GET_PATH_LIST Count field is used
 * the specify the number of MPL_PATH_LIST_ENTRYs that make up the route.
 * 
 * A path may be a member of one an only one route. If an attempt is made to
 * add a path to a route and the path is already in use, an error will occur.
 *
 * The PathControl argument should point to one of the various path control
 * structures. The type of multipath control will be obtained from the type
 * field contained in the header of the chosed path control structure.
 *
 ***************************************************************************/

MPL_STATUS mplCreateRoute(PMPL_GET_PATH_LIST PathList,
                          PMPL_PATH_CONTROL_HEADER PathControl);






/***************************************************************************
 *
 * mplDeleteRoute
 *
 * Delete a previously created route 
 *
 * Parameters:
 *
 * RouteIdentifier   - Device identifier to be used to identify the route
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the route is created successfully.
 * Other error codes will indicate the reason for failure.
 *
 * Comments:
 *
 * The route identifier used to identify the route must be unique on the
 * system it is defined. 
 *
 ***************************************************************************/

MPL_STATUS mplDeleteRoute(elxDiDeviceIdentifier *RouteIdentifier);





/***************************************************************************
 *
 * mplGetPathControl
 *
 * Retrieve the Path Controls for a specified route identifier.
 *
 * Parameters:
 *
 * RouteIdentifier  - A valid route identifier used to get the path controls
 *
 * PathControl      - A ptr to the buffer to contain the path controls struture
 *
 * BufferSize       - The size of the buffer pointed to by PathControl
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the PathControl structure is valid 
 * An error indicating the failure if the request is unsuccessful.
 *
 * Comments:
 *
 * The client application allocates a buffer to receive the path control
 * structure and initializes BufferSize with the size of the buffer.
 *
 * RouteId should be set to the identifier for the route from which the path
 * control structure is to be returned.
 *
 ***************************************************************************/

MPL_STATUS mplGetPathControl(elxDiDeviceIdentifier *RouteIdentifier,
                             PMPL_PATH_CONTROL_HEADER PathControl,
                             int BufferSize);






/***************************************************************************
 *
 * mplSetPathControl
 *
 * Set the Path Controls for a specified route identifier.
 *
 * Parameters:
 *
 * RouteIdentifier  - A valid route identifier used to set the path controls
 *
 * PathControl      - A ptr to the buffer containing the path controls struture
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the PathControl had been updated.
 * An error indicating the failure if the request is unsuccessful.
 *
 * Comments:
 *
 * The client builds a Path Control structure and passes to mplSetPathControl
 * to set the new Path Control information for the specified route.
 *
 ***************************************************************************/

MPL_STATUS mplSetPathControl(elxDiDeviceIdentifier *RouteIdentifier,
                             PMPL_PATH_CONTROL_HEADER PathControl);





/*******************************************************************************************
 *
 * mplGetPathInfo
 *
 * Get information about the specified path
 *
 * Parameters:
 *
 * DevId             - The DevId portion of the path identifier
 *
 * ValidationId      - The Validation ID port of the path identifier
 *
 * PathInfo          - A pointer to a buffer to receive the response data
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the PathInfo structure is valid 
 * An error indicating the failure if the request is unsuccessful.
 *
 * Comments:
 *
 * The client initializes the PathInfo Length member with the size of the buffer
 * allocated for PathInfo.
 *
 * The client then indicates the path using the DevId and ValidationId values returned
 * in a path list entry from a call to mplGetPaths.
 *
 *******************************************************************************************/

MPL_STATUS mplGetPathInfo(elxDiDeviceIdentifier *DevId,
                          elxDiDeviceIdentifier *ValidationId,
                          PMPL_PATH_INFO PathInfo);




/***************************************************************************
 *
 * mplGetPathStatus
 *
 * Get the current status of the specified path
 *
 * Parameters:
 *
 * DevId             - The DevId portion of the path identifier
 *
 * ValidationId      - The Validation ID port of the path identifier
 *
 * PathStatus        - A pointer to a buffer to receive the response data
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the PathStatus structure is valid 
 * An error indicating the failure if the request is unsuccessful.
 *
 * Comments:
 *
 * The client initializes the PathStatus Length member with the size of the
 * buffer allocated for PathStatus.
 *
 * The client then indicates the path using the DevId and ValidationId values
 * returned in a path list entry from a call to mplGetPathList.
 *
 ***************************************************************************/

MPL_STATUS mplGetPathStatus(elxDiDeviceIdentifier *DevId,
                            elxDiDeviceIdentifier *ValidationId,
                            PMPL_PATH_STATUS PathStatus);





/***************************************************************************
 *
 * mplSetPathStatus
 *
 * Get the current status of the specified path
 *
 * Parameters:
 *
 * DevId           - The DevId portion of the path identifier
 *
 * ValidationId    - The Validation ID port of the path identifier
 *
 * PathStatus      - A ptr to a PathStatus structure with the new status values
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the update to the path status was successful.
 * An error indicating the failure if the request is unsuccessful.
 *
 * Comments:
 *
 * The client initializes the PathStatus Length member with the size of the
 * buffer allocated for PathStatus.
 *
 * The client sets the mask word to indicate which fields of the structure are
 * to be updated. For example, to update just the status, set the mask to
 * MPL_PATH_MASK_STATE and set the new status value in the status member of the
 * PathStatus structure.
 *
 * The client then indicates the path using the DevId and ValidationId values
 * returned in a path list entry from a call to mplGetPathList.
 *
 ***************************************************************************/

MPL_STATUS mplSetPathStatus(elxDiDeviceIdentifier *DevId,
                            elxDiDeviceIdentifier *ValidationId,
                            PMPL_PATH_STATUS PathStatus);






/***************************************************************************
 *
 * mplAddPathListToRoute
 *
 * Add the paths in the path list to the route specified as the route
 * identifier in the path list.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure indicating the route
 *                and new path identifiers. PathList->Count is initialized
 *                to the number of paths to add to the route.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the paths are successfully added to the route
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_ROUTE_NOT_FOUND if the indicated route identifier is not valid
 * MPL_STATUS_PATH_NOT_FOUND if any of the indicated path identifiers are not
 * valid.
 *
 * Comments:
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of valid path elements in the path list array.
 *
 * The Route member of the GET_PATH_LIST structure should contain the route
 * identifier to indicate the route to be updated.
 *
 ***************************************************************************/

MPL_STATUS mplAddPathListToRoute(PMPL_GET_PATH_LIST PathList);





/***************************************************************************
 *
 * mplRemovePathListFromRoute
 *
 * Remove the paths in the path list from the route specified as the route
 * identifier in the path list.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure indicating the route 
 *                identifier from which the paths in the path list array
 *                are to be removed. PathList->Count is initialized to the
 *                number of paths to remove from the route.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the paths are successfully removed from the route
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_ROUTE_NOT_FOUND if the indicated route identifier is not valid
 * MPL_STATUS_PATH_NOT_FOUND if any of the indicated path identifiers are not
 *
 * Comments:
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of valid path elements in the path list array.
 *
 * The Route member of the GET_PATH_LIST structure should contain the route
 * identifier to indicate the route to be updated.
 *
 ***************************************************************************/

MPL_STATUS mplRemovePathListFromRoute(PMPL_GET_PATH_LIST PathList);






/***************************************************************************
 *
 * mplRegisterForEvent
 *
 * Register a callback function to respond to events
 *
 * Parameters:
 *
 * EventMask        - The event for which this function is to be called
 *
 * DeviceId         - The device for which this event handler is to be called
 *
 * Function         - The function to be called when an event occurs
 *
 * Arg              - An opaque value to be returned as an argument to Function
 * 
 * Return Values:
 *
 * A handle to the event function if successful, EMP_INVALID_HANDLE_VALUE if
 * not successful
 *
 * Comments:
 *
 * The client sets the mask for the type of event the function that is being
 * registered will handle. The function is of the type:
 *
 * VOID (*Function)(PVOID Arg,
 *                  MPL_EVENT_TYPE EventType,
 *                  PVOID EventData)
 *
 * In the callback, the Arg parameter has the same value that was passed in to
 * mplRegisterForEvent. The EventType parameter indicates the type of event.
 * The EventData argument is a pointer to the data associated with the event.
 * There are separate structures for each event type.
 * The structure at EventData is only valid until the handler function returns.
 * Any data that is to be used later must be saved by the handler function.
 *
 * The event handler creates a thread to distribute events as they occur. The
 * event handler should not take a long amount of time to execute as this could
 * cause other events to be delayed.
 *
 * If this function is called multiple times for the same EventType and DeviceId,
 * the function will return the same handle if the Function and Arg are identical, 
 * separate handles will be returned if either of the Function or Arg parameters 
 * are different.
 *
 ***************************************************************************/

MPL_EVENT_HANDLE mplRegisterForEvent(MPL_EVENT_MASK EventMask,
                                     elxDiDeviceIdentifier *DeviceId,
                                     void (*Function)(MPL_EVENT_TYPE EventType,
                                                      void *EventData,
                                                      void *Arg),
                                     void *Arg);





/***************************************************************************
 *
 * mplUnregisterForEvent
 *
 * Unregister a callback function to respond to events
 *
 * Parameters:
 *
 * EventHandle      - Event handle returned from a previous call to
 *                    mplRegisterForEvent
 * 
 * Return Values:
 *
 * None.
 *
 * Comments:
 *
 * The client calls this function to indicate to the library the library should
 * stop indicating events on the particular callback function.
 *
 ***************************************************************************/

void mplUnregisterForEvent(MPL_EVENT_HANDLE EventHandle);





/***************************************************************************
 *
 * mplCommitRouteToNonvolatileStorage
 *
 * Cause information to be written into non-volatile storage so the route can be
 * reconstructed at the next system boot.
 *
 * Parameters:
 *
 * RouteId        - Device Identifier identifying the route
 * 
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the request was successful. If the request was not
 * successful, then an error code indicating the failure.
 *
 * Comments:
 *
 * The client calls this function to cause the indicated route to be saved in
 * non-volatile storage so the route can be reconstructed at the next system
 * restart. The information that will be saved is the current configuration of
 * the route.
 *
 ***************************************************************************/

MPL_STATUS mplCommitRouteToNonvolatileStorage(elxDiDeviceIdentifier *RouteId);





/***************************************************************************
 *
 * mplRemoveRouteFromNonvolatileStorage
 *
 * Cause the route information to be removed from non-volatile storage.
 * This will not remove the route from the current instance of the driver.
 *
 * Parameters:
 *
 * RouteId        - Device Identifier identifying the route
 * 
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the request was successful. If the request was not
 * successful, then an error code indicating the failure.
 *
 * Comments:
 *
 * The client calls this function to cause the indicated route to be removed
 * from non-volatile storage so the route will not be reconstructed at the 
 * next system restart. 
 *
 ***************************************************************************/

MPL_STATUS mplRemoveRouteFromNonvolatileStorage(elxDiDeviceIdentifier *RouteId);




/***************************************************************************
 *
 * mplGetRouteControlTypes
 *
 * Retrieve the different types of route controls available.
 *
 * Parameters:
 *
 * None
 * 
 * Return Values:
 *
 * A pointer to the Route Control Type table. NULL is returned if there are
 * no route control types available
 * 
 * Comments:
 *
 * The client calls this function to retrieve the different route control types
 * avaiable.
 * An example of the different types are Simple Failover, Lun balancing, etc.
 *
 ***************************************************************************/

PMPL_ROUTE_CONTROL_TYPES mplGetRouteControlTypes(void);





/***************************************************************************
 *
 * mplClose
 *
 * The client application calls this routine to indicate that it is finished
 * using the MultiPulse API library. This function frees resources allocated
 * during mplInitialize.
 *
 * Parameters:
 *
 * None.
 *
 * Return Values:
 *
 * None.
 *
 * Comments:
 *
 * This function should be called only when there will be no more calls into
 * the MultiPulse API library by the client application. After this function
 * has been called, the application would have to call mplInitialize to use
 * any MultiPulse API library function calls.
 *
 ***************************************************************************/

void mplClose(void);


/***************************************************************************
 *
 * mplExportPaths
 *
 * Cause the paths in the path list structure to not be used by the 
 * MultiPulse filter driver.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure indicating the paths  
 *                that are to be removed. PathList->Count is initialized to 
 *                the number of paths to remove from filtering. The route
 *                identifier is ignored.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the paths are successfully removed
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_PATH_NOT_FOUND if any of the indicated path identifiers are not
 * found
 *
 * Comments:
 *
 * If MPL_STATUS_PATH_NOT_FOUND is returned, some of the paths may be changed
 * to the not filter status. It is up to the caller to determine if any of the 
 * paths were successfully updated.
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of valid path elements in the path list array.
 *
 * Once the paths are removed from the filter driver, they will no longer be
 * visible via the normal MultiPulse library calls. In order to get a list
 * of paths that are being exported, use mplGetExportedPaths.
 *
 * The windows registry will be updated to reflect successful changes.
 *
 ***************************************************************************/

MPL_STATUS mplExportPaths(PMPL_GET_PATH_LIST PathList);





/***************************************************************************
 *
 * mplImportPaths
 *
 * Cause the paths in the path list structure to be used by the 
 * MultiPulse filter driver.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure indicating the paths  
 *                that are to be restored. PathList->Count is initialized to 
 *                the number of paths to restore to the filter driver. The 
 *                route identifier is ignored.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the paths are successfully restored
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 * MPL_STATUS_PATH_NOT_FOUND if any of the indicated path identifiers are not
 * found
 *
 * Comments:
 *
 * If MPL_STATUS_PATH_NOT_FOUND is returned, some of the paths may have been 
 * restored to filtering. It is up to the caller to determine if any of the 
 * paths were successfully updated.
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of valid path elements in the path list array.
 *
 * The windows registry will be updated to reflect successful changes.
 *
 ***************************************************************************/

MPL_STATUS mplImportPaths(PMPL_GET_PATH_LIST PathList);





/***************************************************************************
 *
 * mplGetExportedPaths
 *
 * Obtain a list of paths that are currently not being filtered by the driver.
 *
 * Parameters:
 *
 * PathList     - Points a GET_PATH_LIST structure indicating the paths  
 *                that are being exporteded. PathList->Count is initialized 
 *                to the number entries available in the path list array. The 
 *                route identifier is ignored.
 *
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if the paths are successfully restored
 * MPL_STATUS_UNSUCCESSFUL completed unsuccessfully for an unspecified reason
 *
 * Comments:
 *
 * The Count member of GET_PATH_LIST structure must be initialized to the number
 * of available path elements in the path list array.
 *
 ***************************************************************************/

MPL_STATUS mplGetExportedPaths(PMPL_GET_PATH_LIST PathList);





/***************************************************************************
 *
 * mplCreateDevIdLookup
 *
 * Create a Device ID Lookup
 *
 * Parameters:
 *
 * KeyLength      - Length of the key field
 *
 * Key            - Binary Key data
 *
 * DataLength     - Length of the data portion
 * 
 * Data           - Binary Data
 * 
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if Created successfully
 * Non-zero error codes if not successful
 *
 * Comments:
 *
 * If the Key already exists, then STATUS_SUCCESS will be returned and the 
 * data will not be updated.
 *
 * If the Data exists, then STATUS_SUCCESS will be returned and the callers 
 * key will be replaced with the current key, assuming the callers key length 
 * matches the existing key length
 *
 ***************************************************************************/

MPL_STATUS mplCreateDevIdLookup(unsigned long KeyLength,
                                char *Key,
                                unsigned long DataLength,
                                const char *Data);
                    


/***************************************************************************
 *
 * mplGetGlobalPreferences
 *
 * Get MultiPulse Global Preferences
 *
 * Parameters:
 *
 * Preferences     - A pointer to the buffer to receive the preferences
 * 
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if Preferences buffer filled in
 * MPL_STATUS_BUFFER_TOO_SMALL if Preferences buffer is too small
 * Other non-zero error codes if not successful
 *
 * Comments:
 *
 * The caller must initialize Preferences->Count with the number of 
 * option elements for which memory has been provided. If the number of 
 * option elements exceeds Preferences->Count, MPL_STATUS_BUFFER_TOO_SMALL
 * will be returned and Preferences->Count will be updated with the actual
 * number of preferences.
 *
 ***************************************************************************/

MPL_STATUS mplGetGlobalPreferences(PMPL_GLOBAL_PREFERENCES Preferences);





/***************************************************************************
 *
 * mplSetGlobalPreferences
 *
 * Set MultiPulse Global Preferences
 *
 * Parameters:
 *
 * Preferences     - A pointer to the buffer to specify the preferences
 *
 * Persistent      - A flag to indicate whether the changes should persist
 * 
 * Return Values:
 *
 * MPL_STATUS_SUCCESS if Preferences are updated
 * Non-zero error codes if not successful
 *
 * Comments:
 *
 * The caller intializes the Preferences structure and option elements. The 
 * Persistent flag should be set to non-zero if the options specified should
 * persist across system boots, otherwise the changes will only be in effect
 * until the system is rebooted.
 *
 * If a particular option element makes no change to the current setting for
 * that option, MPL_STATUS_SUCCESS will be returned.
 *
 ***************************************************************************/

MPL_STATUS mplSetGlobalPreferences(PMPL_GLOBAL_PREFERENCES Preferences,
                                   int Persistent);


/* Get module configuration parameters */
int mplGetCfgParam(MPL_CFGPARAM *cfgparam);

/* Set module configuration parameter */
int mplSetCfgParam(int offset, int value);

#ifdef __cplusplus
}
#endif

#endif  /* _H_MPAPI */
