/*******************************************************************
 * This file is part of the Emulex Linux Device Driver for         *
 * Fibre Channel Host Bus Adapters.                                *
 * Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
 * EMULEX and SLI are trademarks of Emulex.                        *
 * www.emulex.com                                                  *
 *                                                                 *
 * This program is free software; you can redistribute it and/or   *
 * modify it under the terms of version 2 of the GNU General       *
 * Public License as published by the Free Software Foundation.    *
 * This program is distributed in the hope that it will be useful. *
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
 * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
 * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
 * more details, a copy of which can be found in the file COPYING  *
 * included with this package.                                     *
 *******************************************************************/

/* 
 * $Id: mpl_init.c 1251 2006-11-01 00:03:28Z jinfante $
 */

#include <linux/version.h>
#include <linux/spinlock.h>
#include <linux/pci.h>
#include <linux/blkdev.h>
#include <linux/moduleparam.h>
#include <scsi/scsi.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_transport_fc.h>

#include <linux/ctype.h>
#include <linux/string.h>

#include "mpl.h"
#include "mpl_mem.h"
#include "mpl_log.h"
#include "mpl_crtn.h"

#define MPL_SYNTAX_ERR   0xff	

#define IGNORE_LPFC_CONFIGURATION
#define MPULSE

char *lpfc_multipulse[MPL_MAX_STATIC_DEVICES] = { 0 };
char *mpl_exclusion[MPL_MAX_STATIC_DEVICES] = { 0 };
int  mpl_max_static_devices = MPL_MAX_STATIC_DEVICES;
int  mpl_max_exclusion_devices = MPL_MAX_EXCLUSION_DEVICES;
int mpl_num_exclusion_devices = 0;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
module_param_array(lpfc_multipulse, charp, mpl_max_static_devices, 0);
module_param_array(mpl_exclusion, charp, mpl_max_exclusion_devices, 0);
#else
module_param_array(lpfc_multipulse, charp, &mpl_max_static_devices, 0);
module_param_array(mpl_exclusion, charp, &mpl_max_exclusion_devices, 0);
#endif

int  mpl_use_static_config = 1;
int  mpl_use_exclusion_list = 1;
module_param(mpl_use_static_config, int, 0);
module_param(mpl_use_exclusion_list, int, 0);
MODULE_PARM_DESC(lpfc_multipulse, "Define static MultiPulse configuration");
MODULE_PARM_DESC(mpl_exclusion, "Define an exclusion list of Paths");
MODULE_PARM_DESC(mpl_use_static_config, "Use static configuration entries");
MODULE_PARM_DESC(mpl_use_exclusion_list, "Use exclusion list entries");

static void *config_space;
static void *config_ptr;
struct config_device *mpl_config_devices = NULL;
struct config_path *mpl_config_paths = NULL;
struct config_device *mpl_exclusion_device = NULL;

extern int  mpl_auto_configure;

struct config_path *
mpl_get_path_flags(int *flags, uint8_t host_wwpn[8],
		       uint8_t target_wwpn[8], int lun) 
{
	struct config_path *path;

	for (path = mpl_config_paths; path; path = path->next) {
		if (!memcmp(path->hba_wwpn, host_wwpn, sizeof(path->hba_wwpn)) &&
		    !memcmp(path->target_wwpn, target_wwpn, sizeof(path->target_wwpn)) &&
		    path->lun_no == lun) {
			*flags = path->flags;
			path->found = 1;
			return path;
		}
	}
	return NULL;
}

struct config_path *
mpl_get_device_flags_by_sn(int *flags, uint8_t *sn, int len) 
{
	struct config_device *device;

	for (device = mpl_config_devices; device; device = device->next) {
		if ((device->serial_size == len) &&
			(!memcmp(sn, device->serial_number, len))) {
			*flags = device->flags;
			device->found = 1;
			return device->path;
		}
	}
	*flags = 0;
	return NULL;
}

static int strtoi(const char *cp, char **endp, int base)
{
	while (isspace(*cp))
		++cp;
	if (*cp == '-')
		return -simple_strtoul(cp + 1, endp, 0);
	else
		return simple_strtoul(cp, endp, 0);
}

static inline void
mpl_crack_wwn_no_colon(unsigned char wwn[8], const char *cp,
		       const char **wwn_buffer)
{
	int  i;

	memset(wwn, 0, 8);
	for (i = 0; i < 8; ++i) {
		while (isspace(*cp)) {
			++cp;
		}
		if (!isxdigit(*cp)) {
			break;
		}
		wwn[i] = mpl_xtobin(*cp);
		++cp;
		if (isxdigit(*cp)) {
			wwn[i] = (wwn[i] << 4) + mpl_xtobin(*cp);
			++cp;
		}
	}
	if (wwn_buffer) {
		*wwn_buffer = cp;
	}
}

static inline int
mpl_crack_serial(unsigned char *serial_number, int serial_size, const char *cp,
		 const char **next_cp)
{
	unsigned char *sb = serial_number;
	int  i;

	memset(sb, 0, serial_size);
	for (i = 0; i < serial_size; ++i) {
		while (isspace(*cp)) {
			++cp;
		}
		if (!isxdigit(*cp)) {
			break;
		}
		*sb = mpl_xtobin(*cp);
		++cp;
		if (isxdigit(*cp)) {
			*sb = (*sb << 4) + mpl_xtobin(*cp);
			++sb;
			++cp;
		}
	}
	if (next_cp) {
		*next_cp = cp;
	}
	return sb - serial_number;
}

static struct config_device *
mpl_config_device(const char *s, char **ptr)
{
	struct config_device *device;
	int  serial_size;
	unsigned char serial_number[256];
	int  flags;
	const char *cp = s;
	char *endp;

	while (isspace(*cp))
		++cp;
	serial_size = mpl_crack_serial(serial_number, sizeof(serial_number),
				       cp, &cp);
	if (*cp != ':') {
		mpl_print(0, MPL_ERR "Bad device serial: %s (%s)\n", s, cp);
		*ptr = NULL;
		return NULL;
	}
	flags = strtoi(cp + 1, &endp, 0);
	cp = endp;
	while (isspace(*cp) || *cp == ',' || *cp == '"' || *cp == '\'') {
		++cp; ++endp;
	}
	if (*cp != 0) {
		*ptr = endp;
	}
	else {
		*ptr = NULL;
	}
	device = config_ptr;
	config_ptr = device + 1;
	memcpy(device->serial_number, serial_number,
	       sizeof(device->serial_number));
	device->serial_size = serial_size;
	device->flags = flags;
	device->found = 0;
	device->path = NULL;
	device->next = mpl_config_devices;
	mpl_config_devices = device;
	return device;
}

static struct config_path *
mpl_config_path(struct config_device *device, const char *s, char **ptr)
{
	struct config_path *path;
	unsigned char host_wwpn[8];
	unsigned char target_wwpn[8];
	unsigned int  lun;
	int  flags;
	const char *cp = s;
	char *endp;

	mpl_crack_wwn_no_colon(host_wwpn, cp, &cp);
	if (memcmp(cp, ":tgt:", 5)) {
		mpl_print(0, MPL_ERR "Bad hba wwpn: %s\n", s);
		*ptr = NULL;
		return NULL;
	}
	mpl_crack_wwn_no_colon(target_wwpn, cp + 5, &cp);
	if (memcmp(cp, ":lun:", 5)) {
		mpl_print(0, MPL_ERR "Bad target wwpn: %s\n", s);
		*ptr = NULL;
		return NULL;
	}
	lun = simple_strtoul(cp + 5, &endp, 0);	
	cp = endp;
	if (*cp != ':') {
		mpl_print(0, MPL_ERR "Bad lun number: %s\n", s);
		*ptr = NULL;
		return NULL;
	}
	flags = simple_strtoul(cp + 1, &endp, 0);
	if ((flags != 1) && (flags != 2)) {
		mpl_print(0, MPL_ERR "Bad flag: %d\n", flags);
		*ptr = NULL;
		return NULL;
	}
	cp = endp;
	while (isspace(*cp) || *cp == ',' || *cp == '"' || *cp == '\'') {
		++cp; ++endp;
	}
	if (*cp != 0) {
		*ptr = endp;
	}
	else {
		*ptr = NULL;
	}

	path = config_ptr;
	config_ptr = path + 1;

	memcpy(path->hba_wwpn, host_wwpn, sizeof(path->hba_wwpn));
	memcpy(path->target_wwpn, target_wwpn, sizeof(path->target_wwpn));
	path->lun_no = lun;
	path->flags = flags;
	path->device = device;
	path->found = 0;
	path->path = NULL;
	device->path = path;
	path->next = mpl_config_paths;
	mpl_config_paths = path;

	return path;
}

static int
mpl_parse_entry(void)
{
	struct config_device *device = NULL;
	char *cp;
	char *ptr;
	int  i, num_path;

	if (!config_space)
		return 0;
	num_path = 0;
	for (i = 0;
	     i < ARRAY_SIZE(lpfc_multipulse) && lpfc_multipulse[i] != 0;
	     ++i) {
		cp = lpfc_multipulse[i];
		while (isspace(*cp) || *cp == '"' || *cp == '\'')
			++cp;

		/* This while loop is just in case multiple entries are on one line */
		while (cp) {
			if (!memcmp(cp, "dev:", 4)) {
				device = mpl_config_device(cp + 4, &ptr);
				num_path = 0;
				if (!device) {
					mpl_print(0, MPL_ERR
					  "Syntax error: invalid device %s\n",
					  lpfc_multipulse[i]);
				}
			} else if (!memcmp(cp, "hba:", 4)) {
				num_path++;
				if (!device || (num_path > 1) ||
					!mpl_config_path(device, cp+4, &ptr)) {
					mpl_print(0, MPL_ERR
					  "Syntax error: invalid path %s\n",
					  lpfc_multipulse[i]);
					if (num_path == 1)
						num_path = 0;
				}
			}
			cp = ptr;
		}
	}
	return i;
}

static struct config_device *
mpl_dummy_device(void)
{
	struct config_device *device;

	if (!config_space)
		return 0;

	device = config_ptr;
	config_ptr = device + 1;
				/* Ensure we don't match serial number
				 * with null serial number.
				 */
	device->serial_size = sizeof(device->serial_size);
	device->flags = 0;
	device->next = mpl_config_devices;
	mpl_config_devices = device;
	mpl_exclusion_device = device;
	return device;
}

static int
mpl_parse_exclusion(void)
{
	struct config_device *device = mpl_dummy_device();
	char *cp;
	char *ptr;
	int  i;

	if (!config_space)
		return 0;
	for (i = 0;
	     i < ARRAY_SIZE(mpl_exclusion) && mpl_exclusion[i] != 0;
	     ++i) {
		cp = mpl_exclusion[i];
		while (isspace(*cp) || *cp == '"' || *cp == '\'')
			++cp;

		/* This while loop is just in case multiple entries are on one line */
		while (cp) {
			if (!memcmp(cp, "hba:", 4)) {
				if (!device || !mpl_config_path(device, cp + 4, &ptr)) {
					mpl_print(0, MPL_ERR
					  "Syntax error: invalid path %s\n",
					  mpl_exclusion[i]);
				} else {
					mpl_config_paths->flags |= MPL_CFG_PATH_EXPORT;
					mpl_num_exclusion_devices++;
				}
			}
			cp = ptr;
		}
	}
	return i;
}

static int
mpl_alloc_config_tables(void)
{
	int  device_count = 0;
	int  path_count = 0;
	char *cp;
	int  i;

	if (mpl_use_static_config) {
		for (i = 0;
		     i < ARRAY_SIZE(lpfc_multipulse) && lpfc_multipulse[i] != 0;
		     ++i) {
			cp = lpfc_multipulse[i];
			while (isspace(*cp) || *cp == '"' || *cp == '\'')
				++cp;
			if (!memcmp(cp, "dev:", 4)) {
				++device_count;
			} else if (!memcmp(cp, "hba:", 4)) {
				++path_count;
			} else {
				mpl_print(0, MPL_ERR "Syntax error: unrecognized "
				  "configuration: %s\n", lpfc_multipulse[i]);
				return 1;
			}
		}
	}
	if (mpl_use_exclusion_list) {
		if (mpl_exclusion[0] != 0) {
			++device_count;
		}
		for (i = 0;
		     i < ARRAY_SIZE(mpl_exclusion) && mpl_exclusion[i] != 0;
		     ++i) {
		
			cp = mpl_exclusion[i];
			while (isspace(*cp) || *cp == '"' || *cp == '\'')
				++cp;
			if (!memcmp(cp, "hba:", 4)) {
				++path_count;
			} else {
				mpl_print(0, MPL_ERR "Syntax error: unrecognized "
				  "exclusion: %s\n", mpl_exclusion[i]);
				return 1;
			}
		}
	}
	if (device_count && path_count) {
		int size = (device_count * sizeof(struct config_device) +
			    path_count * sizeof(struct config_path));

		config_space = kmalloc(size, GFP_KERNEL);
		config_ptr = config_space;
		if (config_space) {
			memset(config_space, size, 0);
		}
		else {
			mpl_print(0, MPL_ERR
			"Cannot alloc memory for static configuration (0x%x)\n",
			size);
		}
	}
	return 0;
}

int
mpl_setup_multipulse(void)
{
	int num;

	if ((ARRAY_SIZE(lpfc_multipulse) && lpfc_multipulse[0] != NULL) ||
	    (ARRAY_SIZE(mpl_exclusion) && mpl_exclusion[0] != NULL)) {
		if (mpl_alloc_config_tables())
			return 1;
	}

	if (mpl_use_static_config && ARRAY_SIZE(lpfc_multipulse) &&
						lpfc_multipulse[0] != NULL) {
		num = mpl_parse_entry();
		mpl_print(0, MPL_ERR
			"Processed Static Configuration: %d entries\n", num);
	}
	else {
		if (!mpl_auto_configure)
			mpl_auto_configure = MPL_PC_TYPE_FAILOVER;

		mpl_print(0, MPL_ERR
			"Using AutoConfiguration: Type %d\n",
			mpl_auto_configure);
	}

	if (mpl_use_exclusion_list && ARRAY_SIZE(mpl_exclusion) &&
						mpl_exclusion[0] != NULL) {
		num = mpl_parse_exclusion();
		mpl_print(0, MPL_ERR
			"Processing Exclusion List: %d entries\n", num);
	}

	return 0;
}

void mpl_free_initial_data(void)
{
	kfree(config_space);
	config_space = NULL;
}
