/*******************************************************************
 * This file is part of the Emulex Linux Device Driver for         *
 * Fibre Channel Host Bus Adapters.                                *
 * Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
 * EMULEX and SLI are trademarks of Emulex.                        *
 * www.emulex.com                                                  *
 *                                                                 *
 * This program is free software; you can redistribute it and/or   *
 * modify it under the terms of version 2 of the GNU General       *
 * Public License as published by the Free Software Foundation.    *
 * This program is distributed in the hope that it will be useful. *
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
 * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
 * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
 * more details, a copy of which can be found in the file COPYING  *
 * included with this package.                                     *
 *******************************************************************/

/* 
 * $Id: mpl_log.h 1444 2008-03-11 16:36:31Z mukumar $
 */

#ifndef _H_MPL_LOGMSG
#define _H_MPL_LOGMSG

#define MPL_INIT               0x1      /* mpl initialization */
#define MPL_RECOVERY           0x2      /* recovery events */
#define MPL_IOCTL              0x4      /* ioctl events */
#define MPL_HEARTBEAT          0x8      /* very verbose */
#define MPL_TMO                0x10     /* Internal timer expiration */
#define MPL_PSM                0x20     /* Path State Machine transitions */
#define MPL_DSM                0x40     /* Device State Machine transitions */
#define MPL_HBSM               0x80     /* Hbeat State Machine transitions */
#define MPL_IOPATH	       0x100    /* I/O path issues */
#define MPL_INTERNAL           0x200    /* driver initiated internal cmds */
#define MPL_ALUA               0x400    /* ALUA mode (RTPG support) */
#define MPL_DFLT               0x800    /* Default: path state transitions */
#define MPL_HBEATDEFQ          0x1000   /* Heart beat defferal queue handling */
#define MPL_DEBUG	       0x2000   /* Debug to chase some race condition */

/* The first two MPL_ERR, MPL_ALERT will always
 * log a message, thus mask can be 0 in mpl_print for these.
 */
#define MPL_ERR  KERN_ERR"lpfcmpl:"
#define MPL_ALERT  KERN_ALERT"lpfcmpl:"
#define MPL_INFO  KERN_INFO"lpfcmpl:"
#define MPL_WARNING  KERN_WARNING"lpfcmpl:"

#define mpl_print(mask, fmt, arg...)                                    \
	{                                                               \
		if (((mask) & mpl_log_verbose) != 0 ||			\
		    (fmt[0] == '<' && fmt[1] <= '3' && fmt[2] == '>'))	\
			printk(fmt, ##arg);                             \
	}

extern int mpl_log_verbose;

static void inline mpl_log_set_verbose(int level)
{
	mpl_log_verbose = level;
}

#endif
