#!/bin/bash
#
# Name: master.sh
# Copyright: (c) Copyright 2006 Hewlett-Packard Development Company, L.P.
#
# Description: Master build script for MultiPulse RPM 
#
# CD            05/05/06 Initial Development
# CD		07/21/06 Call edit_conf after building the driver

#
# defines
#

SRCDIR="/opt/hp/hp-multipulse"

#
# functions
#

# prints help message

print_help () {
 echo "Usage: $0 [-h -m module_directory ]"
 echo ""
 echo "-m: kernel module directory to use"
 echo "-h: prints help"
 echo ""
 echo "example: $0 -m 2.4.21-9.ELsmp"
}

# This function reads in command line arguments

read_args () {

 if [ $# -gt 0 ]
 then
 	# check arguments

 	MODULEFLAG=0
 	HELPFLAG=0

 	# parse command line into arguments

 	getopt m:h $* 1>/dev/null 2>/dev/null

 	# check result of parsing

 	if [ $? != 0 ]
 	then
 		echo "Bad argument or missing argument"
 		exit 1
 	fi

 	set -- `getopt m:h $*`

 	while [ $1 != -- ]
 	do
 		case $1 in
			-m) MODULEFLAG=1
		    	    MODULEDIR=$2
		    	    shift;;
			-h) HELPFLAG=1;;
			*) echo "$1 is an illegal argument"
		   	   exit 1;;
 		esac
 		shift   # next flag
 	done

 	shift   # skip --

 	if [ $HELPFLAG -eq 1 ]
 	then
 		print_help
 		exit 0
 	fi

 	# make sure that kernel module directory exist

 	if test ! -d /lib/modules/${MODULEDIR}
 	then
 		echo "/lib/modules/${MODULEDIR} does not exist"
 		exit 1
 	fi

 	# set kernel version to be what MODULEDIR is

 	KERNELVERSION=$MODULEDIR
 else
 	# no command line options were given, so use uname -r to get the kernel
 	# version

 	KERNELVERSION=`uname -r`
 fi
}

#
# Script main
#

# read command line args

read_args $*

# Build the kernel module

cd $SRCDIR
./build_driver -m $KERNELVERSION

if [ $? -ne 0 ]
then
 echo ""
 echo "The driver compilation failed!"
 exit 1
fi

# Edit any configuration files if necessary

echo ""
echo "Make changes to configurations files if necessary..."
./edit_conf

# Build the initrd

echo ""
./make_initrd $KERNELVERSION

if [ $? -ne 0 ]
then
 echo ""
 echo "The command to remake the initrd failed.  This will have to be done"
 echo "manually"
fi
 
exit 0


