/*******************************************************************
 * This file is part of the Emulex Linux Device Driver for         *
 * Fibre Channel Host Bus Adapters.                                *
 * Copyright (C) 2003-2008 Emulex.  All rights reserved.           *
 * EMULEX and SLI are trademarks of Emulex.                        *
 * www.emulex.com                                                  *
 *                                                                 *
 * This program is free software; you can redistribute it and/or   *
 * modify it under the terms of version 2 of the GNU General       *
 * Public License as published by the Free Software Foundation.    *
 * This program is distributed in the hope that it will be useful. *
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
 * DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
 * TO BE LEGALLY INVALID.  See the GNU General Public License for  *
 * more details, a copy of which can be found in the file COPYING  *
 * included with this package.                                     *
 *******************************************************************/

/* 
 * $Id: mpl_shim.c 1491 2008-05-29 12:54:30Z jamesw $
 */

#include <linux/version.h>
#include <linux/spinlock.h>
#include <linux/pci.h>
#include <linux/blkdev.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <scsi/scsi.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_transport_fc.h>
#include <linux/smp_lock.h>
extern int scsi_internal_device_block(struct scsi_device *sdev);
extern int scsi_internal_device_unblock(struct scsi_device *sdev);
extern int scsi_queue_insert(struct scsi_cmnd *cmd, int reason);
extern int scsi_delete_timer(struct scsi_cmnd *);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
static int mpl_is_cmd_aborted( struct scsi_cmnd *) ;
#endif

#include "mpl.h"
#include "mpl_mem.h"
#include "mpl_log.h"
#include "mpl_crtn.h"
#include "mpl_version.h"

static int host_cnt = 0;

#define DEFAULT_DEVICE_HASH_SIZE  1024
#define DEFAULT_PATH_HASH_SIZE  1024
#define DEFAULT_STUB_HASH_SIZE  1024

#define NUM_ENTRIES(x) (sizeof(x) / sizeof(x[0]))

extern struct mpl_plugin plugin_data_start;
extern struct mpl_plugin plugin_data_end;
extern int mpl_setup_multipulse(void);

LIST_HEAD(mpl_targets);
LIST_HEAD(mpl_hosts);
static LIST_HEAD(reaper_queue);

LIST_HEAD(deferral_queue); 

/* For debug only */
#ifdef MPL_SCSICMD_SANITY
LIST_HEAD(scsiglobal_queue);
#endif

static int  mpl_thread_quit;
static DECLARE_MUTEX_LOCKED(mpl_thread_sleep);
static DECLARE_MUTEX_LOCKED(mpl_thread_gone);


LIST_HEAD(eh_deferral_queue);
static int  mpl_eh_thread_quit;
static DECLARE_MUTEX_LOCKED(mpl_eh_thread_sleep);
static DECLARE_MUTEX_LOCKED(mpl_eh_thread_gone);

LIST_HEAD(int_scsi_tmo_queue);
spinlock_t stub_queue_lock = SPIN_LOCK_UNLOCKED;

struct mpl_device **device_hash;
int  device_hash_size = DEFAULT_DEVICE_HASH_SIZE;
spinlock_t device_hash_table_lock = SPIN_LOCK_UNLOCKED;
struct mpl_path **path_hash;
int  path_hash_size = DEFAULT_PATH_HASH_SIZE;
spinlock_t path_hash_table_lock = SPIN_LOCK_UNLOCKED;

spinlock_t mpl_module_spin_lock = SPIN_LOCK_UNLOCKED;
unsigned long mpl_module_flag = 0;

#ifdef MPL_SCSICMD_SANITY
void mpl_dump_all_scsi_cmd(void );
int is_mid_layer_timer_pending( struct io_stub *stub );
#endif
/* A workaround to handle internal command tag check
 * in lpfc driver using scsi_populate_tag_msg
 * function.
*/
struct request mpl_internal_req ;

/* alternate paths are standby paths */
int  mpl_auto_configure = MPL_PC_TYPE_FAILOVER;
int  mpl_auto_lun_balance = 1; /* Enabled by default */
int  mpl_auto_sas_opt     = 1; /* Enabled by default */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
int  mpl_serial_scan = 1; /* Enabled by default */
struct mpl_path   *scan_path;
#else
int  mpl_serial_scan = 1; /* Disabled by default */
#endif

int  mpl_initial_scan = 0;
#define MPL_SCAN_INP    1   /* initial scan in progress */
#define MPL_SCAN_DONE   2   /* scan is done */


int (* mpl_base_queuecommand)(struct scsi_cmnd *, void (*done)(struct scsi_cmnd *));
int (* mpl_base_release)(struct Scsi_Host *);
struct scsi_host_template *mpl_template;
int   (* mpl_base_slave_alloc)(struct scsi_device *sdev);
int   (* mpl_base_slave_config)(struct scsi_device *sdev);
void  (* mpl_base_slave_destroy)(struct scsi_device *sdev);

int (* mpl_base_abort_handler)(struct scsi_cmnd *);
int (* mpl_base_device_reset_handler)(struct scsi_cmnd *);
int (* mpl_base_bus_reset_handler)(struct scsi_cmnd *);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
void (* mpl_base_devloss_tmo)(struct fc_rport *);
#endif

void mpl_done(struct scsi_cmnd *);
int mpl_abort_handler(struct scsi_cmnd *);
int mpl_do_dev_reset(struct io_stub *);
int mpl_log_verbose = MPL_DFLT;
int mpl_enable_tgt_qdepth = 1;
volatile int driver_unloading = 0;
atomic_t total_internal_count = ATOMIC_INIT(0);
int mpl_hbeat_interval = MPL_DFT_HBEAT_INTERVAL;
int mpl_hbeat_fail_interval = MPL_DFT_HBEATFAIL_INTERVAL;
int mpl_max_hbeat_retry = MPL_DFT_HBEAT_RETRY;
int mpl_nopath_tmo = 0;
int mpl_msa_nopath_tmo = 60;
int mpl_generic_nopath_tmo = 0;
int mpl_eva_nopath_tmo = 60;
int mpl_enable_nopath = 1;
int mpl_pfq_use_lun_notsup = 1;
int mpl_hbeat_tmo_busy = 1;
int mpl_hbeat_enable = 1;

uint32_t mpl_max_can_queue = 0xffffff;

static struct mpl_device *mpl_match_device_by_serial(struct mpl_device *,
						 unsigned char *, int);
static void mpl_hbeat_timeout(unsigned long);
static void mpl_nopath_timeout(unsigned long);
static void mpl_hb_done(struct scsi_cmnd *);
void scsi_block_requests_wrapped(struct Scsi_Host *);
void scsi_unblock_requests_wrapped(struct Scsi_Host *);
void mpl_timeout_blocked_host(unsigned long);

int mpl_handle_qfull = 0;
module_param(mpl_log_verbose, int, 0);
module_param(mpl_hbeat_interval, int, 0);
module_param(mpl_hbeat_fail_interval, int, 0);
module_param(mpl_auto_configure, int, 0);
module_param(mpl_auto_lun_balance, int, 0);
module_param(mpl_auto_sas_opt, int, 0);
module_param(mpl_serial_scan, int, 0);
module_param(mpl_max_hbeat_retry, int, 0);
module_param(mpl_enable_tgt_qdepth, int, 0);
module_param(mpl_handle_qfull, int, 0);
module_param(mpl_nopath_tmo, int, 0);
module_param(mpl_generic_nopath_tmo, int, 0);
module_param(mpl_msa_nopath_tmo, int, 0);
module_param(mpl_eva_nopath_tmo, int, 0);
module_param(mpl_enable_nopath, int, 0);
module_param(mpl_pfq_use_lun_notsup, int, 0);
module_param(mpl_hbeat_tmo_busy, int, 0);
module_param(mpl_hbeat_enable, int, 0);
MODULE_PARM_DESC(mpl_log_verbose, "Mpl log verbose setting (default = 0)");
MODULE_PARM_DESC(mpl_hbeat_interval, "Seconds between Inquiry heartbeats on a device (default = 30)");
MODULE_PARM_DESC(mpl_hbeat_fail_interval, "Seconds between Inquiry heartbeats on a failed path (default = 1)");
MODULE_PARM_DESC(mpl_auto_configure, "Automatically configured discovered paths into MultiPulse devices");
MODULE_PARM_DESC(mpl_auto_lun_balance, "Balance primary paths across a target for autoconfiguration");
MODULE_PARM_DESC(mpl_auto_sas_opt, "Only balance optimized SAS ports as primary paths");
MODULE_PARM_DESC(mpl_serial_scan, "Serialize SCSI layer scanning to one scsi host at a time");
MODULE_PARM_DESC(mpl_max_hbeat_retry, "Max retries for a failed hbeat cmd");
MODULE_PARM_DESC(mpl_enable_tgt_qdepth, "Enable target queue depth per plugin");
MODULE_PARM_DESC(mpl_nopath_tmo, "Override plugin setting for time to wait when no paths are available");
MODULE_PARM_DESC(mpl_nopath_tmo, "Override plugin setting for time to wait when no paths are available");
MODULE_PARM_DESC(mpl_generic_nopath_tmo, "Override plugin setting for older MSA time to wait when no paths are available");
MODULE_PARM_DESC(mpl_msa_nopath_tmo, "Override plugin setting for (MSA) time to wait when no paths are available");
MODULE_PARM_DESC(mpl_eva_enable_nopath, "Enable setting for (EVA) time to wait when no paths are available");
MODULE_PARM_DESC(mpl_pfq_use_lun_notsup, "Use LUN not supported PFQ when hiding paths from SCSI Mid-layer");
MODULE_PARM_DESC(mpl_handle_qfull, "Tell SCSI Layer to reduce qdepth on QFULL");
MODULE_PARM_DESC(mpl_hbeat_tmo_busy, "Define behavior for a heartbeat cmd timeout");
MODULE_PARM_DESC(mpl_hbeat_enable, "Enable path validation heartbeat");

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
static void mpl_timeout_blocked_sdev(unsigned long);
#endif

extern int mpl_data_save_cmd;

static char *mpl_dstates[] = {
	"DEV_ONLINE",
	"DEV_FAILING",
	"DEV_FAILED",
	"DEV_DISABLED",
};
static char *mpl_pstates[] = {
	"ONLINE",
	"FAILED",
	"STANDBY",
	"DISABLED",
};

/* Ensure can_queue for all mpl hosts are the same, ie the
 * minimum Scsi_Host value for can_queue.
 */
void
mpl_setup_can_queue(struct mpl_host *newhost)
{
	struct Scsi_Host * sh;
	struct mpl_host *host;

	if (newhost->can_queue)
		return;
	sh = newhost->scsi_host;
	newhost->can_queue = sh->can_queue;
	/* we take 5% of available qdepth to handle heartbeat */
	newhost->hbeat_canqueue = ( sh->can_queue * 5 ) / 100 ;
 	/* Allow some space for mpl HeartBeat commands */
 	sh->can_queue -= newhost->hbeat_canqueue ; 
	mpl_print(MPL_HBEATDEFQ,MPL_INFO "hostno %d hbeat_canqueue is %d and canq %d\n",
		newhost->host_no, newhost->hbeat_canqueue , sh->can_queue) ;

	if (!sh || (sh->can_queue >= mpl_max_can_queue) || !sh->can_queue)
		return;
	mpl_max_can_queue = sh->can_queue;

	/* A side effect of this is, if one HBA has a smaller can_queue, ALL
	 * HBAs will get that can_queue.
	 */
	/* Since we are not sure which host is going to be presentation path 
	 * host, we want to sent lowest can_queue to each host 
	 */
	
	list_for_each_entry(host, &mpl_hosts, host_list) {
		mpl_print(MPL_HBEATDEFQ, MPL_INFO "host %d intial can queuie %d initial"
			"hbeatq %d and new queue %d\n",
		 host->host_no, host->scsi_host->can_queue,host->hbeat_canqueue, sh->can_queue );
		/* Since we are reducing can queue because we have an hba which has lower can 
		 * queue  we want to take advanctage of extra can_queue to send more heartbeat
		 * on that hba
		*/
		host->hbeat_canqueue = ((host->scsi_host->can_queue + host->hbeat_canqueue) 
			- sh->can_queue);
		host->scsi_host->can_queue = sh->can_queue;
		
	}
	return;
}


static int
mpl_unblock_scsi_scan(struct mpl_host *host)
{
	struct mpl_host *scanhost;

	if (host && !(host->flags & MPL_HOST_SCAN_INP))
		return 1;

	mpl_module_lock();
	mpl_initial_scan = 0;
	host->flags &= ~MPL_HOST_SCAN_INP;

	mpl_print(MPL_IOPATH, MPL_INFO "scanHost: UNBLOCK: %s\n", mpl_print_host(host));

	/* The scan for the current host is completed, check to see if any other
	 * hosts have had their scanning blocked.
	 */
	list_for_each_entry(scanhost, &mpl_hosts, host_list) {
		if (scanhost->flags & MPL_HOST_SCAN_BLOCKED) {
			scanhost->flags &= ~MPL_HOST_SCAN_BLOCKED;
			scanhost->flags |= MPL_HOST_SCAN_INP;
			mpl_initial_scan = MPL_SCAN_INP;

			mpl_print(MPL_IOPATH, MPL_INFO
				"scanHost: Start scan: %s\n", mpl_print_host(host));

			mod_timer(&scanhost->host_scan, jiffies + MPL_WAIT_FOR_SCAN*HZ);
			mpl_module_unlock();
			scsi_unblock_requests_wrapped(scanhost->scsi_host);
			return 0;
		}
	}
	/* All scan is done, lets start heart beat now */
	if (mpl_hbeat_enable) {
		list_for_each_entry(scanhost, &mpl_hosts, host_list) {
			if (scanhost->flags & MPL_HOST_HBEAT_START)
				continue ;
			scanhost->flags |= MPL_HOST_HBEAT_START ;
			mod_timer(&scanhost->host_hbeat, jiffies + mpl_hbeat_interval * HZ);
		}
	}
	mpl_module_unlock();
	return 0;
}

static void
mpl_timeout_scan_host(unsigned long data)
{
	struct mpl_host *host = (struct mpl_host *)data;

	/* The scan for the current host is not progressing, so start scanning
	 * on any other hosts that are waiting.
	 */
	mpl_print(MPL_IOPATH, MPL_INFO
		"scanHost: Timeout: %s\n", mpl_print_host(host));
	mpl_unblock_scsi_scan(host);
	return;
}

static int
mpl_slave_alloc(struct scsi_device *scsi_devs)
{
	int ret;
	struct mpl_host   *host;
	struct mpl_path   *path;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	struct fc_rport *rport = starget_to_rport(scsi_target(scsi_devs));

	if (!rport || fc_remote_port_chkready(rport))
		return -ENXIO;
#endif

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(scsi_devs->host);
	if (!host) {
		mpl_module_unlock();
		return -ENXIO;
	}
	mpl_get_hba_wwpn(host->hba, &host->wwpn[0]);
	mpl_setup_can_queue(host);
	path = mpl_find_or_create_path(host,
					scsi_devs->id, 
					scsi_devs->lun);
	if (!path) {
		mpl_module_unlock();
		return -ENXIO;
	}
	if (path->sdev  && (path->sdev != scsi_devs)) {
		mpl_print(MPL_IOPATH, MPL_ERR "Change SCSI device for %s %s: "
		  "slave sdev = %d/%d/%d\n",
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  scsi_devs->host->host_no,
		  scsi_devs->id, 
		  scsi_devs->lun);
	}
	else {
		mpl_print(MPL_IOPATH, MPL_INFO "Slave Alloc for %s %s: "
		  "sdev = %d/%d/%d\n",
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  scsi_devs->host->host_no,
		  scsi_devs->id, 
		  scsi_devs->lun);
	}

	mpl_module_unlock();

	ret = mpl_base_slave_alloc(scsi_devs);

	if (!ret)
		path->sdev = scsi_devs;

	if (mpl_serial_scan && (mpl_initial_scan == MPL_SCAN_INP)) {
		mpl_module_lock();
		if (!(host->flags & MPL_HOST_SCAN_INP)) {
			if (!(host->flags & MPL_HOST_SCAN_BLOCKED) && !ret) {
				/* Block this scsi host till scan on
				 * presentation host is done.
				 */
				mpl_module_unlock();
				mpl_print(MPL_IOPATH, MPL_INFO "mpl_slave call block_request for %s %s: "
		  		"sdev = %d/%d/%d\n",
		  		mpl_print_device(path->device), mpl_print_path(path),
		 		scsi_devs->host->host_no, scsi_devs->id, 
			        scsi_devs->lun);

				scsi_block_requests_wrapped(scsi_devs->host);
				mpl_module_lock();
				host->flags |= MPL_HOST_SCAN_BLOCKED;
			}
		}
                else {
			/* We have MPL_RETRY_CMD_TMO secs for slave_configure
			 * to be called
			 */
			mod_timer(&host->host_scan, jiffies + MPL_WAIT_FOR_INQ*HZ);
                }
		mpl_module_unlock();
	}
	return ret;
}

static int
mpl_slave_config(struct scsi_device *scsi_devs)
{
	int ret;
	struct mpl_host   *host;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(scsi_devs->host);
	if (!host) {
		mpl_module_unlock();
		return -ENXIO;
	}
	mpl_module_unlock();

	ret = mpl_base_slave_config(scsi_devs);

	mpl_print(MPL_IOPATH, MPL_INFO "Slave Config: %d slave sdev = %d/%d/%d\n",
		  ret, scsi_devs->host->host_no,
		  scsi_devs->id, 
		  scsi_devs->lun);

	if (!ret) {
		mpl_module_lock();
		if (mpl_serial_scan && !mpl_initial_scan) {
			mpl_initial_scan = MPL_SCAN_INP;
			host->flags |= MPL_HOST_SCAN_INP;
		}
		mpl_module_unlock();
		if (host->flags & MPL_HOST_SCAN_INP) {
	                if (scsi_devs->id == (scsi_devs->host->max_id - 1)) {
		                del_timer_sync(&host->host_scan);
		                mpl_unblock_scsi_scan(host);
	                }
                        else {
				/* This assumes the SCSI layer will call the next
				 * device its scanning for (slave alloc) within 
				 * 3 seconds.
				 * However when storage is really busy its takes
				 * more than 45 second. Give a big time to avoid
				 * parellel scanning.
				 */
			         mod_timer(&host->host_scan, jiffies + MPL_WAIT_FOR_SCAN *HZ); 
                        }
		}
	}
	return ret;
}

static void
mpl_slave_destroy(struct scsi_device *sdev)
{
	struct mpl_host   *host;
	struct mpl_path   *path = NULL;

	if (driver_unloading)
		goto out;
	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sdev->host);
	if (host) {
		path = mpl_find_path(host, sdev->id, sdev->lun);
		if (path) {
			if (path->configured != CONFIG_UNAVAILABLE) {
				mpl_hbeat_state_machine(path, HB_STOP);
				mpl_set_path_configured(path, CONFIG_UNCONFIGURED);
			}
			path->seen_ext_inq = 0;
			path->sdev = 0;
		}
	}
	mpl_module_unlock();

	mpl_print(MPL_IOPATH, MPL_INFO "Slave Destroy for slave sdev = %d/%d/%d\n",
		sdev->host->host_no,
		sdev->id,
		sdev->lun);

	if (mpl_serial_scan && (sdev->id == (sdev->host->max_id - 1))) {
		/* If we hit the max_id for this host, start scanning on
		 * the other scsi hosts we blocked in the process.
		 */
		del_timer_sync(&host->host_scan);
		mpl_unblock_scsi_scan(host);
	}
out:
	mpl_base_slave_destroy(sdev);
	return;
}

/* The following 2 functions support a global lock on the mpl module */
void
mpl_module_lock(void)
{
	unsigned long flag;

	flag = 0;
	spin_lock_irqsave(&mpl_module_spin_lock, flag);
	mpl_module_flag = flag;
	return;
}

void
mpl_module_unlock(void)
{

	unsigned long flag;

	flag = mpl_module_flag;
	spin_unlock_irqrestore(&mpl_module_spin_lock, flag);
	return;
}

/* The following functions support a generic module logging mechanism */
char *
mpl_print_host(struct mpl_host   *host)
{
	static char hostbuf[32];

	snprintf(hostbuf, sizeof(hostbuf), "mplHost:%d sHost:%d",
		 host->host_no,
		 host->scsi_host->host_no);
	return(hostbuf);
}

char *
mpl_print_device(struct mpl_device *device)
{
	static char devbuf[32];

	snprintf(devbuf, sizeof(devbuf), "Dev:%d:%d:%d",
		 device->id.host_no,
		 device->id.tgt_no, device->id.lun_no);
	return(devbuf);
}

char *
mpl_print_path(struct mpl_path   *path)
{
	static char pathbuf[32];

	snprintf(pathbuf, sizeof(pathbuf), "Path:%d:%d:%d",
		 path->host->host_no,
		 path->target, path->lun);
	return(pathbuf);
}

char *
mpl_print_target(struct mpl_target   *tgtp)
{
	static char pathbuf[32];

	snprintf(pathbuf, sizeof(pathbuf), "Tgt:%x:%x:%x:%x:%x:%x:%x:%x",
		tgtp->wwpn[0], tgtp->wwpn[1], tgtp->wwpn[2], tgtp->wwpn[3],
		tgtp->wwpn[4], tgtp->wwpn[5], tgtp->wwpn[6], tgtp->wwpn[7]);
	return(pathbuf);
}

char *
mpl_print_wwpn(unsigned char *wwpn)
{
	static char wwpnbuf[32];

	snprintf(wwpnbuf, sizeof(wwpnbuf), "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
		 wwpn[0], wwpn[1], wwpn[2], wwpn[3], 
		 wwpn[4], wwpn[5], wwpn[6], wwpn[7]); 
	return(wwpnbuf);
}

int
mpl_device_path_cnt(struct mpl_device *device)
{
	struct mpl_path *path;
	int cnt = 0;

	if(!device)
		return 0;

	list_for_each_entry(path, &device->paths, path_list) {
		cnt++;
	}
	return cnt;
}

unsigned char *
mpl_find_cmnd_data(struct scsi_cmnd *cmd, int *plen)
{
	unsigned char *ptr;
	struct scatterlist *sgel;
	void * kaddr;
	int len;

	if (cmd->use_sg) {
		sgel = (struct scatterlist *)cmd->request_buffer;
		kaddr = kmap(sgel->page);
		if (kaddr == NULL)
			return 0;
		ptr = (unsigned char *)kaddr + sgel->offset;
		len = sg_dma_len(sgel);
	}
	else {
		ptr = cmd->request_buffer;
		len = cmd->request_bufflen;
	}
	*plen = len;
	return ptr;
}

void
mpl_release_cmnd_data(struct scsi_cmnd *cmd)
{
	struct scatterlist *sgel;

	if (cmd->use_sg) {
		sgel = (struct scatterlist *)cmd->request_buffer;
		kunmap(sgel->page);
	}
	return;
}

static void
mpl_timeout_hbeat_host(unsigned long data)
{
	struct mpl_host *host = (struct mpl_host *)data;
	struct mpl_device *device;
	struct mpl_path *path;
        int cnt ;
	
	mpl_module_lock();
	cnt = atomic_read(&host->hbeat_count) ;
	if ( cnt != 0 ) {
		/* We still have pending heartbeat reset the timer */
		mpl_module_unlock() ;
		mod_timer(&host->host_hbeat, jiffies + mpl_hbeat_interval * HZ);
		mpl_print(MPL_HBEATDEFQ, MPL_INFO "mpl_timeout_hbeat_host: still have some"
			"pending heartbeat %d\n", cnt );
		return;
	}
	list_for_each_entry(device, &host->devices, devices) {
		list_for_each_entry(path, &device->paths, path_list) {
			if (!(path->flag & PATH_FAST_POLL) &&
				(path->hbstate == HB_WAITING)) {

				/* If we have successful scsi IOs, and no
				 * errors during the past hbeat interval
				 * then we don't need to send a hbeat on
				 * this path.
				 */
				if (!path->hbsm_io_success ||
					path->hbsm_io_error) {
					mpl_module_unlock();
					mpl_hbeat_timeout((unsigned long) path);
					mpl_module_lock();
				}

				/* Zero these cnts for next hbeat interval */
				path->hbsm_io_success = 0;
				path->hbsm_io_error = 0;
			}
		}
	}
	mpl_module_unlock();
	mod_timer(&host->host_hbeat, jiffies + mpl_hbeat_interval * HZ);
	return;
}

/* The mpl module uses 2 different kernel threads for deferred processing
 * of of events, mpl_thread and mpl_eh_thread. 
 * The following routines support waking up mpl_thread and adding/removing
 * events to mpl_thread's deferral_queue.
 */
void
mpl_wake_thread(void)
{
	up(&mpl_thread_sleep);
}

/* The mpl module uses 2 different kernel threads for deferred processing
 * of of events, mpl_thread and mpl_eh_thread. 
 * The following routines support waking up mpl_eh_thread and adding/removing
 * events to mpl_eh_thread's eh_deferral_queue.
 */
void
mpl_wake_eh_thread(void)
{
	up(&mpl_eh_thread_sleep);
}

struct mpl_host *
mpl_find_host(int n)
{
	struct mpl_host *host;

	list_for_each_entry(host, &mpl_hosts, host_list) {
		if (host->host_no == n) {
			return host;
		}
	}
	return NULL;
}

struct mpl_target *
mpl_find_target(struct mpl_path *path)
{
      struct mpl_target *tgtp;

      list_for_each_entry(tgtp, &mpl_targets, target_list) {
              if ((tgtp->host == path->host) &&
                  (tgtp->id == path->target)) {
                      return tgtp;
              }
      }
      return NULL;
}

struct mpl_target *
mpl_find_target_wwpn(struct mpl_host *host, uint64_t wwpn)
{
	struct mpl_target *tgtp;
	uint64_t tgt_wwpn;

	list_for_each_entry(tgtp, &mpl_targets, target_list) {
		tgt_wwpn = wwn_to_u64(tgtp->wwpn);
		if ((tgtp->host == host) &&
			(tgt_wwpn == wwpn)) {
			return tgtp;
		}
	}
	return NULL;
}

struct mpl_host *
mpl_find_or_create_host(int n)
{
	struct mpl_host *host;

	host = mpl_find_host(n);
	if (host == NULL) {
		host = kmalloc(sizeof *host, GFP_ATOMIC);
		if (!host) {
			mpl_print(0, MPL_ERR "Cannot allocate host mem\n");
			return NULL;
		}
		memset(host, 0, sizeof(*host));
		host->host_no = n;
		INIT_LIST_HEAD(&host->devices);
		INIT_LIST_HEAD(&host->lpfc_queue);
		INIT_LIST_HEAD(&host->hbeat_deferral_queue);
		atomic_set(&host->hbeat_count, 0 );
		atomic_set(&host->inflight_hbeat_count, 0 );

		init_timer(&host->host_block);
		host->host_block.function = mpl_timeout_blocked_host;
		host->host_block.expires = 0;
		host->host_block.data = (unsigned long) host;

		init_timer(&host->host_scan);
		host->host_scan.function = mpl_timeout_scan_host;
		host->host_scan.expires = 0;
		host->host_scan.data = (unsigned long) host;

		init_timer(&host->host_hbeat);
		host->host_hbeat.function = mpl_timeout_hbeat_host;
		host->host_hbeat.expires = 0;
		host->host_hbeat.data = (unsigned long) host;

		list_add_tail(&host->host_list, &mpl_hosts);
	}
	return host;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
static void
mpl_dev_loss_timeout(unsigned long data)
{
	struct mpl_target *tgtp = (struct mpl_target *)data;
	struct fc_rport *rport;	  
	struct mpl_path *path;
	struct io_stub *stub;
	unsigned long flags;

	mpl_print(MPL_IOPATH, MPL_ERR "Pseudo dev_loss tmo for %s\n",
	  mpl_print_target(tgtp));

	mpl_module_lock();
	rport = tgtp->rport;
	path = list_entry(list_first(&tgtp->paths), struct mpl_path,
				   tgt_path_list);
	if (rport && path) {
		if((stub = mpl_stub_alloc(0, path, 0))) {
			stub->state = STUB_DQ;
			stub->flag |= MPL_INTERNAL_CMD_STUB;
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub->reason = MPL_DO_DEVLOSS_TMO;
			mpl_add_stub_to_eh_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_module_unlock();
			mpl_wake_eh_thread();
			return;
		}
	}

	list_for_each_entry(path, &tgtp->paths, tgt_path_list) {
		mpl_path_state_machine(path, PATH_DOWN);
		path->lld_blocked = 0;
	}
	mpl_module_unlock();
	return;
}
#endif

struct mpl_target *
mpl_find_or_create_target(struct mpl_host *host, struct mpl_path *path)
{
	struct mpl_target *tgtp;
	struct mpl_path *pathp;

	tgtp = mpl_find_target(path);
	if (tgtp)
		goto out;

	tgtp = kmalloc(sizeof(*tgtp), GFP_ATOMIC);
	if (!tgtp)
		goto out;
	memset(tgtp, 0, sizeof(*tgtp));
	INIT_LIST_HEAD(&tgtp->paths);

	tgtp->host = path->host;
	tgtp->id = path->target;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	init_timer(&tgtp->dev_loss);
	tgtp->dev_loss.function = mpl_dev_loss_timeout;
	tgtp->dev_loss.expires = 0;
	tgtp->dev_loss.data = (unsigned long) tgtp;
#endif

	list_add_tail(&tgtp->target_list, &mpl_targets);
out:
	/* Check to see if target already points to this path */
	list_for_each_entry(pathp, &tgtp->paths, tgt_path_list) {
		if (pathp == path) {
			return tgtp;
		}
	}
	list_add_tail(&path->tgt_path_list, &tgtp->paths);
	atomic_inc(&tgtp->path_ref_count);
	if (path->is_default) {
		atomic_inc(&tgtp->prim_path_ref_count);
	}
	path->tgt = tgtp;
	return tgtp;
}

struct mpl_host *
mpl_find_host_by_scsi_host(struct Scsi_Host *scsi_host)
{
	struct mpl_host *host, *ret_host;

	ret_host = NULL;
	list_for_each_entry(host, &mpl_hosts, host_list) {
		if (host->scsi_host == scsi_host) {
			ret_host = host;
			break;
		}
	}
	return ret_host;
}

static inline int
mpl_mangle_host_pointer(struct mpl_host *host)
{
	return (unsigned long) host >> 8;
}

static inline int
mpl_device_hash_fn(struct mpl_host *host, int target, int lun)
{
	int h;
				/* For lack of imagination... */
	h = mpl_mangle_host_pointer(host) + target * 128 + lun;
	return h & (device_hash_size - 1);

}

static inline int
mpl_path_hash_fn(struct mpl_host *host, int target, int lun)
{
	int h;
				/* For lack of imagination... */
	h = mpl_mangle_host_pointer(host) + target * 128 + lun;
	return h & (path_hash_size - 1);

}

struct mpl_device *
mpl_find_device(struct mpl_host *host, int target, int lun)
{
	struct mpl_device *device;
	int hash = mpl_device_hash_fn(host, target, lun);
	unsigned long flags;

	spin_lock_irqsave(&device_hash_table_lock, flags);
	for (device = device_hash[hash]; device; device = device->next) {
		/* Look for a device that matches the specified presentation id */
		if (device->id.host_no == host->host_no &&
		    device->id.tgt_no  == target  &&
		    device->id.lun_no  == lun)
		{
			spin_unlock_irqrestore(&device_hash_table_lock, flags);
			return device;
		}
	}
	spin_unlock_irqrestore(&device_hash_table_lock, flags);
	return NULL;
}

struct mpl_path *
mpl_find_path(struct mpl_host *host, int target, int lun)
{
	struct mpl_path *path;
	int hash = mpl_path_hash_fn(host, target, lun);
	unsigned long flags;

	spin_lock_irqsave(&path_hash_table_lock, flags);
	for (path = path_hash[hash]; path; path = path->next) {
		if (path->host == host &&
		    path->target  == target  &&
		    path->lun     == lun)
		{
			spin_unlock_irqrestore(&path_hash_table_lock, flags);
			return path;
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	return NULL;
}

void
mpl_set_dev_ctrl_header(struct mpl_device *device, int flag)
{
	if (flag & MPL_PC_CFG_AUTO_FAILBACK)
		device->auto_failback = 1;
	flag = flag & 0xf;
	switch (flag) {
	case MPL_PC_TYPE_FAILOVER:      /* alternate paths are standby paths */
	case MPL_PC_TYPE_ROUND_ROBIN:   /* alt paths balanced by round robin */
	case MPL_PC_TYPE_DYNAMIC_LUNQ:  /* Dynamic balancing algorithm used */
	case MPL_PC_TYPE_DYNAMIC_TGTQ:  /* Dynamic balancing algorithm used */
	case MPL_PC_TYPE_DYNAMIC_HBAQ:  /* Dynamic balancing algorithm used */
	case MPL_PC_TYPE_DYNAMIC_TIME:  /* Dynamic balancing algorithm used */
		device->path_cntl_type = flag;
		break;
	default:
		device->path_cntl_type = MPL_PC_TYPE_FAILOVER; /* default */
	}
	return;
}

static void
mpl_set_device_nopath(struct mpl_device *device)
{
	init_timer(&device->nopath);
	device->nopath.function = mpl_nopath_timeout;
	device->nopath.expires = 0;
	device->nopath.data = (unsigned long) device;
}

static void
mpl_set_path_hbeat(struct mpl_path *path)
{
	init_timer(&path->hbeat);
	path->hbeat.function = mpl_hbeat_timeout;
	path->hbeat.expires = 0;
	path->hbeat.data = (unsigned long) path;
}

static void
mpl_set_path_cmd_scheduler(struct mpl_path *path)
{
	init_timer(&path->sched_cmd);
	path->sched_cmd.function = mpl_path_schedule_cmd;
	path->sched_cmd.expires = 0;
	path->sched_cmd.data = (unsigned long) path;
}

struct mpl_device *
mpl_find_or_create_device(struct mpl_host *host, int channel,
			  int target, int lun)
{
	struct mpl_device *device;
	unsigned long flags;
	int device_hash_index;

	device = mpl_find_device(host, target, lun);
	if (device)
		goto out;

	/* Allocate a new device and the first
	 * path that gets us to it. */
	device = kmalloc(sizeof(*device), GFP_ATOMIC);
	if (!device)
		goto out;	
	memset(device, 0, sizeof(*device));

	device_hash_index = mpl_device_hash_fn(host, target, lun);
	
	device->host = host;
	device->failing_path = 0;
	device->dev_need_inqsn_data = 1;
	device->just_created = 1;
	device->inquiry_heartbeat = 1;
	device->state = DEV_FAILED;

	/* By default, all devices / paths are "imported" for multipulse */
	device->configured = CONFIG_UNCONFIGURED;
	
	INIT_LIST_HEAD(&device->pending_ios);
	INIT_LIST_HEAD(&device->paths);
	mpl_set_device_nopath(device);
	if(mpl_nopath_tmo && mpl_enable_nopath) {
		device->nopath_tmo = mpl_nopath_tmo;
	}
	
	/* Take care of presentation id */
	device->id.host_no = host->host_no;
	device->id.tgt_no  = target;
	device->id.lun_no  = lun;
	
	list_add_tail(&device->devices, &device->host->devices);
	device->host->dev_count++;
	
	spin_lock_irqsave(&device_hash_table_lock, flags);
	device->next  = device_hash[device_hash_index];
	wmb();
	device_hash[device_hash_index] = device;
	spin_unlock_irqrestore(&device_hash_table_lock, flags);
	
out:
	return device;
}

/* Presentation path is always path 0.
 * This could translate to hbaN in the base driver.
 */
int
mpl_is_presentation_path(struct mpl_path *path)
{
	return !mpl_is_path_configured(path) ||
		path == list_entry(list_first(&path->device->paths),
				   struct mpl_path,
				   path_list);
}


/* Detects if there is an outstanding command that will change the
 * array controller priority / ownership of a LUN.
 */
int
mpl_is_out_lun_own_chg(struct mpl_device *device)
{
	struct mpl_path *path;

	list_for_each_entry(path, &device->paths, path_list) {
		if (path->flag & PATH_OUT_CHG_LUN_OWN_CMD) {
			return 1;
		}
	}
	return 0;
}


/* By default, primary path is the same as presentation path,
 * until changed by a library call.  Primary path is identified
 * by the path->is_default setting.
 */
struct mpl_path *
mpl_get_primary_path(struct mpl_device *device)
{
	struct mpl_path *path;

	list_for_each_entry(path, &device->paths, path_list) {
		if (mpl_is_path_primary(path)) {
			return path;
		}
	}
	return 0;
}

int
mpl_set_device_auto_failback(struct mpl_device *device, int boolean)
{
	device->auto_failback = boolean;
	return 0;
}

int
mpl_set_device_optimized(struct mpl_device *device, int boolean)
{
	device->optimized_only = boolean;
	return 0;
}

static void
mpl_revoke_other_primaries(struct mpl_path *path)
{
	struct mpl_device *device = path->device;
	struct mpl_path *p;


	list_for_each_entry(p, &device->paths, path_list) {
		if (p != path) {
			if(p->is_default) {
				p->StaticConfig = 0;
				atomic_dec(&p->tgt->prim_path_ref_count);
			}
			p->is_default = 0;
			if(path->state == PATH_ONLINE)
				path->state = PATH_STANDBY;
		}
	}
}

int
mpl_set_path_primary(struct mpl_path *path, int boolean)
{
	struct mpl_path *next_path;
	int new_primary = !path->is_default;
	int status = 0;
	int fb;

	mpl_print(MPL_IOCTL, MPL_INFO "Set primary path for %s %s: "
		  "current = %d: %d %d\n",
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  mpl_is_path_primary(path),
		  boolean,
		  new_primary);

	if (!boolean) {		/* Let someone grab primary, but don't
				 * arbitrarily try to give up primary.
				 */
		return 0;
	}

	/* Cannot make inactive controller primary */
	if (new_primary && !path->active_controller) {
		status = -EBUSY;
		new_primary = 0;
	}

	/* For ALUA, cannot make non-optimized path primary */
	if (new_primary && path->device->alua_support &&
	   (path->sas != MPL_ALUA_ACTIVE_OPT)) {
		/* Add code here to send STPG */
	}

	if (new_primary) {
		path->is_default = 1;
		atomic_inc(&path->tgt->prim_path_ref_count);
		mpl_print(MPL_IOCTL, MPL_INFO "New primary path for %s %s\n",
			mpl_print_device(path->device), mpl_print_path(path));
		fb = path->device->auto_failback;
		path->device->auto_failback = 1;
		mpl_revoke_other_primaries(path);
		mpl_path_state_machine(path, PATH_UP);
		list_for_each_entry (next_path, &path->device->paths, path_list) {
			if (path != next_path)
				mpl_path_state_machine(next_path, PATH_UP);
		}
		path->device->auto_failback = fb;
	}
	path->device->StaticConfigPrimary = 1;
	path->StaticConfig = 1;
	return status;
}

struct mpl_path *
mpl_get_next_path(struct mpl_path *path)
{
	struct mpl_device *device = path->device;
	struct mpl_path *next_path;
	struct list_head *l;

	l = path->path_list.next;
	if (l == &device->paths) {
		l = l->next;
	}

	next_path = list_entry(l, struct mpl_path, path_list);
	return next_path;
}

struct mpl_path *
mpl_get_next_avail_path(struct mpl_path *path, struct mpl_path *first_path)
{
	struct mpl_device *device = path->device;
	struct mpl_path *next_path;
	struct list_head *l;

	do {
		l = path->path_list.next;
		if (l == &device->paths) {
			l = l->next;
		}

		next_path = list_entry(l, struct mpl_path, path_list);
		if (next_path->active_controller &&
		   (next_path->state == PATH_ONLINE ||
		    next_path->state == PATH_STANDBY)) {
			/* Skip path if the optimized bit for the path is set,
			 * and this path is not optimized,
			 * and there are optimized paths available.
			 */
			if(mpl_is_optimized(device) &&
			   next_path->sas != MPL_ALUA_ACTIVE_OPT &&
			   atomic_read(&device->opt_path_cnt)) {
				path = next_path;
				continue;
			}
			return next_path == first_path ? NULL : next_path;
		}
		path = next_path;
	} while (path != first_path);
	return 0;
}

struct mpl_path *
mpl_find_or_create_path(struct mpl_host *host, int target, int lun)
{
	struct mpl_device *device;
	struct mpl_path   *path;

	/* If the path already exists, return it */
	path = mpl_find_path(host, target, lun);
	if (path) {
		return path;
	}

	/* Otherwise create a new device with this path */
	device = mpl_find_or_create_device(host, 0, target, lun);
	if (!device) {
		mpl_print(0, MPL_ERR "find/create device for %s failed\n",
		mpl_print_host(host));
		return 0;
	}

	path = mpl_add_path(device, host, 0, target, lun);
	if (!path) {
		mpl_print(0, MPL_ERR "find/create path for %s failed\n",
		       mpl_print_device(device));
		return 0;
	}
	device->last_path = path;
	return path;
}

/* NOTE:
 *    adding a new path to an existing mpl_device.
 *
 *    device->host == device->1st_path->host
 *    param "host" for this call applies for this new path ONLY
 *    Therefore, device->host and this "host" should NOT be the same.
 */
struct mpl_path *
mpl_add_path(struct mpl_device *device, struct mpl_host *host,
	int channel, int target, int lun)
{
	struct mpl_path   *path;
	unsigned long flags;
	int path_hash_index = mpl_path_hash_fn(host, target, lun);

	path = kmalloc(sizeof(*path), GFP_ATOMIC);
	if (path == NULL) {
		mpl_print(0, MPL_ERR "Could not alloc path\n");
		return 0;
	}

	memset(path, 0, sizeof(*path));
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	init_timer(&path->sdev_block);
	path->sdev_block.function = mpl_timeout_blocked_sdev;
#endif

	path->device  = device;
	path->channel = channel;
	path->target  = target;
	path->lun     = lun;
	path->host    = host;
	path->active_controller = 1;
	path->sas = MPL_ALUA_ACTIVE_OPT;
	atomic_inc(&device->opt_path_cnt);

	list_add_tail(&path->path_list, &device->paths);

	spin_lock_irqsave(&path_hash_table_lock, flags);
	path->next = path_hash[path_hash_index];
	wmb();
	path_hash[path_hash_index] = path;
	spin_unlock_irqrestore(&path_hash_table_lock, flags);

	mpl_set_path_hbeat(path);
	mpl_set_path_cmd_scheduler(path);
	path->state = PATH_FAILED;
	path->hbstate = HB_STOPPED;
	path->configured = CONFIG_UNCONFIGURED;

	mpl_find_or_create_target(host, path);
	return path;
}

void
mpl_free_path(struct mpl_path *path)
{
	unsigned long flags;
	int  hash;

	mpl_path_state_machine(path, PATH_DISABLE);
	mpl_set_path_configured(path, CONFIG_UNCONFIGURED);
	hash = mpl_path_hash_fn(path->host, path->target, path->lun);
	if (path->tgt_port_info) {
		mpl_data_save_cmd--;
		mpl_put_scsi_buf(path->tgt_port_info);
		path->tgt_port_info = 0;
	}

	list_del(&path->path_list);

	list_del(&path->tgt_path_list);
	atomic_dec(&path->tgt->path_ref_count);
	if (path->is_default) {
		atomic_dec(&path->tgt->prim_path_ref_count);
	}
	if(!atomic_read(&path->tgt->path_ref_count)) {
		list_del(&path->tgt->target_list);
		mpl_module_unlock();
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
		del_timer_sync(&path->tgt->dev_loss);
#endif
		mpl_module_lock();
		kfree(path->tgt);
	}

	spin_lock_irqsave(&path_hash_table_lock, flags);
	if (path_hash[hash] == path) {
		path_hash[hash] = path->next;
	} else {
		struct mpl_path *prev = path_hash[hash];

		while (prev->next && prev->next != path)
			prev = prev->next;
		if (!prev->next) {
			mpl_print(0, MPL_ERR "Cannot find path %p "
				"(%s %s) in path hash.\n",
				path, mpl_print_device(path->device),
				mpl_print_path(path));
			
		} else {
			prev->next = path->next;
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	if (path->delayed_cmd && del_timer(&path->delayed_cmd->cmd.eh_timeout)){
		path->delayed_cmd->cmd.result = ScsiResult(DID_NO_CONNECT, 0);
		mpl_call_done_free_stub(path->delayed_cmd);
		path->delayed_cmd = NULL;
	}
			
	mpl_module_unlock();
	del_timer_sync(&path->hbeat);
	del_timer_sync(&path->sched_cmd);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	del_timer_sync(&path->sdev_block);
#endif
	mpl_module_lock();
	kfree(path);
}

static int
mpl_scan_for_device_hash(struct mpl_device *device)
{
	struct mpl_device *d;
	unsigned long flags;
	int  hash;
				/* If we've lost all paths, need to
				 * scan the hash table to find the
				 * device entry. */
	spin_lock_irqsave(&device_hash_table_lock, flags);
	for (hash = 0; hash < device_hash_size; ++hash) {
		for (d = device_hash[hash]; d; d = d->next) {
			if (d == device) {
				goto out;
			}
		}
	}
	hash = -1;
out:
	spin_unlock_irqrestore(&device_hash_table_lock, flags);
	return hash;
}

static int
mpl_device_is_active(struct mpl_device *device)
{
	struct mpl_path *path;

	if (atomic_read(&device->ref_count)) {
		return 1;
	}
	list_for_each_entry(path, &device->paths, path_list) {
		if (atomic_read(&path->io_count) ||
		    atomic_read(&path->internal_count)) {
			return 1;
		}
	}
	return 0;
}

static void
mpl_remove_device_from_device_hash(struct mpl_device *device)
{
	struct mpl_path *path = mpl_first_path(device);
	unsigned long flags;
	int hash;

	hash = path ? mpl_device_hash_fn(device->host, path->target, path->lun)
		: mpl_scan_for_device_hash(device);
	if (hash < 0) {
		mpl_print(0, MPL_ERR "Device %s not found in hash: %p\n",
			  mpl_print_device(device),
			  device);
		BUG();
	}
	spin_lock_irqsave(&device_hash_table_lock, flags);
	if (!device_hash[hash]) {
		mpl_print(0, MPL_ERR "No such device (%s) in device hash.\n",
			  mpl_print_device(path->device));
	} else if (device_hash[hash] == device) {
		device_hash[hash] = device->next;
	} else {
		struct mpl_device *prev;

		prev = device_hash[hash];
		while (prev->next != NULL && prev->next != device) {
			prev = prev->next;
		};
		if (prev->next == NULL) {
			mpl_print(0, MPL_ERR "Cannot find device %p "
				"(%s) in device hash.\n",
				device, mpl_print_device(path->device));
		} else {
			prev->next = device->next;
		}
	}
	spin_unlock_irqrestore(&device_hash_table_lock, flags);
}

int
mpl_free_device(struct mpl_device *device)
{
	struct list_head *l, *n;

	if (device == NULL) {
		mpl_print(0, MPL_ERR "No device to free\n");
		return 1;
	}

	if (mpl_device_is_active(device)) {
		return 1;
	}

	mpl_device_state_machine(device, NULL, DEV_DISABLE);
	mpl_set_device_configured(device, CONFIG_UNCONFIGURED);
	list_del(&device->devices);

	/* free all paths */
	list_for_each_safe(l, n, &device->paths) {
		mpl_free_path(list_entry(l, struct mpl_path, path_list));
	}
	mpl_module_unlock();
	del_timer_sync(&device->nopath);
	mpl_module_lock();

	kfree(device);
	return 0;
}

static struct mpl_plugin *
mpl_find_plugin(struct scsi_cmnd *cmd, struct mpl_device *device, struct mpl_path *path)
{
	struct mpl_plugin *plugin;
	uint8_t *request;
	int len = 0;

	request = mpl_find_cmnd_data(cmd, &len);
	if ((request == NULL) || (len < 36)) {
		mpl_print(0, MPL_ERR "No data in find plugin len %d: %s %s\n",
      			len, mpl_print_device(path->device),
      			mpl_print_path(path))
		return NULL;
	}
	for (plugin = &plugin_data_start;
	     plugin < &plugin_data_end;
	     ++plugin) {
		if (plugin->probe && plugin->probe(cmd, path)) {
			device->scsi_type = request[0] & 0x1f;
			memcpy(device->vendor, request + 8, 8);
			memcpy(device->product, request + 16, 16);
			memcpy(device->rev, request + 32, 4);
			mpl_release_cmnd_data(cmd);
			return plugin;
		}
	}
	mpl_release_cmnd_data(cmd);
	return NULL;
}

void
mpl_throttle_up_queue_full(struct io_stub *stub)
{
	struct scsi_device *sdev;
	struct mpl_device *device;
	unsigned short depth;
	int tag_type;
#ifndef MSG_ORDERED_TAG
#define MSG_ORDERED_TAG 0x22
#define MSG_SIMPLE_TAG  0x20
#endif	
	if ((!stub->sdev) || (!stub->path->device)){
		return;
	}
	device = stub->path->device;
	sdev = stub->sdev;

	depth = sdev->queue_depth;
	if((!sdev->last_queue_full_count) ||    // never changed yet ???
	   (!device->queue_max) ||		// not init yet ???
	   (device->queue_max == depth)){  	// up to max
		return;
	}

	if ((jiffies >> 4) > sdev->last_queue_full_time + 500){
		depth++;
		if(sdev->ordered_tags)
			tag_type = MSG_ORDERED_TAG;
		else
			tag_type = MSG_SIMPLE_TAG;

		scsi_adjust_queue_depth(sdev, tag_type, depth);
		sdev->last_queue_full_time = jiffies >> 4;
	}
}

void
mpl_throttle_down_queue_full(struct io_stub *stub)
{
	struct scsi_device *sdev;
	struct mpl_device *device;
	unsigned short current_depth, half_depth;

	if (!stub->sdev || !stub->path->device) {
		return;
	}
	device = stub->path->device;
	sdev = stub->sdev;
	if (!sdev->last_queue_full_count) {  /*  never changed ? */
		device->queue_max = sdev->queue_depth;
	}

	half_depth = device->queue_max >> 1;
	current_depth = sdev->queue_depth;
	if (current_depth == device->queue_max || current_depth > half_depth) {
		current_depth = half_depth;
		scsi_track_queue_full(sdev, current_depth);
	} else if (current_depth > 3){
		--current_depth;
		scsi_track_queue_full(sdev, current_depth);
	}
}

void
mpl_call_done_free_stub(struct io_stub *stub)
{
	/* Restore everything the lpfc base driver might change */
	stub->cmdp->result = stub->cmd.result;

	/* Keep track of successful /error SCSI IOs for hbeat */
	if (stub->cmdp->result)
		stub->path->hbsm_io_error++;
	else
		stub->path->hbsm_io_success++;
#ifdef MPL_DBL_FREE	
	BUG_ON(stub->cmd.host_scribble == (void*)0x8585 );
#endif
	stub->cmdp->resid = stub->cmd.resid;
	stub->cmdp->host_scribble = NULL;
	memcpy(stub->cmdp->sense_buffer, stub->cmd.sense_buffer,
		SCSI_SENSE_BUFFERSIZE);

	mpl_module_unlock();
	if (mpl_handle_qfull){
		if ((stub->cmd.result == SAM_STAT_TASK_SET_FULL) ||
		   (stub->cmd.result == SAM_STAT_BUSY)) {
			mpl_throttle_down_queue_full(stub);
		} else {
			mpl_throttle_up_queue_full(stub);
		}
	}

	if (!(stub->flag & MPL_SCSI_CMD_STUB)) {
		mpl_print(0, MPL_ERR "Invalid scsi done call"
	  			"%s %s, cmd = %02x %02x %02x %02x\n",
      			mpl_print_device(stub->path->device),
      			mpl_print_path(stub->path),
      			stub->cmd.cmnd[0], stub->cmd.cmnd[1],
      			stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
	}
	else {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
		/* To handle data frame loss */
		if ((stub->cmdp->result == ScsiResult( DID_ERROR, 0)) ||
			((status_byte(stub->cmdp->result) == CHECK_CONDITION)
				&& mpl_is_cmd_aborted( stub->cmdp ))) {
			if (!blk_noretry_request(stub->cmdp->request)) {
				if (stub->cmdp->retries > 20 ) {
				 mpl_print(MPL_DEBUG, MPL_ERR 
				 "Retry count 20"
				  "cmd x%x cmdptr %p path %s device %s result x%x \n",
				  stub->cmdp->cmnd[0],stub->cmdp, mpl_print_path(stub->path),
				  mpl_print_device(stub->path->device), stub->cmdp->result ); 	
				  stub->cmdp->retries = 0 ;
				  stub->cmdp->jiffies_at_alloc = jiffies ;
					
				}
			} else 
			{ 
				mpl_print(0, MPL_ERR "NO retry request set IGNORE IO error\n"
	  			"%s %s, cmd = %02x %02x %02x %02x\n",
      				mpl_print_device(stub->path->device),
      				mpl_print_path(stub->path),
      				stub->cmd.cmnd[0], stub->cmd.cmnd[1],
      				stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
			}
		}
#else
		if (stub->cmdp->result == ScsiResult( DID_ERROR, 0)) {
			if (!blk_noretry_request(stub->cmdp->request)) {
				if (stub->cmdp->retries > 20 ) {
				 mpl_print(MPL_DEBUG, MPL_ERR 
				 "Retry count 20\n"); 	
				  stub->cmdp->retries = 0 ;
				}
			} else
				mpl_print(0, MPL_ERR "NO retry request set IGNORE IO error"
	  			"%s %s, cmd = %02x %02x %02x %02x\n",
      				mpl_print_device(stub->path->device),
      				mpl_print_path(stub->path),
      				stub->cmd.cmnd[0], stub->cmd.cmnd[1],
      				stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
		
		}

		
#endif
		
		
		if (timer_pending(&stub->cmd.eh_timeout)) {
			printk(KERN_ERR "##Free whilepending timer cmd_p x%p stub x%p"
			"scsi x%p \n", &stub->cmd,stub,stub->cmdp);
			dump_stack();
			del_timer(&stub->cmd.eh_timeout);
		}
		if (!timer_pending(&stub->cmdp->eh_timeout)) {
			printk(KERN_ERR  "##mpl_call:Midlaye timer fire  cmd_p x%p stub x%p"
			"scsi x%p reason x%x  \n", &stub->cmd,stub,stub->cmdp,stub->reason);
			dump_stack();
		}	
		stub->done(stub->cmdp);
	}

	mpl_module_lock();
	mpl_stub_free(stub);
}

void
mpl_add_pending_ios(struct io_stub *stub, struct mpl_device *device)
{
	unsigned long flags;

	/* Double check to ensure we are in the right state */
	if (((device->state == DEV_FAILED) && !timer_pending(&device->nopath)) ||
		(device->state == DEV_DISABLED)) {
		if (stub->done && (stub->flag & MPL_SCSI_CMD_STUB)) {
			stub->cmd.result = ScsiResult(DID_NO_CONNECT, 0);
			mpl_call_done_free_stub(stub);
		} else {
			mpl_print(0, MPL_ERR 
			"Try to add Internal cmd to PendingIOs %s\n",
			mpl_print_path(stub->path));
			mpl_stub_free(stub);
		}
		return;
	}
#ifdef MPL_STUB_SANITY
	if ((stub->flag & MPL_INTERNAL_CMD_STUB) ||
		(!(stub->cmdp))) {
		mpl_print(0, MPL_ERR "Bad stub in pending_ios:4: %p from  %p, "
	  			"%s %s, cmd = %02x %02x %02x %02x\n",
      			stub, (&stub)[-1],
      			mpl_print_device(stub->path->device),
      			mpl_print_path(stub->path),
      			stub->cmd.cmnd[0], stub->cmd.cmnd[1],
      			stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
	}
#endif
	/* Return command to midlayer if scsi_cmnd timer expires or null */
	if (stub->cmdp->eh_timeout.function != NULL && 
		del_timer(&stub->cmdp->eh_timeout)) {

		mod_timer(&stub->cmdp->eh_timeout, jiffies + 180*HZ);
		if ( timer_pending(&device->nopath))
			mod_timer(&stub->cmd.eh_timeout,
				 jiffies + MPL_CMD_TMO *HZ + device->nopath_tmo *HZ);
		else
			mod_timer(&stub->cmd.eh_timeout, jiffies + MPL_CMD_TMO *HZ);
		spin_lock_irqsave(&stub_queue_lock, flags);
		mpl_add_stub_to_pendq(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	} 
	else {
		mpl_call_done_free_stub(stub);
	}
	return;
}

static void
mpl_retry_pending_ios(struct io_stub *stub)
{
	struct mpl_device *device = stub->path->device;
	struct mpl_path *path = device->last_path;
	struct scsi_cmnd  *cmd = &stub->cmd;
	unsigned long flags ;

	memset(&stub->cmd.sense_buffer[0], 0, SCSI_SENSE_BUFFERSIZE);

        if (cmd->eh_timeout.function != NULL && !del_timer(&cmd->eh_timeout)) {
		/* Add command  to int_scsi_tmo_queue  to handle by timer routine */
		BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
		stub->has_timer = 1 ; 
		stub->reason = MPL_DO_INT_SCSI_TMO ;
		spin_lock_irqsave(&stub_queue_lock, flags);
		mpl_add_stub_to_scsitmo_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		/* Let timer function return command */
		mpl_module_unlock();
		schedule_timeout(HZ);
		mpl_module_lock();
		return ;
	}
	if (((device->state == DEV_FAILED) && !timer_pending(&device->nopath)) ||
		(device->state == DEV_DISABLED)) {
		if (stub->done && (stub->flag & MPL_SCSI_CMD_STUB)) {
			stub->cmd.result = ScsiResult(DID_NO_CONNECT, 0);
			mpl_call_done_free_stub(stub);
		} else {
			mpl_stub_free(stub);
		}
	} else if (path->host->blocked || path->lld_blocked ||
		   timer_pending(&device->nopath) ||
		   mpl_queuecommand_alternate_path(device, path, stub)) {
		mpl_add_pending_ios(stub, device);
	}
	return;
}

static int is_inq83_valid(struct scsi_cmnd  *cmd)
{
	uint8_t *data = cmd->request_buffer;
	int response_length;
	uint32_t size;

	response_length = cmd->request_bufflen - cmd->resid;
	size = data[7];

	return (cmd->result == 0 &&
		(data[0] >> 5) == 0 &&          /* peripheral qualifier */
		response_length > 8 &&          /* truncation */
		response_length >= (size + 8));
}

static int
mpl_compare_serial_number(struct mpl_device *device, uint8_t *serial, int size)
{
	return device->serial_size != size || memcmp(device->serial_number,
						     serial, size);
}

int
mpl_setup_inq_lun_skip(struct scsi_cmnd *cmd)
{
	uint8_t *sns = cmd->sense_buffer;
	uint8_t *request;
	int len = 0;

	request = mpl_find_cmnd_data(cmd, &len);
	if ((request == NULL) || (len < 36)) {
		mpl_print(0, MPL_ERR "No data in lun skip: len %d\n",
      			len);
		return 0;
	}
	if ((cmd->cmnd[0] == INQUIRY) && (cmd->cmnd[2] == 0)) {
		/* INQ Page 0 from midlayer scan */
		if (!(*request & 0x1f)) {
			memset(request, 0, len); 
			*request++ = TYPE_PROCESSOR; /* Processor type device */
#if 0
			if (cmd->device->lun == 0)
				*request++ = 0x3;   /* Processor type device */
			else {
				if (mpl_pfq_use_lun_notsup)
					*request++ = 0x60; /* Lun not supported (pqb = 3) */
				else
					*request++ = 0x3f; /* Lun supported not available (pqb = 1) */
			}
#endif
			*request++ = 0x0;
			*request++ = 0x3;
			*request++ = 0x12;
			*request++ = 0x1f;  /* 2.6 midlayer expects at least
					      36 byte inquiry response */
			*request++ = 0x0;
			*request++ = 0x0;
			*request++ = 0x2;
			memset(sns, 0, SCSI_SENSE_BUFFERSIZE);
			cmd->result = ScsiResult(DID_OK, 0);
		}
	}
	else {
		cmd->result = ScsiResult(DID_NO_CONNECT, 0);
	}
	mpl_release_cmnd_data(cmd);
	return cmd->result;
}

int 
mpl_no_alternate_path (struct mpl_device *device, struct mpl_path *path)
{
	struct mpl_path *p;
	
	list_for_each_entry(p, &device->paths, path_list) {
		if (device->alua_support && mpl_auto_sas_opt &&
		   (p->sas != MPL_ALUA_ACTIVE_OPT))
			continue;
		if(p != path){
			return 0;
		}
	}
	return 1;
}

struct mpl_path *
mpl_locate_min_primary(struct mpl_path *path, int *ocnt)
{
	struct mpl_device *device = path->device;
	struct mpl_path   *minpath = NULL;
	struct mpl_path   *p;
	int cnt;
	
	list_for_each_entry(p, &device->paths, path_list) {
		if (device->alua_support && mpl_auto_sas_opt &&
		   (p->sas != MPL_ALUA_ACTIVE_OPT))
			continue;
		if(p == path)
			continue;
		cnt = atomic_read(&p->tgt->prim_path_ref_count);
		if(cnt < *ocnt) {
			*ocnt = cnt;
			minpath = p;
		}
	}
	return minpath;
}

static int
mpl_auto_primary_select(struct mpl_device *device, struct mpl_path *path)
{
	int use_path = 0;
	int cnt, ocnt, pcnt;
	struct mpl_path *prim_path;
	struct mpl_path *opath = NULL;

	if(device->active_passive) {
		/* active - passive only uses active controllers. */
		if (path->active_controller) {
			use_path = 1;
		}
	}
	else {
		/* active - active check to use optimized controllers.
		 */
		if (device->alua_support && mpl_auto_sas_opt &&
		   (path->sas != MPL_ALUA_ACTIVE_OPT)) {
			use_path = 0;
		}
		else {
			use_path = 1;
		}
	}

	if(!use_path) {
		/* if this is the only path available, use it anyway */
		use_path = mpl_no_alternate_path(device, path);
		if(!use_path)
			return 0;
	}

	/* Do not load balance devices that have a statically
	 * configured primary path.
	 */
	if (device->StaticConfigPrimary) {
		if (path->StaticConfig)
			return 0;
		if((prim_path = mpl_get_primary_path(device)))
			if (prim_path->StaticConfig)
				return 0;
	}

	/* At this point, we've established the path is usable for
	 * primary path assignment. Next we need to check out how
	 * we will load balance this assignment across all devices paths.
	 */

	/* Is primary path already assigned */
	if((prim_path = mpl_get_primary_path(device))) {
		/* If the primary path is not optimized, switch if necessary */
		if (device->alua_support && mpl_auto_sas_opt &&
		    (path->sas == MPL_ALUA_ACTIVE_OPT) &&
		    (prim_path->sas != MPL_ALUA_ACTIVE_OPT)) {
			mpl_revoke_other_primaries(path);
			goto primit;
		}

		if(mpl_auto_lun_balance) {
			pcnt = atomic_read(&prim_path->tgt->prim_path_ref_count);
			cnt = atomic_read(&path->tgt->prim_path_ref_count);
			ocnt = cnt;
			/* Check all paths on this device for the 
			 * mnimun target primary path count.
			 */
			opath = mpl_locate_min_primary(path, &ocnt);
			if (opath) {
				mpl_revoke_other_primaries(path);
				path = opath;
				goto primit;
			}
			else {
				/* Balance primary path across targets */
				if ((pcnt > 0) && ((pcnt - 1) >  cnt)) {
					mpl_revoke_other_primaries(path);
					goto primit;
				}
			}
		}
		return 0;
	}


primit:
	if (use_path) {
		path->is_default = 1;
		atomic_inc(&path->tgt->prim_path_ref_count);

		mpl_print(MPL_INIT, MPL_INFO " Set Primary %s for %s: tgt cnt %d\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  atomic_read(&path->tgt->prim_path_ref_count));
		if (opath)
			mpl_path_state_machine(opath, PATH_UP);	
		
	}
	return use_path;
}

static int
mpl_export_list(struct mpl_path *path)
{
	/* Check if device is on the export List
	 * return (1) true if it is.
	 */
	if (path->configured == CONFIG_UNAVAILABLE) {
		mpl_print(MPL_INIT, MPL_INFO " %s UNAVAILABLE %s \n",
			  mpl_print_path(path),
			  mpl_print_device(path->device));
		return 1;
	}
	return 0;
}

static int
mpl_evaluate_config_path_control(struct mpl_path *path)
{
	struct mpl_device *device;
	struct config_path *cpath;
	int device_flags;
	int path_flags;
	int path_cntl;


	if (!path->tgt || !path->host) {
		mpl_print(0, MPL_ERR "Missing host or target: %p, %p\n",
			  path->tgt, path->host);
		return 1;
	}
	
	if (path->configured == CONFIG_UNAVAILABLE) {
		return 1;
	}

	device_flags = 0;
	path_flags = 0;
	device = path->device;
	cpath = mpl_get_device_flags_by_sn(&device_flags, device->serial_number,
							device->serial_size);
	if (!device_flags)
		return 1;

	if (cpath) {
		if (!memcmp(cpath->hba_wwpn, path->host->wwpn,
					sizeof(cpath->hba_wwpn)) &&
		    !memcmp(cpath->target_wwpn, path->tgt->wwpn,
					sizeof(cpath->target_wwpn)) &&
		    cpath->lun_no == path->lun) {
			path_flags = cpath->flags;
		}
	}

	mpl_set_device_configured(device, CONFIG_CONFIGURED);
	mpl_set_path_configured(path,CONFIG_CONFIGURED);

	path_cntl = device_flags & 0x0f;
	switch (path_cntl) {
	case MPL_PC_TYPE_ROUND_ROBIN:
		path->traffic_ratio = 1;
				/* Intentional fall through */
	case MPL_PC_TYPE_DYNAMIC_LUNQ:
	case MPL_PC_TYPE_DYNAMIC_TGTQ:
	case MPL_PC_TYPE_DYNAMIC_HBAQ:
	case MPL_PC_TYPE_DYNAMIC_TIME:
		device->path_cntl_type = path_cntl;
		break;
	case MPL_PC_TYPE_FAILOVER:
		device->path_cntl_type = path_cntl;
		device->auto_failback = (device_flags & 
					 MPL_PC_CFG_AUTO_FAILBACK) != 0;
		if (path_flags & 1) {
			cpath->found = 1;
			mpl_revoke_other_primaries(path);
			mpl_set_path_static_config(path);
			path->device->StaticConfigPrimary = 1;
			path->is_default = 1;
			atomic_inc(&path->tgt->prim_path_ref_count);
		}
		break;
	default:
		mpl_print(0, MPL_ERR "Bad path control type (%#x) for %s\n",
			  path_cntl, mpl_print_device(device));
		break;
	}
	mpl_set_device_static_config(device);
	return 0;
}

static void
mpl_evaluate_config_exclusion(struct mpl_path *path)
{
	struct config_path *cpath;
	int path_flags;

	if (!path->tgt || !path->host) {
		mpl_print(0, MPL_ERR "Missing host or target: %p, %p\n",
			  path->tgt, path->host);
		return;
	}
	
	cpath = mpl_get_path_flags(&path_flags, path->host->wwpn,
			       path->tgt->wwpn, path->lun);
	if (!cpath)
		return;

	if (path_flags & MPL_CFG_PATH_EXPORT) {
		cpath->path = path;
		mpl_set_path_configured(path,CONFIG_UNAVAILABLE);
		mpl_set_path_static_config(path);
	}
	return;
}

static int
mpl_hbeat_inquirysn(struct io_stub *stub)
{
	struct scsi_cmnd  *cmd = &stub->cmd;
	struct mpl_path   *path = stub->path;
	struct mpl_device *device = path->device;
	uint8_t *request = cmd->request_buffer;
	int  rv = 0, serial_size = 0;
	uint8_t serialbuff[SCSI_INQ_BUFF_LEN];

	if (driver_unloading || path->state == PATH_DISABLED) {
		goto inqsn_exit;
	}

	if (cmd->result != 0) {
				/* cmd failed */
		mpl_print(MPL_RECOVERY, MPL_WARNING "InqSN for %s %s failed "
			  "but this should have been processed earlier"
			  " %x cnt %x: %02x/%02x/%02x\n", 
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  cmd->result, atomic_read(&total_internal_count),
			  cmd->sense_buffer[2], cmd->sense_buffer[12],
			  cmd->sense_buffer[13]);
		rv = -1;
		goto inqsn_exit;
	}

	if (!is_inq83_valid(cmd)) {
				/* ignore inq 83 response that makes
				 * no sense to us. */
		mpl_print(MPL_HEARTBEAT, MPL_WARNING
			  "Invalid InqSN response for %s %s, ignoring.\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path));
		goto inqsn_exit;
	}

	if (device->plugin &&
	    device->plugin->inq_validation) {
		if (device->plugin->inq_validation(path, request, serialbuff,
		    &serial_size)){
			mpl_print(MPL_HEARTBEAT, MPL_WARNING
			  "Invalid InqSN Data for %s %s, ignoring.\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path));
			goto inqsn_exit;
		}
	}
	else {
		if(mpl_get_validation_id(request, serialbuff, &serial_size)){
			mpl_print(MPL_HEARTBEAT, MPL_WARNING
			  "Invalid InqSN Data for %s %s, ignoring.\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path));
			goto inqsn_exit;
		} 
	} 

	if(!device->serial_size) {
		struct mpl_device *d;
				/* If we already have this device
				 * defined, we need to redirect local
				 * pointers to the older device and
				 * prepare to drop the current
				 * device.  */

		/* Save necessary info from inq response */
		device->dev_need_inqsn_data = 0;
		device->serial_size = serial_size;
		memcpy(device->serial_number, serialbuff, serial_size);

		if (mpl_export_list(path)) {
			mpl_set_path_configured(path, CONFIG_UNAVAILABLE);
		}
		else if (path->delayed_cmd && !device->scsi_type) {
			if (mpl_evaluate_config_path_control(path)) {
				mpl_set_device_configured(device, CONFIG_CONFIGURED);
				mpl_set_path_configured(path,CONFIG_CONFIGURED);
				mpl_set_dev_ctrl_header(device, mpl_auto_configure);

				/*  set auto failback by default for auto configured
				 * devices. 
				 */
				device->auto_failback = 1;
			}	
		}	

		d = mpl_match_device_by_serial(device, serialbuff, serial_size);
 		mpl_print(MPL_RECOVERY, MPL_WARNING "InqSN data needed for "
			"%s %s (%d) type %d config %d/%d\n", 
			mpl_print_device(device),
			mpl_print_path(path),
			(d != device),
		    	device->scsi_type, device->configured, path->configured);

		if (path->delayed_cmd && (del_timer(&path->delayed_cmd->cmd.eh_timeout))){
			if ((!device->scsi_type) &&
			    (path->configured == CONFIG_CONFIGURED)) {
				/* Change primary path, if necessary */
				mpl_auto_primary_select(d, path);
			}

			/* This is only if mpl_match_device_by_serial moved the path
			 * to a different device, thus its not the presentation path.
			 * The first path to a device will always be the presentation path.
			 */
			if ((path->device != device) &&
			    (path->configured == CONFIG_CONFIGURED)) {
				mpl_module_unlock();
				mpl_setup_inq_lun_skip(&path->delayed_cmd->cmd);
				mpl_module_lock();
			}
			mpl_call_done_free_stub(path->delayed_cmd);
			path->delayed_cmd = NULL;
		}
		else {
			mpl_print(MPL_TMO, MPL_INFO "Delayed cmd timeout cmd %p scsi_cmd %p stub %p"
				" path %s\n",
				&path->delayed_cmd->cmd, &path->delayed_cmd->cmdp, &path->delayed_cmd,
				mpl_print_path(path));
		}
	} else if (mpl_compare_serial_number(device, serialbuff, serial_size) != 0) {
		mpl_print(MPL_RECOVERY, MPL_WARNING "InqSN data for %s %s: "
			  "did not match\n", 
			  mpl_print_device(device),
			  mpl_print_path(path));
		rv = -1;
	} else {
		mpl_print(MPL_RECOVERY, MPL_INFO "InqSN Heartbeat success "
			  "for %s %s\n",
			  mpl_print_device(device),
			  mpl_print_path(path));

				/* inq sn data matched, but if path is
				 * ONLINE or DISABLED, do nothing */
	}


inqsn_exit:

	if (path->delayed_cmd && (del_timer(&path->delayed_cmd->cmd.eh_timeout))) {
		mpl_call_done_free_stub(path->delayed_cmd);
		path->delayed_cmd = NULL;
	}
	return rv;
}

static void
mpl_flush_hbeat(struct mpl_path *path)
{
	struct io_stub *stub, *tmp;
	unsigned long flags;

top1:
	/* Make sure any remenants of Hbeat is flushed */
	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry_safe(stub, tmp, &deferral_queue, stub_queue) {
		if (stub && stub->path == path) {
			if ((stub->reason == MPL_HB_CMD) || 
				(stub->reason == MPL_HEARTBEAT_CMD)) {
				mpl_remove_stub_from_deferral(stub);
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				mpl_stub_free(stub);
                                goto top1;
			}
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
top2:
	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry_safe(stub, tmp, &eh_deferral_queue, stub_queue) {
		if (stub && stub->path == path) {
			if (stub->reason == MPL_DO_HBEAT_TMO) {
				mpl_remove_stub_from_eh_deferral(stub);
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				mpl_stub_free(stub);
                                goto top2;
			}
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
top3:
	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry_safe(stub, tmp, &path->host->hbeat_deferral_queue, stub_queue) {
		if (stub && stub->path == path) {
			mpl_remove_stub_from_hbeat_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_stub_free(stub);
                        goto top3;
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
}

/* default to 30 second heart beat */
static void
mpl_hbeat_timeout(unsigned long data)
{
	struct mpl_path *path = (struct mpl_path *)data;

	mpl_module_lock();
	/* Don't send heart beat and don't restart timer if the path has been deconfigured.
	 * For initial Inquiry scan, let unconfigured paths thru.
	 */
	if ((path->state == PATH_DISABLED) ||
	(!mpl_is_path_configured(path) && !path->device->dev_need_inqsn_data)) {
		path->hbstate = HB_STOPPED;
		mpl_flush_hbeat(path);
	}
	else {
		path->flag &= ~PATH_FAST_POLL;
		mpl_hbeat_state_machine(path, HB_TIMEOUT);
	}
	mpl_module_unlock();
	return;
}

static void
mpl_nopath_timeout(unsigned long data)
{
	struct mpl_device *device = (struct mpl_device *)data;

	mpl_module_lock();

	mpl_print(MPL_IOPATH, MPL_ERR "NoPath timeout for %s: State %s\n",
	  mpl_print_device(device), mpl_dstates[device->state]);

	if (device->state == DEV_FAILED) {
		mpl_device_process_pendq(device);
	}
	mpl_module_unlock();
	return;
}

int
mpl_quiesce_device(struct mpl_device *device, int tmo)
{
	struct mpl_path *path;
	int  n, cnt;

	cnt = 0;
	do {
		n = 0;
		list_for_each_entry(path, &device->paths, path_list) {
			n += atomic_read(&path->io_count);
			n += atomic_read(&path->internal_count);
		}
		if (n) {
			mpl_module_unlock();
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(2);
			mpl_module_lock();
			cnt += 2;
		}
		if (tmo && (cnt > tmo))
			break;
	} while (n);
	return n;
}

#define configure_transition(from, to) ((from << 3) | to)

int
mpl_set_path_configured(struct mpl_path *path,
			enum mpl_config_state configured)
{
	int rc = 0;

	switch (configure_transition(path->configured, configured)) {
	case configure_transition(CONFIG_UNCONFIGURED, CONFIG_UNCONFIGURED):
	case configure_transition(CONFIG_CONFIGURED,   CONFIG_CONFIGURED):
	case configure_transition(CONFIG_UNAVAILABLE,  CONFIG_UNAVAILABLE):
				/* Do nothing. */
		break;

	case configure_transition(CONFIG_UNCONFIGURED, CONFIG_UNAVAILABLE):
		if(path->is_default) {
			atomic_dec(&path->tgt->prim_path_ref_count);
			path->is_default = 0;
		}
		path->configured = configured;
		break;
	case configure_transition(CONFIG_UNCONFIGURED, CONFIG_CONFIGURED):
	case configure_transition(CONFIG_UNAVAILABLE,  CONFIG_CONFIGURED):
	case configure_transition(CONFIG_UNAVAILABLE,  CONFIG_UNCONFIGURED):
		path->configured = configured;
		if(path->is_default) {
			mpl_revoke_other_primaries(path);
                }
		break;

	case configure_transition(CONFIG_CONFIGURED, CONFIG_UNAVAILABLE):
		rc = -EBUSY;
		break;

	case configure_transition(CONFIG_CONFIGURED, CONFIG_UNCONFIGURED):
		if (mpl_is_device_configured(path->device) &&
		    mpl_is_path_primary(path)) {
			rc = -EBUSY;
			break;
		}
		mpl_path_state_machine(path, PATH_DISABLE);
		path->configured = configured;
		break;

	default:
		mpl_print(0, MPL_ERR "Invalid path configuration transition: "
			  "%#x to %#x, %s %s\n",
			  path->configured,
			  configured,
			  mpl_print_device(path->device),
			  mpl_print_path(path));
		break;
	}
	return rc;
}

void
mpl_internal_done(struct scsi_cmnd *cmd)
{
	struct io_stub *stub;
	struct mpl_path *path;
	struct mpl_device *device;
	unsigned long flags;

        /* If timer fired, this command will be handled by timeout handler */
        if (!del_timer(&cmd->eh_timeout)) {
                return;
        }

	/* io_request_lock is held when called by lpfc driver */
	mpl_module_lock();

	spin_lock_irqsave(&stub_queue_lock, flags);
	/* Map cmnd to corresponding stub */
	stub = mpl_locate_stub(cmd);
	stub = mpl_find_stub_in_lpfcq(stub);
	if (stub) {
		mpl_remove_stub_from_lpfcq(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	}
	else {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_print(0, MPL_ERR "Internal cmpl: NO stub\n");
		mpl_module_unlock();
		return;
	}

	path = stub->path;
	device = path->device;
	if ((stub->flag & MPL_ABORT_FAILED) != 0) {
		mpl_print(0, MPL_ERR "Internal cmd 0x%x completed after "
			  "abort failed for %s %s, result = %08x\n",
			  cmd->cmnd[0],
			  mpl_print_device(path->device),
			  mpl_print_path(path), cmd->result);
		mpl_stub_free(stub);
		mpl_module_unlock();
		return;
	}
	if (!(stub->flag & MPL_INTERNAL_CMD_STUB)) {
		mpl_print(0, MPL_ERR "Internal cmpl: Bad stub %p: %d %s\n",
		stub, stub->flag, mpl_print_path(path));
		mpl_stub_free(stub);
		mpl_module_unlock();
		return;
	}

	if ((cmd->cmnd[0] == REPORT_TGT_PORT_GROUP) ||
	    (cmd->cmnd[0] == SET_TGT_PORT_GROUP)) {
		device->tpg_cmd_inp = 0;
	}

	if (driver_unloading) {
		mpl_stub_free(stub);
		mpl_module_unlock();
		return;
	}

	mod_timer(&cmd->eh_timeout, jiffies + MPL_CMD_TMO *HZ);
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub->reason = MPL_INT_CMD;
	mpl_add_stub_to_deferral(stub);
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	mpl_module_unlock();
	mpl_wake_thread();
	return;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
int
mpl_is_cmd_aborted( struct scsi_cmnd *cmd)
{
	int sensekey ; 

	if ( (cmd->sense_buffer[0] & 0x7f)  >= 0x72  ) {
		sensekey = cmd->sense_buffer[1] & 0xf ;
	}
	 else
	{
		sensekey = cmd->sense_buffer[2] & 0xf ;
	}	 
	if ( sensekey == ABORTED_COMMAND )
		return  1 ;
	else 
		return  0 ;
}
#endif

void
mpl_done(struct scsi_cmnd *cmd)
{
	struct io_stub *stub;
	struct mpl_path *path;
	struct mpl_device *device;
	uint32_t cnt;
	uint32_t outio = 1;
	unsigned long tm, avg;
	unsigned long flags;

	/* If internal scsi timer fired, this command will be handled by timeout handler */
        if (cmd->eh_timeout.function != NULL && !del_timer(&cmd->eh_timeout)) {
		return;
	}
	/* io_request_lock is held when called by lpfc driver */
	mpl_module_lock();

	spin_lock_irqsave(&stub_queue_lock, flags);
	/* Map cmnd to corresponding stub */
	stub = mpl_locate_stub(cmd);
	stub = mpl_find_stub_in_lpfcq(stub);
	if (stub) {
		mpl_remove_stub_from_lpfcq(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	}
	else {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_print(0, MPL_ERR "SCSI cmd cmpl: NO stub\n");
		mpl_module_unlock();
		return;
	}

	path = stub->path;
	device = path->device;
	if (!(stub->flag & MPL_SCSI_CMD_STUB)) {
		mpl_print(0, MPL_ERR "SCSI cmd cmpl: Bad stub %p: %d %s\n",
		stub, stub->flag, mpl_print_path(path));
		mpl_stub_free(stub);
		mpl_module_unlock();
		return;
	}

	if (atomic_read(&path->io_count) == 1) {
		/* If there are no outstanding IOs on this path,
		 * zero the avg time.
		 */
		atomic_set(&path->avg_cnt, 0);
		path->avg_time = 0;
		outio = 0; /* outstanding IOs? */
	}

	if (stub->reason == MPL_DO_ABORT) {
		mpl_module_unlock();
		return;
	}


	if ((cmd->cmnd[0] == INQUIRY) && (cmd->cmnd[2] == 0) && !path->seen_ext_inq) {
		/* INQ Page 0 from midlayer scan */
		/* Added timer because this command is going to take longer timer */
		/* Command has not time out we are going to add into deferral queue */
		if (stub->cmdp->eh_timeout.function != NULL && 
			del_timer(&stub->cmdp->eh_timeout)) {

			mod_timer(&stub->cmdp->eh_timeout, jiffies + 300*HZ);
			mod_timer(&cmd->eh_timeout, jiffies + MPL_FIRST_INQ_TMO *HZ);
			path->seen_ext_inq = 1;
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub->reason = MPL_INQ_CMD;
			mpl_add_stub_to_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_module_unlock();
			mpl_wake_thread();
		}
		else {
			mpl_call_done_free_stub(stub);
			mpl_module_unlock();
		}
		return;
	} else if (cmd->result) {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
		if (cmd->result == ScsiResult(DID_BUS_BUSY, 0)) {
			/* This error condition indicates the fc_transport
			 * is getting is sync with the base driver.
			 * Change this to an always retryable error.
			 */
			if (mpl_get_ndlp(cmd->device->hostdata)) {
				mpl_print(0, MPL_ERR "MPdone %s DID_BUS_BUSY with ndlp: cmd 0x%x\n",
					mpl_print_path(path), cmd->cmnd[0]);
			}
  			cmd->result = ScsiResult(DID_IMM_RETRY, 0);
		}
#endif
		if (cmd->result == SAM_STAT_TASK_SET_FULL) {
			/* Lets pass queue full errors up to
			 * the SCSI layer immediately.
			 */
			mpl_call_done_free_stub(stub);
			mpl_module_unlock();
			return;
		}
		if (path->state == PATH_DISABLED) {
			stub->reason = MPL_RETRY_CMD;
			/* Added timer because this command is going to take longer timer */
			/* Command has not time out we are going to add into deferral queue */
			if (stub->cmdp->eh_timeout.function != NULL && 
				del_timer(&stub->cmdp->eh_timeout)) {

				mod_timer(&stub->cmdp->eh_timeout, jiffies + 180*HZ);
				mod_timer(&cmd->eh_timeout, jiffies + MPL_CMD_TMO *HZ);
				spin_lock_irqsave(&stub_queue_lock, flags);
				mpl_add_stub_to_deferral(stub);
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				mpl_module_unlock();
				mpl_wake_thread();
			} else {
				/* AT this point mid layer scsi command already timeout */
				mpl_call_done_free_stub(stub);
				mpl_module_unlock();
			}
			return;
		}
		else if (mpl_is_path_configured(path) || (stub->cmd.cmnd[0] == TEST_UNIT_READY)) {
			/* Added timer because this command is going to take longer timer */
			/* Command has not time out we are going to add into deferral queue*/
			if (stub->cmdp->eh_timeout.function != NULL && 
				del_timer(&stub->cmdp->eh_timeout)) {

				mod_timer(&stub->cmdp->eh_timeout, jiffies + 180*HZ);
				mod_timer(&cmd->eh_timeout, jiffies + MPL_CMD_TMO *HZ);
				/* All TURs should go thru plugin diagnostic check */
				spin_lock_irqsave(&stub_queue_lock, flags);
				stub->reason = MPL_FAILED_CMD;
				mpl_add_stub_to_deferral(stub);
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				mpl_module_unlock();
				mpl_wake_thread();
			} else {
				/* AT this point mid layer scsi command already timeout */
				mpl_call_done_free_stub(stub);
				mpl_module_unlock();
			}
			return;
		}
	}

	/* Calculate average times for good IOs */
	if (outio) {
		cnt = atomic_read(&path->avg_cnt);
		tm = jiffies - stub->start_time;
		avg = ((path->avg_time * cnt) + tm) / (cnt + 1);
		atomic_inc(&path->avg_cnt);
		path->avg_time = avg;
	}

	mpl_call_done_free_stub(stub);
	mpl_module_unlock();
	return;
}

struct mpl_path *
mpl_round_robin_path(struct mpl_device *device, struct scsi_cmnd *cmd)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;

	/* For now just implement round robin, 1 IO at a time */
	path = device->last_path;
	first_path = path;
	if((path = mpl_get_next_avail_path(path, first_path))) {
		device->last_path = path;
	}
	return device->last_path;
}

struct mpl_path *
mpl_load_balance_time(struct mpl_device *device, struct scsi_cmnd *cmd)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;
	struct mpl_path   *low_path;
	unsigned long      tm, path_tm;

	low_path = NULL;
	tm = 0xffffffff;

	/* Load balance based on the lowest path average IO time */

	/* The LUN paths are from a specific HBA Port WWPN to a specific
	 * Controller Port WWPN LUN lun_id.
	 */
	path = device->last_path;
	first_path = path;
	while((path = mpl_get_next_avail_path(path, first_path))) {
		path_tm =  path->avg_time;
		if (path_tm < tm) {
			low_path = path;
			tm = path_tm;
		}
	}
	if(low_path) {
		device->last_path = low_path;
	}
	return device->last_path;
}

struct mpl_path *
mpl_load_balance_lun(struct mpl_device *device, struct scsi_cmnd *cmd,
			struct mpl_path *xmitp)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;
	struct mpl_path   *low_path;
	uint32_t           count, path_count;

	low_path = NULL;
	count = 0xffffffff;

	/* Load balance based on the LUN path outstanding IO */

	/* The LUN paths are from a specific HBA Port WWPN to a specific
	 * Controller Port WWPN LUN lun_id.
	 */
	path = device->last_path;
	first_path = path;
	while((path = mpl_get_next_avail_path(path, first_path))) {
		path_count =  atomic_read(&path->io_count);
		if (path == xmitp)
			path_count--;
		if (path_count < count) {
			low_path = path;
			count = path_count;
		}
	}
	if(low_path) {
		device->last_path = low_path;
	}
	return device->last_path;
}

struct mpl_path *
mpl_load_balance_target(struct mpl_device *device, struct scsi_cmnd *cmd,
			struct mpl_path *xmitp)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;
	struct mpl_path   *low_path;
	uint32_t           count, target_count;

	low_path = NULL;
	count = 0xffffffff;

	/* Load balance based on the target path outstanding IO */

	/* The target paths are all paths from a specific HBA Port WWPN
	 * to a specific Controller Port WWPN.
	 */
	path = device->last_path;
	first_path = path;
	while((path = mpl_get_next_avail_path(path, first_path))) {
		target_count =  atomic_read(&path->tgt->io_count);
		if (path->tgt == xmitp->tgt)
			target_count--;
		if (target_count < count) {
			low_path = path;
			count = target_count;
		}
	}
	if(low_path) {
		device->last_path = low_path;
	}
	return device->last_path;
}

struct mpl_path *
mpl_load_balance_hba(struct mpl_device *device, struct scsi_cmnd *cmd,
			struct mpl_path *xmitp)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;
	struct mpl_path   *low_path;
	uint32_t           count, hba_count;

	low_path = NULL;
	count = 0xffffffff;

	/* Load balance based on the hba path outstanding IO */

	/* The hba paths are all paths from a specific HBA Port WWPN.
	 */
	path = device->last_path;
	first_path = path;
	while((path = mpl_get_next_avail_path(path, first_path))) {
		hba_count =  atomic_read(&path->host->io_count);
		if (path->host == xmitp->host)
			hba_count--;
		if (hba_count < count) {
			low_path = path;
			count = hba_count;
		}
	}
	if(low_path) {
		device->last_path = low_path;
	}
	return device->last_path;
}

int
mpl_get_device_outio(struct mpl_device *device)
{
	struct mpl_path   *path;
	struct mpl_path   *first_path;
	uint32_t           count;

	count = 0;

	/* The hba paths are all paths from a specific HBA Port WWPN.
	 */
	path = device->last_path;
	first_path = path;
	do {
		count +=  atomic_read(&path->tgt->io_count);
		path = mpl_get_next_path(path);
	} while (path != first_path);

	return count;
}

void
mpl_inq_done(struct scsi_cmnd *cmd)
{
	struct io_stub *stub;
	unsigned long flags;
	
	/* Once command is timed out need to handle by 
	 * mpl_scsi_timesout function
 	 */
        if (!del_timer(&cmd->eh_timeout)) {
		return;
	}
	/* io_request_lock is held when called by lpfc driver */
	mpl_module_lock();

	spin_lock_irqsave(&stub_queue_lock, flags);
	/* Map cmnd to corresponding stub */
	stub = mpl_locate_stub(cmd);
	stub = mpl_find_stub_in_lpfcq(stub);
	if (stub) {
		mpl_remove_stub_from_lpfcq(stub);
	}
	else {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_print(0, MPL_ERR "SCSI INQ cmpl: NO stub\n");
		mpl_module_unlock();
		return;
	}

	/* Process "Out Of Scan" Inquiry command */
	stub->reason = MPL_OOS_INQ_CMD;
	/* 
	 * Since internal timer is always less than scsi timer    
	 * I donot have to check whether scsi_cmnd expires or not. 
  	 */
	if (stub->cmdp->eh_timeout.function != NULL && 
		del_timer(&stub->cmdp->eh_timeout)) {

		mod_timer(&stub->cmdp->eh_timeout, jiffies + 180*HZ);
		mod_timer(&stub->cmd.eh_timeout, jiffies + MPL_CMD_TMO *HZ);
		mpl_add_stub_to_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		mpl_wake_thread();
	} 
	else {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_call_done_free_stub(stub);
		mpl_module_unlock();
	}
	return;
}

static void
mpl_process_oos_inquiry(struct io_stub *stub)
{
	struct scsi_cmnd *cmd = &stub->cmd;
	unsigned long flags ;

        if (cmd->eh_timeout.function != NULL && !del_timer(&cmd->eh_timeout)) {
		
		/* Add command  to int_scsi_tmo_queue  to handle by timer routine */
		BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
		stub->has_timer = 1 ; 
		stub->reason = MPL_DO_INT_SCSI_TMO ;
		spin_lock_irqsave(&stub_queue_lock, flags);
		mpl_add_stub_to_scsitmo_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		/* Let timer function return command */
		mpl_module_unlock();
		schedule_timeout(HZ);
		mpl_module_lock();
		return ;
	}

	/* Process a successful "Out Of Scan" Inquiry command */
	if (mpl_is_inquiry_page_0(cmd) && !cmd->result) {
		mpl_module_unlock();
		mpl_setup_inq_lun_skip(cmd);
		mpl_module_lock();
	}
	mpl_call_done_free_stub(stub);
	return;
}

/* any non zero return code indicates error.  exact value not percolated,
 * but we differentiate anyway */
int
mpl_queuecommand(struct scsi_cmnd *cmd, void (*done)(struct scsi_cmnd *))
{
	struct mpl_host   *host;
	struct mpl_device *device;
	struct mpl_path   *path;
	struct io_stub    *stub;
	struct scsi_cmnd  *scsi_cmd;
	struct mpl_path *xmit_path;
	int  rv = DID_NO_CONNECT;
	unsigned long flags;

	scsi_cmd = cmd;
	mpl_module_lock();
	/* If the device is a dead device return DID_NO_CONNECT */
	if (!scsi_cmd->device->hostdata) {
		scsi_cmd->result = ScsiResult(rv, 0);
		goto out_error;
	}

	host  = mpl_find_host_by_scsi_host(mpl_scsi_host_from_cmd(cmd));
	if (!host || driver_unloading) {
		scsi_cmd->result = ScsiResult(rv, 0);
		goto out_error;
	}

	path = mpl_find_or_create_path(host, mpl_target_no_from_cmd(cmd),
				       mpl_lun_no_from_cmd(cmd));
	if (!path) {
		mpl_print(0, MPL_ERR "queuecommand failed to find path %d/%d/%d\n",
			  mpl_host_no_from_cmd(cmd),
			  mpl_target_no_from_cmd(cmd),
			  mpl_lun_no_from_cmd(cmd));
		scsi_cmd->result = ScsiResult(rv, 0);
		goto out_error;
	}

	if (path->sdev  && (path->sdev != cmd->device)) {
		mpl_print(MPL_IOPATH, MPL_ERR "Change SCSI device for %s %s: "
		  "cmd sdev = %d/%d/%d\n",
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  cmd->device->host->host_no,
		  cmd->device->id, 
		  cmd->device->lun);

		if (!cmd->device) {
			scsi_cmd->result = ScsiResult(rv, 0);
			goto out_error;
		}
	}

	path->sdev = cmd->device;

	/* If path is exported from MultiPulse, go directly to lpfc driver */
	if (path->configured == CONFIG_UNAVAILABLE) {
		mpl_module_unlock();
		/* Bypass MultiPulse totally */
		if (cmd->allowed != MPL_MAX_RETRIES ) 
			cmd->allowed = MPL_MAX_RETRIES ;
		if (!blk_noretry_request(cmd->request)) {
			if (cmd->retries == 20) {
				  cmd->retries = 0 ;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
				  cmd->jiffies_at_alloc = jiffies ;
#endif 
			}
		} else
			mpl_print(0, MPL_ERR "Exclude path NO retry flag set \n");

			
		rv = mpl_base_queuecommand(cmd, done);
		return rv;
	}

	device = path->device;
	if (device->plugin &&
	    device->plugin->pre_queue_filter &&
	    device->plugin->pre_queue_filter(path, cmd, done)) {
		mpl_module_unlock();
		return 0;
	}

	if ((!mpl_is_presentation_path(path)) &&
	    (cmd->cmnd[0] != REPORT_LUNS)) { /* Just let Report LUN go thru */

		mpl_print(MPL_IOPATH, MPL_INFO 
			"queuecommand rejected x%x x%x: non presentation I/O %d/%d/%d\n",
			  cmd->cmnd[0], cmd->cmnd[2],
			  mpl_host_no_from_cmd(cmd),
			  mpl_target_no_from_cmd(cmd),
			  mpl_lun_no_from_cmd(cmd));

		switch (cmd->cmnd[0]) {
		case TEST_UNIT_READY:
			if (!mpl_is_path_configured(path)) {
				mpl_module_unlock();
				/* Bypass MultiPulse totally */
				rv = mpl_base_queuecommand(cmd, done);
				return rv;
			}
			scsi_cmd->result = ScsiResult(rv, SAM_STAT_COMMAND_TERMINATED);
			goto out_error;
		case INQUIRY:
			stub = mpl_stub_alloc(cmd, path, done);
			if (!stub) {
				scsi_cmd->result = ScsiResult(DID_BUS_BUSY, 0);
				mpl_print(0, MPL_ERR "QCMD INQ no memory: cmd 0x%x\n",
					scsi_cmd->cmnd[0]);
				goto out_error;
			}
			stub->flag |= MPL_SCSI_CMD_STUB;
			stub->state = STUB_DQ;
			stub->cmdp->host_scribble = (unsigned char *)stub;

			/* From this point on, use the copy of cmd */
			cmd = &stub->cmd;
			stub->scsi_host = mpl_scsi_host_from_cmd(cmd);
			stub->target = mpl_target_no_from_cmd(cmd);
			stub->lun = mpl_lun_no_from_cmd(cmd);
			stub->path = path;
			spin_lock_irqsave(&stub_queue_lock, flags);
			mpl_add_stub_to_lpfcq(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_module_unlock();
			rv = mpl_base_queuecommand(cmd, mpl_inq_done);
			if (rv) {
				mpl_module_lock();
				spin_lock_irqsave(&stub_queue_lock, flags);
				mpl_remove_stub_from_lpfcq(stub);
				spin_unlock_irqrestore(&stub_queue_lock,flags);
				mpl_module_unlock();	
				del_timer_sync(&stub->cmd.eh_timeout );
				mpl_module_lock();
				mpl_stub_free(stub);
				mpl_module_unlock();	
			}
			return rv;
		default:
			scsi_cmd->result = ScsiResult(rv, 0);
			goto out_error;
		}
	}

	if ( mpl_enable_tgt_qdepth) {
		if (device->tgt_queue_depth && mpl_is_path_configured(path) &&
		    (mpl_get_device_outio(device) >= device->tgt_queue_depth)) {
			mpl_module_unlock();
			return SCSI_MLQUEUE_HOST_BUSY;
		}
	}

	/* Don't switch paths if seen_inq isn't set yet */
	if (mpl_is_path_configured(path) && path->seen_ext_inq)
		xmit_path = device->last_path;
	else
		xmit_path = path;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	if(scan_path == path) {
		int i = 0;

		/* Delay a bit, if needed, for lpfc ndlp to sync up
		 * with FC transport.
		 */
		while (!mpl_get_ndlp(cmd->device->hostdata)) {
			i++;
			if (i > 5) {
				break;
			}
			mpl_module_unlock();
			schedule_timeout(5);
			mpl_module_lock();
		}
		xmit_path = path;
	}
#endif
	stub = mpl_stub_alloc(cmd, xmit_path, done);
	if (!stub) {
		scsi_cmd->result = ScsiResult(DID_BUS_BUSY, 0);
		mpl_print(0, MPL_ERR "QCMD no memory: cmd 0x%x\n",
			scsi_cmd->cmnd[0]);
		goto out_error;
	}
	stub->flag |= MPL_SCSI_CMD_STUB;
	stub->state = STUB_DQ;
	stub->cmdp->host_scribble = (unsigned char *)stub;

	/* From this point on, use the copy of cmd */
	cmd = &stub->cmd;

	stub->scsi_host = mpl_scsi_host_from_cmd(cmd);
	stub->target = mpl_target_no_from_cmd(cmd);
	stub->lun = mpl_lun_no_from_cmd(cmd);

	if (path->seen_ext_inq) {

		if (device->state == DEV_FAILING || device->reset_pending ||
			(xmit_path->host->blocked || xmit_path->lld_blocked)) {
			if (mpl_is_path_configured(path)) {
				del_timer(&stub->cmd.eh_timeout) ;
				mpl_add_pending_ios(stub, device);
				mpl_module_unlock();
				return 0;
			} else {
				mpl_print(MPL_IOPATH, MPL_INFO "Allow command"
					"on path %s cmd x%x \n",
					mpl_print_path(path),stub->cmdp->cmnd[0]);
			} 
		}

		if (mpl_is_path_configured(path) &&
		    ((device->state == DEV_FAILED) ||
		     (device->state == DEV_DISABLED))) {

			if ((device->state == DEV_FAILED) &&
				timer_pending(&device->nopath)) {
				del_timer(&stub->cmd.eh_timeout);
				mpl_add_pending_ios(stub, device);
				mpl_module_unlock();
				return 0;
			}
			del_timer(&stub->cmd.eh_timeout );
			mpl_stub_free(stub);
			scsi_cmd->host_scribble = NULL;
			scsi_cmd->result = ScsiResult(rv, 0);
			goto out_error;
		}

		switch(device->path_cntl_type) {
		case 0: /* default */
		case MPL_PC_TYPE_FAILOVER:
			break;
		case MPL_PC_TYPE_ROUND_ROBIN:
			xmit_path = mpl_round_robin_path(device, cmd);
			mpl_stub_switch_path(stub, xmit_path);
			break;
		case MPL_PC_TYPE_DYNAMIC_LUNQ:
			xmit_path = mpl_load_balance_lun(device, cmd, xmit_path);
			mpl_stub_switch_path(stub, xmit_path);
			break;
		case MPL_PC_TYPE_DYNAMIC_TGTQ:
			xmit_path = mpl_load_balance_target(device, cmd, xmit_path);
			mpl_stub_switch_path(stub, xmit_path);
			break;
		case MPL_PC_TYPE_DYNAMIC_HBAQ:
			xmit_path = mpl_load_balance_hba(device, cmd, xmit_path);
			mpl_stub_switch_path(stub, xmit_path);
			break;
		case MPL_PC_TYPE_DYNAMIC_TIME:
			xmit_path = mpl_load_balance_time(device, cmd);
			mpl_stub_switch_path(stub, xmit_path);
			break;
		default:
			del_timer(&stub->cmd.eh_timeout );
			scsi_cmd->host_scribble = NULL; 
			scsi_cmd->result = ScsiResult(rv, 0);
			goto out_error;
		}
	}


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	rv = mpl_do_base_queuecommand(device, xmit_path, stub);
#else
	rv = mpl_do_base_queuecommand(device, xmit_path, stub);
#endif

	if (rv) {
		/* If base driver cannot process this cmd,
		 * then perform cleanup for this cmd before
		 * returning.  */
		mpl_module_unlock();
		del_timer_sync(&stub->cmd.eh_timeout );
		mpl_module_lock();
		scsi_cmd->host_scribble = NULL; 
		mpl_stub_free(stub);
	}

	mpl_module_unlock();
	return rv;

out_error:
	if (scsi_cmd->allowed != MPL_MAX_RETRIES ) 
		scsi_cmd->allowed = MPL_MAX_RETRIES ;
	if (!blk_noretry_request(scsi_cmd->request)) {
		if (scsi_cmd->retries == 20) {
			 scsi_cmd->retries = 0 ;
			 mpl_print(0, MPL_ERR "mpl_queuecommand : Retry count 20\n");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
			  scsi_cmd->jiffies_at_alloc = jiffies ;
#endif 
		}
	} else
	 mpl_print(0, MPL_ERR "mpl_queuecommand:No retry flag set ignore I/O error \n");
	mpl_module_unlock();
	done(scsi_cmd);
	return 0;
}

static inline int
mpl_logical_unit_not_supported(unsigned char *sense)
{
	return (sense[2]  == 0x05 &&		/* sense key */
		sense[12] == 0x25 &&		/* asc */
		sense[13] == 0x00);		/* asq */
}

static inline int
mpl_illegal_request(unsigned char *sense)
{
	return (sense[2] == 0x05);
}

static void
mpl_no_plugin(struct scsi_cmnd *cmd, struct mpl_device *device,
	      struct mpl_path *path)
{				/* simply used for diagnostic output. */
	uint8_t vendor[8 + 1];
	uint8_t product[16 + 1];
	uint8_t rev[4 + 1];
	uint8_t *request;
	int len = 0;

	request = mpl_find_cmnd_data(cmd, &len);
	if ((request == NULL) || (len < 36)) {
		mpl_print(0, MPL_ERR "No data in no_plugin len %d: %s %s\n",
      			len, mpl_print_device(path->device),
      			mpl_print_path(path))
		return;
	}

	trim_to(vendor, request + 8, sizeof(vendor));
	trim_to(product, request + 16, sizeof(product));
	trim_to(rev, request + 32, sizeof(rev));
	mpl_release_cmnd_data(cmd);
	mpl_print(0, MPL_ERR "Cannot find plugin for %s %s, %s, %s, %s\n",
		  mpl_print_device(device),
		  mpl_print_path(path),
		  vendor, product, rev);
	return;
}

static void
mpl_process_inquiry(struct io_stub *stub)
{
	struct mpl_path   *path = stub->path;
	struct mpl_path   *nextpath;
	struct mpl_device *device = path->device;
	struct scsi_cmnd *cmd = &stub->cmd;
	uint8_t *sns = cmd->sense_buffer;
	uint8_t *request;
	int len = 0;
	int  failure;
	unsigned long flags;
	
	/* This function is also called for internal command */
        if (stub->done && (cmd->eh_timeout.function != NULL && !del_timer(&cmd->eh_timeout))) {
		BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
		stub->has_timer = 1 ; 
		stub->reason = MPL_DO_INT_SCSI_TMO ;
		spin_lock_irqsave(&stub_queue_lock, flags);
		mpl_add_stub_to_scsitmo_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		/* Let timer function return command */
		mpl_module_unlock();
		schedule_timeout(HZ);
		mpl_module_lock();
		return ;
	}
	mpl_module_unlock();
	request = mpl_find_cmnd_data(cmd, &len);
	if ((request == NULL) || (len < 36)) {
		mpl_print(0, MPL_ERR "No data in process inquiry req %p len %d result %x: %s %s\n",
      			request, len, cmd->result, mpl_print_device(path->device),
      			mpl_print_path(path))
		goto bypass_snooping;
	}

	/* Path this cmd just finished on might not be the original
	 * path.
	 */

	mpl_print(MPL_RECOVERY, MPL_INFO "Inquiry Page 0 "
			  "done %s %s - result %x, sns: "
			  "%x/%x/%x\n",
			  mpl_print_device(device),
			  mpl_print_path(path),
			  cmd->result, sns[2],
			  sns[12], sns[13]);

	failure = (cmd->result || (request[0] == 0x7f) ||
		   mpl_illegal_request(sns)  ||
		   ((request[0] & 0xE0) == 0x20));

	mpl_module_lock();
	if (failure) {
		/* If device is unconfigured or this is the first
		 * inquiry on a newly created device, remove / free it.
		 */
		if (!mpl_is_device_configured(device) ||
		    (mpl_auto_configure && device->just_created)) {
			if (device->deconfig_api) {
				goto bypass_snooping;
			}
			mpl_remove_device_from_device_hash(device);
			list_del(&device->devices);
			mpl_module_unlock();
			del_timer_sync(&device->nopath);
			mpl_module_lock();
			device->on_reaperq = 1;
			list_add_tail(&device->devices, &reaper_queue);
			device->host->dev_count--;
		}
	} else {
		device->just_created = 0;
		device->scsi_type = request[0] & 0x1f;
		/* Check for tape device */
		if (device->scsi_type == 1) {
			/* Tape devices are just pass-thru */
			if(!path->got_wwpn_data && path->host->hba) {
				/* Get the WWPN for this path */
				path->got_wwpn_data =
				  mpl_get_tgt_wwinfo(path->host->hba, path->target, path->tgt);
			}
			mpl_set_path_configured(path, CONFIG_UNAVAILABLE);
			goto bypass_snooping;
		}

		/* TPGS support */
		path->tpgs = (request[5] & MPL_ALUA_TPGS_MASK);

		if (path->tpgs) {
			mpl_print(MPL_ALUA, MPL_INFO "Inquiry P0 ALUA support x%x: %s %s\n",
				path->tpgs,
		  		mpl_print_device(device),
		  		mpl_print_path(path));

			/* path->tpgs will be used later in the HSM to decide whether or not
			 * send RTPG before InquirySN.
			 */
		}

		mpl_module_unlock();
		if (device->plugin == NULL) {
			device->plugin = mpl_find_plugin(cmd, device, path);
			if (!device->plugin) {
				mpl_no_plugin(cmd, device, path);
				mpl_module_lock();
				goto bypass_snooping;
			}
		}
		mpl_module_lock();

		if (stub->done) {
			if (mpl_is_device_configured(device)) {
				list_for_each_entry(nextpath, &device->paths, path_list) {
					if(nextpath != path) {
						nextpath->flag |= PATH_NEEDS_INQUIRYP0;
						nextpath->retries = 1;
						mpl_restart_sched_cmd(nextpath, jiffies + 1);
					}
				}
			}
		}
		if(!path->got_wwpn_data && path->host->hba) {
			/* Get the WWPN for this path */
			path->got_wwpn_data = 
				mpl_get_tgt_wwinfo(path->host->hba,
				path->target, path->tgt);
			mpl_evaluate_config_exclusion(path);
		}

		mpl_path_state_machine(path, PATH_UP);

		if (!failure && stub->done && !device->serial_size) {
			/* Make sure the HBeatSM has started */
			if (path->hbstate == HB_TUR) {

				/* 
				   During the initial lun scan, timeout value is
				   only 5 seconds. If we are delaying the completion
				   of this command increase the timeout.
			 	 */
				path->delayed_cmd = stub;
				mpl_module_unlock();
				if (del_timer(&stub->cmdp->eh_timeout)) {
					mod_timer(&cmd->eh_timeout, jiffies + MPL_FIRST_INQ_TMO *HZ);
					mod_timer(&stub->cmdp->eh_timeout, jiffies + 300*HZ );
				}
				else {
					/* mid layer command timeout just free data */
					mpl_module_lock();
					goto bypass_snooping;
				}

				mpl_module_lock();
				mpl_release_cmnd_data(cmd);
				return;
			}
		}
	}
	
bypass_snooping:
	mpl_release_cmnd_data(cmd);
	if (stub->done) {
		mpl_call_done_free_stub(stub);
	}
	return;
}

/* Move paths from one device to another. 
 */
static void
mpl_adjust_count(struct mpl_device *src, 
	struct mpl_device *dest, struct mpl_path *path)
{
	struct io_stub *stub;
	struct mpl_host *host;
	unsigned long flags;

	path->device = dest;

	/* For the current IO */
	atomic_dec(&src->io_count);
	atomic_inc(&dest->io_count);
	while ((int)(atomic_read(&src->ref_count)) > 0) {
		atomic_dec(&src->ref_count);
		atomic_inc(&dest->ref_count);
	}

	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry(host, &mpl_hosts, host_list) {
		list_for_each_entry(stub, &host->lpfc_queue, stub_queue) {
			if (stub && stub->path == path) {
				atomic_dec(&src->io_count);
				atomic_inc(&dest->io_count);
			}
		}
		list_for_each_entry(stub, &deferral_queue, stub_queue) {
			if (stub && stub->path == path) {
				switch (stub->reason) {
				case MPL_INQ_CMD:
				case MPL_FAILED_CMD:
				case MPL_RETRY_CMD:
				case MPL_ABORTED_CMD:
					atomic_dec(&src->io_count);
					atomic_inc(&dest->io_count);
					break;
				default:
					break;
				}
			}
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	return;
}

static void
mpl_redirect_pending_ios(struct mpl_device *dest, struct mpl_device *src)
{
	struct io_stub    *stub;
	unsigned long flags;

	spin_lock_irqsave(&stub_queue_lock, flags);
	while ((stub = mpl_get_first_pendq(src))) {
        	if (stub->cmd.eh_timeout.function != NULL && 
			!del_timer(&stub->cmd.eh_timeout)) {

			/* Add command  to int_scsi_tmo_queue  to handle by timer routine */
			BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
			stub->has_timer = 1 ; 
			stub->reason = MPL_DO_INT_SCSI_TMO ;
			mpl_add_stub_to_scsitmo_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			/* Let timer function return command */
			mpl_module_unlock();
			schedule_timeout(HZ);
			mpl_module_lock();
			spin_lock_irqsave(&stub_queue_lock, flags);
			continue ;
		}
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_add_pending_ios(stub, dest);
		spin_lock_irqsave(&stub_queue_lock, flags);
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	if (dest->state != DEV_FAILING) {
		mpl_device_process_pendq(dest);
	}
}

static void
mpl_redirect_paths(struct mpl_device *dest, struct mpl_device *src)
{
	struct mpl_path *path;

	while (!list_empty(&src->paths)) {
		path = mpl_first_path(src);
		if (path->sas == MPL_ALUA_ACTIVE_OPT) {
			atomic_dec(&path->device->opt_path_cnt);
			atomic_inc(&dest->opt_path_cnt);
		}
		if(path->state == PATH_ONLINE)
			path->state = PATH_STANDBY;
		list_del(&path->path_list);
		list_add_tail(&path->path_list, &dest->paths);

		/* Fix up device reference counts related to
		 * any outstanding io_stubs
		 */
		mpl_adjust_count(src, dest, path);
	}
}

				/* Consolidate paths on two devices, return
				 * the survivor. */
static struct mpl_device *
mpl_consolidate_devices(struct mpl_device *device, struct mpl_device *alt)
{
	struct mpl_path *p;

		/* Calls routines that takes either the device
		 * hash lock or the stub hash lock.  So
		 * neither lock may be held on entry. */

		/* Theoretically only one device could be
		 * marked configured.  If it is the alternate
		 * device, switch roles so we end up with the
		 * device set up at initialization time. */

	if (!mpl_is_device_configured(device) &&
	    mpl_is_device_configured(alt)) {
		struct mpl_device *tmp;

		tmp = device;
		device = alt;
		alt = tmp;
	}
	if (alt->StaticConfig) {
		mpl_set_device_static_config(device);
		if (alt->StaticConfigPrimary) {
			list_for_each_entry(p, &device->paths, path_list) {
				if(p->is_default) {
					p->StaticConfig = 0;
					atomic_dec(&p->tgt->prim_path_ref_count);
				}
				p->is_default = 0;
			}
			device->StaticConfigPrimary = 1;
		}
	}
	mpl_redirect_paths(device, alt);
	mpl_redirect_pending_ios(device, alt);
	mpl_remove_device_from_device_hash(alt);
	list_del(&alt->devices);
	mpl_module_unlock();
	del_timer_sync(&alt->nopath);
	mpl_module_lock();
	alt->on_reaperq = 1;
	list_add_tail(&alt->devices, &reaper_queue);
	alt->host->dev_count--;
	return device;
}

/* Search for existing device with newly probed serial number.  If
 * found, move current device path(s) to existing device.  If not
 * found, update this device serial number.  Return old device, if it
 * existed, current device if not. */

static struct mpl_device *
mpl_match_device_by_serial(struct mpl_device *device, uint8_t *serial, int size)
{
	struct mpl_host *host;
	struct mpl_device *d;
	int  i;

	if (!device->serial_size) {
		return device;
	}
	for (i = 0; ; i++) {
		host = mpl_find_host(i);
		if (host == NULL) {
			break;
		}
		list_for_each_entry(d, &host->devices, devices) {
			if (d != device &&
			    !mpl_compare_serial_number(d, serial, size)) {
				d = mpl_consolidate_devices(d, device);
				return d;
			}
		}
	}
	return device;
}

void
mpl_process_internal_cmd(struct io_stub *stub)
{
	struct scsi_cmnd *cmd = &stub->cmd;
	struct mpl_path *path = stub->path;
	struct mpl_device *device = path->device;
	unsigned long flags;

	mpl_print(MPL_INTERNAL, MPL_INFO "Cmpl cmd %x %x for %s %s "
		  "result %x sk %x asc %x ascq %x\n",
		  cmd->cmnd[0], cmd->cmnd[2],
		  mpl_print_device(device), mpl_print_path(path),
		  cmd->result, cmd->sense_buffer[2],
		  cmd->sense_buffer[12], cmd->sense_buffer[13]); 

	if (mpl_is_power_on_reset(cmd->result, cmd->sense_buffer) ||
	    mpl_is_queue_full(cmd->result) ||
	    mpl_is_queue_busy(cmd->result)) {
		/* Just retry 29/00 and QFULL errors */
		memset(&cmd->sense_buffer[0], 0, SCSI_SENSE_BUFFERSIZE);
		path->retries++;
		mpl_set_cmd_timer(stub, MPL_INTERNAL_CMD_TMO);
		spin_lock_irqsave(&stub_queue_lock, flags);
		stub->reason = MPL_INTERNAL_CMD;
		mpl_add_stub_to_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		mpl_wake_thread();
		mpl_module_lock();
		return;
	}

	if (mpl_is_inquiry_page_0(cmd)) {
		/* Inquiry P0 is not generated by a plugin, the mpl module
		 * generates this when processing a Inquiry P0 on a configured
		 * presentation path. Thus all types of devices should take
		 * the action coded in mpl_process_inquiry().
		 */
		mpl_process_inquiry(stub);
	} else if (device->plugin) {

		/* Always call plugin for plugin generated internal commands */
	 	device->plugin->diagnose_results(stub, PLUG_INTERNAL_CMD);
	}

	mpl_stub_free(stub);
	return;
}


static inline int
mpl_is_retryable(struct scsi_cmnd  *cmd)
{
				/* This is a quick hack to fix CR 15564.  We
				 * need a more robust mechanism that allows
				 * the diagnose plugin to determine whether
				 * the I/O is retryable or not. */
	return (cmd->result == 2 && 
		(cmd->sense_buffer[2] & 0xf) == 0x02 &&
		cmd->sense_buffer[12] == 0x04);
}

static void
mpl_expedite_hbeat(struct mpl_path *path)
{
	struct mpl_path   *hpath;
	unsigned long flags;
	int i;

	/* If path is currently functional and is in process of going down */
	if ((path->state != PATH_ONLINE) &&
	    (path->state != PATH_STANDBY)) 
		return;

	/* Check all other paths on this target by expediting their hbeat */
	spin_lock_irqsave(&path_hash_table_lock, flags);
	for (i=0;i<path_hash_size;i++) {
redohash:
		for (hpath = path_hash[i]; hpath; hpath = hpath->next) {
			if (hpath->host == path->host &&
		    	    hpath->target == path->target)
			{
				if ((hpath->hbstate == HB_WAITING) &&
				   !(hpath->flag & PATH_FAST_POLL)) {
					hpath->flag |= PATH_FAST_POLL;
					spin_unlock_irqrestore(
						&path_hash_table_lock, flags);
					mpl_module_unlock();
					mod_timer(&hpath->hbeat, jiffies + HZ * MPL_DFT_HBEATFAIL_INTERVAL);
					mpl_module_lock();
					spin_lock_irqsave(
						&path_hash_table_lock, flags);
					goto redohash;
				}
			}
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	return;
}

void
mpl_process_hbeat_cmd(struct io_stub *stub)
{
	struct scsi_cmnd *cmd = &stub->cmd;
	struct mpl_path *path = stub->path;
	struct mpl_device *device = path->device;
	enum mpl_hb_event event = HB_SUCCEED;
	int  diagnose_reject = 0;
	uint8_t *datap;
	struct io_stub *deferral_stub ;
	unsigned long flags;
	int temp ;
	int wake_thread = 0;
	if (cmd->result == ScsiResult(DID_NO_CONNECT, 0)) {
		mpl_expedite_hbeat(path);
		event = HB_FAIL;
	} else if (mpl_is_queue_full(cmd->result) ||
		   mpl_is_queue_busy(cmd->result) ||
		   (cmd->result == ScsiResult(DID_ERROR, 0))) {
		event = HB_BUSY;
	} else if (device->plugin) {
		if (cmd->result == ScsiResult(DID_TIME_OUT, 0)) {
			path->flag |= PATH_HBEAT_TMO;
			event = HB_FAIL;
		}
		diagnose_reject = device->plugin->diagnose_results(stub, PLUG_HBEAT_CMD);
		if (diagnose_reject)
			event = HB_FAIL;
		else if (mpl_is_inquiry_page_83(cmd) && mpl_hbeat_inquirysn(stub)) {
			event = HB_FAIL;
		}
	}
	if (event == HB_FAIL) {
		if (path->delayed_cmd &&
			(del_timer(&path->delayed_cmd->cmd.eh_timeout))) {

			mpl_call_done_free_stub(path->delayed_cmd);
			path->delayed_cmd = NULL;
		}
	}

	if ((event == HB_SUCCEED) &&
		(path->hbstate == HB_RTPG)) {
		if (path->tgt_port_info) {
			mpl_data_save_cmd--;
			mpl_put_scsi_buf(path->tgt_port_info);
			path->tgt_port_info = 0;
		}
		datap = mpl_get_scsi_buf(); 
		if (datap) {
			mpl_data_save_cmd++;
			path->tgt_port_info = cmd->request_buffer;
			cmd->request_buffer = datap;
		}
	}
	
	/* Decrement inflight heartbeat count
	 * check to see if there is any thing in hbeat_deferral queue if there
	 *  is any thing add that to deferral queue if there is bandwidth.
	 */
	atomic_dec(&path->host->inflight_hbeat_count) ;
	temp = atomic_read(&path->host->inflight_hbeat_count)  ;
	 if ( (temp < path->host->hbeat_canqueue) 
		&& ( !list_empty(&path->host->hbeat_deferral_queue) ) ) {
		mpl_print( MPL_HBEATDEFQ, MPL_INFO "Process_Hbeat_Cmd:Moving hbeat from"
			" HBEATDEFQ to deferrel queue Inflight hbeat %d\n", temp );
		spin_lock_irqsave(&stub_queue_lock, flags) ;
		deferral_stub = mpl_get_first_hbeat_deferral(path->host)  ;
		if (deferral_stub) {
			mod_timer(&deferral_stub->cmd.eh_timeout, jiffies + MPL_CMD_TMO * HZ );
			mpl_add_stub_to_deferral(deferral_stub);
			wake_thread = 1;
		}
		else
			 mpl_print(MPL_HBEATDEFQ, MPL_WARNING "Process_HBeat_Cmd: Locking issue in"
			" Hbeat_deferral_queue\n" );
		 spin_unlock_irqrestore(&stub_queue_lock,flags);
	 }
	 else {
		mpl_print( MPL_HBEATDEFQ, MPL_INFO "Process_Hbeat_cmd: Inflight_hbeatcount"
		" %d and HBEATDEFERRALQ is %d \n" ,temp,
		list_empty(&path->host->hbeat_deferral_queue) ); 
	}	 

	mpl_stub_free(stub);
	mpl_hbeat_state_machine(path, event);
	path->flag &= ~PATH_HBEAT_TMO;

	if (wake_thread) {
		mpl_module_unlock();
		mpl_wake_thread();
		mpl_module_lock();
	}

	return;
}

static void
mpl_process_failed_cmd(struct io_stub *stub)
{
	struct scsi_cmnd  *cmd = &stub->cmd;
	struct mpl_path   *path = stub->path;
	struct mpl_device *device = path->device;
	int retry_cmd = 0;
	int diag_res;
	unsigned long flags;

        if (cmd->eh_timeout.function != NULL && !del_timer(&cmd->eh_timeout)) {
		/* Add command  to int_scsi_tmo_queue  to handle by timer routine */
		BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
		stub->has_timer = 1 ; 
		stub->reason = MPL_DO_INT_SCSI_TMO ;
		spin_lock_irqsave(&stub_queue_lock, flags);
		mpl_add_stub_to_scsitmo_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		/* Let timer function return command */
		mpl_module_unlock();
		schedule_timeout(HZ);
		mpl_module_lock();
		return ;
	}

	if (device->state == DEV_FAILED) {
		if (timer_pending(&device->nopath)) {
			mpl_add_pending_ios(stub, device);
			return;
		}
		/* All paths to device are already down,
		 * just return the failed command to the SCSI layer.
		 */
		mpl_call_done_free_stub(stub);
		return;
	}

	if (cmd->result == ScsiResult(DID_NO_CONNECT, 0)) {
		mpl_expedite_hbeat(path);
		goto fover;
	}
	/* All TURs should go thru plugin diagnostic check */
	if (device->plugin && 
	    (diag_res = device->plugin->diagnose_results(stub,PLUG_SCSI_CMD))) {
		if (diag_res == 2) { /* retry with no failover */
			retry_cmd = 1;
		} else {
fover:
			/* Attempt retry with failover */
			mpl_path_state_machine(path, PATH_DOWN);
			retry_cmd = 1;
		}
	}

	/* If the path I got this error on is FAILED/DISABLED, and there
	 * are other online / standy paths available, retry.
	 */
	if ((path->state == PATH_FAILED) ||
		 (path->state == PATH_DISABLED)) {
		if (device->state != DEV_FAILED)
			retry_cmd = 1;
	}

	if (retry_cmd) {

		mpl_print(MPL_IOPATH, MPL_INFO "RETRY cmd %x %x for %s %s "
		  "result %x sk %x asc %x ascq %x\n",
		  cmd->cmnd[0], cmd->cmnd[2],
		  mpl_print_device(device), mpl_print_path(path),
		  cmd->result, cmd->sense_buffer[2],
		  cmd->sense_buffer[12], cmd->sense_buffer[13]); 

		mpl_add_pending_ios(stub, device);
	} else {
		mpl_call_done_free_stub(stub);
	}
	return;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
int
mpl_do_devloss_tmo(struct io_stub *stub)
{
	struct fc_rport *rport;	  
	struct mpl_path *path = stub->path;
	struct mpl_target *tgtp;

	if (!path || !path->tgt || driver_unloading) {
		mpl_stub_free(stub);
		return 0;
	}
	tgtp = path->tgt;
	rport = tgtp->rport;
	if (rport) {
		/* Make it look like the rport went away */
		rport->roles = FC_RPORT_ROLE_UNKNOWN;
		rport->port_state = FC_PORTSTATE_NOTPRESENT;
		mpl_module_unlock();

		mpl_base_devloss_tmo(rport);

		mpl_module_lock();
		mpl_do_clear_ndlp(rport->dd_data);
		list_for_each_entry(path, &tgtp->paths, tgt_path_list) {
			mpl_path_state_machine(path, PATH_DOWN);
			path->lld_blocked = 0;
		}
	}
	mpl_stub_free(stub);
	return 0;
}
#endif

/*
   This function is called from mpl_eh_thread context due to internal 
   command timeout and is called with module_lock held.
*/
int
mpl_do_int_tmo(struct io_stub *stub, void (*done)(struct scsi_cmnd *))
{
	struct Scsi_Host *shost;
	struct mpl_host *host;
	struct mpl_path *path;
	struct scsi_cmnd *cmd = &stub->cmd;
	int ret = FAILED;

	shost = cmd->device->host;
	host  = mpl_find_host_by_scsi_host(shost);
	if (!host || !shost) {
		mpl_print(0, MPL_ERR "Internal cmd 0x%x tmo: No host:%p %p\n",
			cmd->cmnd[0], host, shost);
		mpl_stub_free(stub);
		return 0;
	}

	/* path this cmd just finished on. Might not be
	 * the original path.
	 */
	path = stub->path;
#ifdef MPL_DBL_FREE	
	BUG_ON(cmd->host_scribble == (void*)0x8585 );
#endif
	if (driver_unloading || !cmd->host_scribble){
		mpl_print(MPL_TMO, MPL_INFO "mpl_do_int_tmo: host_scriblle is null"
				" stub state x%x reason x%x \n", 
				stub->state, stub->reason);
		ret = SUCCESS;
		goto out;
	}

	mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(shost->host_lock);
	ret = mpl_base_abort_handler(cmd);
	spin_unlock_irq(shost->host_lock);
#else
	ret = mpl_base_abort_handler(cmd);
#endif
	mpl_module_lock();

out:
	/* Call process cmpl with error */
	if (ret == FAILED) {
		mpl_print(0, MPL_ERR "Internal cmd 0x%x abort failed for %s %s result %08x\n",
			  cmd->cmnd[0],
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  cmd->result);
	}
	if (stub->reason == MPL_DO_HBEAT_TMO) {
		if (mpl_hbeat_tmo_busy)
			cmd->result = ScsiResult(DID_BUS_BUSY, 0);
		else
			cmd->result = ScsiResult(DID_TIME_OUT, 0);
		if (ret == SUCCESS) {
			mpl_process_hbeat_cmd(stub);
		} else {
			stub->flag |= MPL_ABORT_FAILED;
			mpl_hbeat_state_machine(path, HB_BUSY);
			mpl_print(MPL_TMO, MPL_ERR "HB cmd 0x%x Failed abort %s %s"
				" reason %d pstate %s dstate %s\n",
				cmd->cmnd[0],
				mpl_print_device(path->device),
				mpl_print_path(path),
				stub->reason, mpl_pstates[path->state], 
				mpl_dstates[path->device->state]);
			mpl_stub_free(stub);
		}
	} else {
		cmd->result = ScsiResult(DID_ERROR, 0);
		if (ret == SUCCESS) {
			mpl_process_internal_cmd(stub);
		} else {
			stub->flag |= MPL_ABORT_FAILED;
			mpl_hbeat_state_machine(path, HB_TIMEOUT);
			mpl_print(MPL_TMO, MPL_ERR "Internal cmd 0x%x Failed abort %s %s"
				" reason %d pstate %s dstate %s\n",
				cmd->cmnd[0],
				mpl_print_device(path->device),
				mpl_print_path(path),
				stub->reason, mpl_pstates[path->state], 
				mpl_dstates[path->device->state]);
			mpl_stub_free(stub);
		}
	}

	mpl_print(MPL_TMO, MPL_INFO "Internal cmd 0x%x tmo for %s %s"
		" reason %d pstate %s dstate %s\n",
		cmd->cmnd[0],
		mpl_print_device(path->device),
		mpl_print_path(path),
		stub->reason, mpl_pstates[path->state], mpl_dstates[path->device->state]);

	return 0;
}

/* this function is called with mpl_module_lock() held */
int
mpl_do_int_scsi_tmo(struct io_stub *stub) {
	int ret ;
	struct scsi_cmnd *scsi_cmd, *cmd;
	struct mpl_path  *path;
	struct Scsi_Host *abort_shost, *host;

	scsi_cmd = stub->cmdp;
	path = stub->path;
	/* Switch to active cmnd */
	cmd = &stub->cmd;
	
	host = mpl_scsi_host_from_cmd(scsi_cmd);
	abort_shost = cmd->device->host;
	mpl_print(MPL_TMO, MPL_INFO "mpl_do_int_scsi_tmo: " 
		"host/target/lun %d/%d/%d"
		"scsi _host busy %u host_failed %u"
		"host_self_blocked %u host_blocked %u" 
		"scsi_cmnd %p cmd %p stub %p\n",
		mpl_host_no_from_cmd(scsi_cmd),
		mpl_target_no_from_cmd(scsi_cmd), mpl_lun_no_from_cmd(scsi_cmd),
		host->host_busy, host->host_failed, 
		host->host_self_blocked,
		host->host_blocked,  scsi_cmd, cmd, stub );
#ifdef MPL_DBL_FREE 
		BUG_ON(cmd->host_scribble == (void *)0x8585 );
#endif

	if (!cmd->host_scribble ) {
#if LINUX_VERSION_CODE >  KERNEL_VERSION(2,6,16)
		mpl_print(MPL_DEBUG, MPL_ERR "##Command already complete path %s stubp %p cmdp %p \n",
		mpl_print_path(path), stub, stub->cmdp );
		return SUCCESS ;
#else
		printk(KERN_ERR "mpl_do_int_scsi_tmo: Line 4082  \n");
#endif
	}
	mpl_module_unlock();

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	/* lpfc driver call block for abort host. 
	 * however presentation path host is not block.
	 * as a result i/o is keep coming but transmit 
	 * path get block causing I/O to go in pendq 
	*/
	if (host != abort_shost) {
		scsi_block_requests_wrapped(host);
	}
	spin_lock_irq(abort_shost->host_lock);
	ret = mpl_base_abort_handler(cmd);
	spin_unlock_irq(abort_shost->host_lock);
	if (host != abort_shost) {
		scsi_unblock_requests_wrapped(host);
	}
#else
	ret = mpl_base_abort_handler(cmd);
#endif
	mpl_module_lock();
	if (ret == FAILED) {
		mpl_print(0, MPL_ERR "mpl_do_int_scsi_tmo : abort failed ret x%x \n", ret);
		return ret;
	}

	if (stub->has_timer) {
		mod_timer(&stub->cmdp->eh_timeout, jiffies + 60 * HZ) ;
	}
	if (stub->cmdp)
		stub->cmdp->host_scribble = NULL;
	stub->cmd.result = ScsiResult(DID_ERROR, 0);
	mpl_call_done_free_stub(stub);
	if (!(atomic_read(&path->io_count))) {
		/* If there are no outstanding IOs on this path,
	 	* zero the avg time.
	 	*/
		atomic_set(&path->avg_cnt, 0);
		path->avg_time = 0;
	}
	return ret ;
}

static void
mpl_process_aborted_cmd(struct io_stub *stub)
{
	mpl_call_done_free_stub(stub);
	return;
}

int
mpl_set_path_online(struct mpl_path *path)
{
	struct mpl_device *device = path->device;

	/* Select primary path, if needed */
	if(!mpl_get_primary_path(device)) {
		path->is_default = 1;
		atomic_inc(&path->tgt->prim_path_ref_count);
	}
	else {
		if (path->state != PATH_DISABLED)
			mpl_auto_primary_select(device, path);
	}

	if (path->state == PATH_DISABLED)
		mpl_path_state_machine(path, PATH_ENABLE);
	else
		mpl_path_state_machine(path, PATH_UP);
	/* Hbeat will bring path up if it succeeds */
	return 0;
}

int
mpl_set_path_offline(struct mpl_path *path)
{
	mpl_path_state_machine(path, PATH_DISABLE);
	return 0;
}

int
mpl_clear_path_statistics(struct mpl_path *path)
{
				/* TODO: UI needs some path statistics */
	return 0;
}

static void
mpl_reap_queue(void)
{
	struct mpl_device *device, *tmp;
	struct mpl_path   *path;
	struct io_stub    *stub;
	unsigned long flags;
	LIST_HEAD(device_list);
	list_splice_init(&reaper_queue, &device_list);

	list_for_each_entry_safe(device, tmp, &device_list, devices) {
				/* Kill any outstanding mid layer I/Os */
		spin_lock_irqsave(&stub_queue_lock, flags);
		while ((stub = mpl_get_first_pendq(device))) {
        		if (stub->cmd.eh_timeout.function != NULL && 
				!del_timer(&stub->cmd.eh_timeout)) {

				/* Add command  to int_scsi_tmo_queue  to handle by timer routine*/
				BUG_ON(!del_timer(&stub->cmdp->eh_timeout));
				stub->has_timer = 1 ; 
				stub->reason = MPL_DO_INT_SCSI_TMO ;
				stub->cmd.result = ScsiResult(DID_NO_CONNECT, 0);
				mpl_add_stub_to_scsitmo_deferral(stub);
				continue ;
			}
			stub->cmd.result = ScsiResult(DID_NO_CONNECT, 0);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_call_done_free_stub(stub);
			spin_lock_irqsave(&stub_queue_lock, flags);

		}
		spin_unlock_irqrestore(&stub_queue_lock, flags);

		/* Make sure HBeat is stopped */
		list_for_each_entry(path, &device->paths, path_list) {
			mpl_hbeat_state_machine(path, HB_STOP);
		}
				/* note: mpl_free_device does the list_del to
				 * remove this device from the reaper queue */
		mpl_free_device(device);
	}
}

static int
mpl_thread(void *arg)
{
	struct io_stub *stub;
	unsigned long flags;

	siginitsetinv(&current->blocked, 0);
	lock_kernel();
	daemonize("mpl_thread");
	snprintf(current->comm, sizeof(current->comm), "mpl_thread");
	unlock_kernel();
	mpl_module_lock();
	while (mpl_thread_quit == 0) {
#ifdef MPL_STUB_SANITY		
		if (mpl_log_verbose == MPL_DEBUG)
			mpl_dump_all_scsi_cmd();	
#endif
		mpl_module_unlock();
		down_interruptible(&mpl_thread_sleep);
		mpl_module_lock();

		spin_lock_irqsave(&stub_queue_lock, flags);
		stub = mpl_get_first_deferral();
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		while (stub) {
			switch (stub->reason) {
			case MPL_OOS_INQ_CMD:
				mpl_process_oos_inquiry(stub);
				break;
			case MPL_INQ_CMD:
				mpl_process_inquiry(stub);
				break;
			case MPL_INT_CMD:
				mpl_process_internal_cmd(stub);
				break;
			case MPL_HB_CMD:
				mpl_process_hbeat_cmd(stub);
				break;
			case MPL_FAILED_CMD:
				mpl_process_failed_cmd(stub);
				break;
			case MPL_RETRY_CMD:
				mpl_retry_pending_ios(stub);
				break;
			case MPL_HEARTBEAT_CMD:
				mpl_send_internal_completion(stub->path, stub,
							     mpl_hb_done);
				break;
			case MPL_INTERNAL_CMD:
				mpl_send_internal_completion(stub->path, stub,
						mpl_internal_done);
				break;
			case MPL_ABORTED_CMD:
				mpl_process_aborted_cmd(stub);
				break;
			default:
				mpl_print(0, MPL_ERR "mpl_thread: bad reason "
					  "code in io_stub: %p\n",
					  stub);
				mpl_stub_free(stub);
				break;
			}

			/* maintain deferral count until after the I/O
			 * has been completely processed
			 */
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub = mpl_get_first_deferral();
			spin_unlock_irqrestore(&stub_queue_lock, flags);
		}
		mpl_reap_queue();
	}
	mpl_module_unlock();
	up(&mpl_thread_gone);
	return 0;
}

/* This routine will handle a internal module initiated abort.
   It is called from mpl_eh_thread context to abort a specific IO
   and is called with module_lock held.
*/
int
mpl_do_abort(struct scsi_cmnd *cmd)
{
	int ret;

	mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(cmd->device->host->host_lock);
	ret = mpl_abort_handler(cmd);
	spin_unlock_irq(cmd->device->host->host_lock);
#else
	ret = mpl_abort_handler(cmd);
#endif

	mpl_module_lock();
	return ret;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
int
mpl_do_dev_scan(struct io_stub *stub)
{
	struct mpl_path   *path;
	struct mpl_target *tgtp;
	struct fc_rport   *rport;
	int i = 0;

	if (stub) {
		path =  stub->path;
		tgtp =  path->tgt;
		rport = tgtp->rport;
		if (!rport)
			return 1;

		mpl_print(MPL_IOPATH, MPL_INFO
			"Process Rescan %s %s rport %p rdy %d\n",
			mpl_print_target(tgtp), mpl_print_path(path), rport,
 			fc_remote_port_chkready(rport)); 

		scan_path = path;
		mpl_serial_scan = 0;
		mpl_module_unlock();
		while (fc_remote_port_chkready(rport)) {
			i++;
			if (i > 5) {
				break;
			}
			schedule_timeout(2);
		}
		scsi_scan_target(&rport->dev, rport->channel,
					rport->scsi_target_id, path->lun, 1);
		mpl_module_lock();
		scan_path = 0;
		mpl_serial_scan = 1;
	}
	return 0;
}
#endif

static int
mpl_eh_thread(void *arg)
{
	struct io_stub *stub;
	unsigned long flags, timeout_count = 0;

	siginitsetinv(&current->blocked, 0);
	lock_kernel();
	daemonize("mpl_eh_thread");
	snprintf(current->comm, sizeof(current->comm), "mpl_eh_thread");
	unlock_kernel();
	mpl_module_lock();
	while (mpl_eh_thread_quit == 0) {
		if (timeout_count > 90) 
			mpl_print(0, KERN_ERR "Total timeout command %lu \n", timeout_count );
		timeout_count = 0 ;
		mpl_module_unlock();
		down_interruptible(&mpl_eh_thread_sleep);
		mpl_module_lock();

		spin_lock_irqsave(&stub_queue_lock, flags);
		stub = mpl_get_first_eh_deferral();
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		while (stub) {
			struct mpl_path *path = stub->path;
			struct mpl_device *device = path->device;

			switch (stub->reason) {
			case MPL_DO_INT_TMO:
				mpl_do_int_tmo(stub, mpl_internal_done);
				break;
			case MPL_DO_HBEAT_TMO:
				mpl_do_int_tmo(stub, mpl_hb_done);
				break;
			case MPL_DO_ABORT:
				mpl_do_abort(stub->cmdp);
				break;
			case MPL_DO_DEV_RESET:
				mpl_do_dev_reset(stub);
				mpl_stub_free(stub);
				mpl_device_process_pendq(device);
				break;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
			case MPL_DO_DEVLOSS_TMO:
				mpl_do_devloss_tmo(stub);
				break;
			case MPL_DO_DEV_SCAN:
				mpl_do_dev_scan(stub);
				mpl_stub_free(stub);
				break;
#endif
			case MPL_DO_INT_SCSI_TMO: /* Internal timer expires */
				timeout_count++ ;
				mpl_do_int_scsi_tmo(stub);
				break ;
			default:
				mpl_print(0, MPL_ERR "mpl_eh_thread: bad reason "
					  "code in io_stub: %p\n",
					  stub);
				mpl_stub_free(stub);
				break;
			}

			/* maintain eh_deferral count until after the I/O
			 * has been completely processed
			 */
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub = mpl_get_first_eh_deferral();
			spin_unlock_irqrestore(&stub_queue_lock, flags);
		}
		mpl_reap_queue();

	}
	mpl_module_unlock();
	up(&mpl_eh_thread_gone);
	return 0;
}

int
mpl_release(struct Scsi_Host *shost)
{
	struct mpl_host *host;
	struct mpl_device *device;
	struct mpl_path *path;
	int ret;

	mpl_module_lock();
	driver_unloading = 1;
	/* Lets cancel all our timers before we unload the base driver */
	list_for_each_entry(host, &mpl_hosts, host_list) {
		if (host->scsi_host != shost)
			continue;
		list_for_each_entry(device, &host->devices, devices) {
			mpl_device_state_machine(device, NULL, DEV_DISABLE);
			list_for_each_entry(path, &device->paths, path_list) {
				mpl_path_state_machine(path, PATH_DISABLE);
				mpl_set_path_configured(path, CONFIG_UNCONFIGURED);
			}
			mpl_set_device_configured(device, CONFIG_UNCONFIGURED);
			mpl_quiesce_device(device, 30);
		}
		mpl_module_unlock();	
		del_timer_sync(&host->host_hbeat);
		mpl_module_lock();
	}
	mpl_module_unlock();
	ret = mpl_base_release(shost);
	return ret;
}

/* This routine will handle a internal module initiated device reset.
   It is called from mpl_eh_thread context to reset a specific physical device
   and is called with module_lock held.
*/
int
mpl_do_dev_reset(struct io_stub *stub)
{
	struct Scsi_Host *shost;
	struct scsi_cmnd *cmd;
	struct mpl_path *path=NULL, *xmit_path;
	int ret = 0;
	struct scsi_device *tmp_sdev;

	if (stub) {
		cmd = &stub->cmd;
		path = stub->path;
		if ((path->state == PATH_FAILED) || path->lld_blocked) {
			ret = 1;
			list_for_each_entry(xmit_path, &path->device->paths,
				path_list) {
				if ((xmit_path->state != PATH_FAILED) &&
					!xmit_path->lld_blocked) {
					ret = 0;
					break;
				}
			}
		}
		else
			xmit_path = path;

		if (ret)
			return ret;

		mpl_print(MPL_IOPATH, MPL_INFO "DEVICE RESET %p, %s %s stub %p\n",
			cmd, 
			mpl_print_path(xmit_path),
			mpl_print_device(xmit_path->device),
			stub);

		if (!cmd->device || !xmit_path->sdev) {
			xmit_path->device->reset_pending = 0;
			mpl_module_unlock();
			scsi_unblock_requests_wrapped(xmit_path->device->host->scsi_host);
			mpl_module_lock();
			return ret;
		}
		tmp_sdev = cmd->device;
		cmd->device = xmit_path->sdev;
		shost = cmd->device->host;

		mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(shost->host_lock);
		ret = mpl_base_device_reset_handler(cmd);
		spin_unlock_irq(shost->host_lock);
#else
		ret = mpl_base_device_reset_handler(cmd);
#endif
		scsi_unblock_requests_wrapped(xmit_path->device->host->scsi_host);
		mpl_module_lock();

		/* Restore the command fields */
		cmd->device = tmp_sdev;
		xmit_path->device->reset_pending = 0;
	}
	return ret;
}

struct io_stub *
mpl_find_scsi_delayed_inqcmd(struct scsi_cmnd  *cmd)
{
	struct mpl_host *host;
	struct mpl_device *device;
	struct mpl_path *path;

	list_for_each_entry(host, &mpl_hosts, host_list) {
		list_for_each_entry(device, &host->devices, devices) {
			list_for_each_entry(path, &device->paths, path_list) {
				if (path->delayed_cmd &&
					path->delayed_cmd->cmdp == cmd)
					return path->delayed_cmd;
			}
		}
	}
	return NULL;
}

void
mpl_scsi_timesout(unsigned long data)
{
	int ret;
	struct scsi_cmnd *scsi_cmd, *cmd;
	struct io_stub   *stub, *stub_scribble;
	struct Scsi_Host *shost;
	unsigned long flags;
#ifdef MPL_SCSICMD_SANITY
	int i ;
#endif

	cmd = (struct scsi_cmnd *)data ;
	stub_scribble = (struct io_stub *)cmd->host_scribble ;
	shost = cmd->device->host;

	mpl_module_lock();

	/* Find command in timeout list */
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub = mpl_match_cmnd_remove_from_scsitmo_deferral(cmd);
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	
	if (stub) {
		if (stub != stub_scribble)
		  printk(KERN_ERR "##mpl_scsi_timesout 1 stub %p stub_scribble %p \n",
			stub, stub_scribble);
		/* if (stub->has_timer) */
		mod_timer(&stub->cmdp->eh_timeout, jiffies + 60 * HZ);

		/* If result is DID_NO_CONNECT the device is already freed */
		if (stub->cmd.result != ScsiResult(DID_NO_CONNECT, 0))
			stub->cmd.result = ScsiResult(DID_ERROR, 0);

		mpl_print(MPL_TMO, MPL_INFO "mpl_scsi_timesout : free command cmd %p"
			"scsi_cmd %p stub %p and cmd x%x \n",
		  	&stub->cmd, stub->cmdp, stub,cmd->cmnd[0]);
		mpl_call_done_free_stub(stub);
		mpl_module_unlock();
		return ;
	} 
	
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub = mpl_match_cmnd_remove_stub(cmd, MPL_ABORT_EXACT);

	if (stub) {
		if (stub != stub_scribble)
		  printk(KERN_ERR "##mpl_scsi_timesout 2 stub %p stub_scribble %p \n",
			stub, stub_scribble);
		spin_unlock_irqrestore(&stub_queue_lock, flags);

		/* Switch to active cmnd */
		cmd = &stub->cmd;
		
		cmd->result = ScsiResult(DID_ERROR, 0);
		scsi_cmd = stub->cmdp;

		mpl_print(MPL_TMO, MPL_INFO
			"ABORT pending IO cmd %p scsi_cmd %p cmnd x%x, %d/%d/%d"
			" stub %p\n",
			cmd,scsi_cmd,cmd->cmnd[0],
			 mpl_host_no_from_cmd(cmd),
			mpl_target_no_from_cmd(cmd),
			mpl_lun_no_from_cmd(cmd), stub);
#ifdef MPL_STUB_SANITY
		if ((stub->flag & MPL_INTERNAL_CMD_STUB) || !scsi_cmd) {
			mpl_print(0, MPL_ERR "Abort Internal Cmd from SCSI %p, "
		  			"%s %s, cmd = %02x %02x %02x %02x\n",
       			stub,
       			mpl_print_device(stub->path->device),
       			mpl_print_path(stub->path),
       			stub->cmd.cmnd[0], stub->cmd.cmnd[1],
       			stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
		}
#endif
		mpl_call_done_free_stub(stub);
		mpl_module_unlock();
		ret = SUCCESS;
	} else {


		/* Command is owned by lpfc driver. We need to issue abort to
		 *  the lpfc driver and then restore the original sdev
		 *  structure. mpl_done can be called while abort is
		 *  pending. However, mpl_done is not guaranteed. We need to
		 *  protect against this. */

		stub = mpl_find_scsi_cmd_in_lpfcq(cmd, MPL_SCSI_CMND);
		if (!stub) {
			/* Check to see if its a SCSI scan Inquiry P0
			 * with a delayed response.
			 */
			stub = mpl_find_scsi_delayed_inqcmd(cmd);
			if (!stub) {
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				/* command got completed while abort pending. Do
				 * nothing.  return SUCCESS */
				mpl_module_unlock();
				mpl_print(0, MPL_ERR "mpl_scsi_timesout:Did not find Cmd  "
		  			"= %02x %02x %02x %02x\n",
       					cmd->cmnd[0], cmd->cmnd[1],
       					cmd->cmnd[2], cmd->cmnd[3]);
				if (stub_scribble) {
				 printk(KERN_ERR "##mpl_scsi :Unable to find stubstub__scribble %p reason x%x"
					" flag x%x sate x%x \n",
					stub_scribble, stub_scribble->reason, stub_scribble->flag,stub_scribble->state);
#ifdef MPL_SCSICMD_SANITY
				 for ( i= 0 ; i < stub_scribble->his_ind ; i++ ){
					printk(KERN_ERR "##Reason x%x State x%x \n",
						stub_scribble->history[i], stub_scribble->history[i+1] );
					i++ ;	
				 }	
#endif
				}
				mpl_print(MPL_TMO, MPL_INFO "Did not find Cmd  "
		  			"= %02x %02x %02x %02x\n",
       					cmd->cmnd[0], cmd->cmnd[1],
       					cmd->cmnd[2], cmd->cmnd[3]);
				return ;
			}
			if (stub != stub_scribble)
		  		printk(KERN_ERR "##mpl_scsi_timesout 4 stub %p stub_scribble %p \n",
				stub, stub_scribble);
			mpl_print(MPL_TMO, MPL_INFO "its Delayed command time out\n");
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			stub->path->delayed_cmd = NULL;
			stub->cmd.result = ScsiResult(DID_OK, 0);
			mpl_call_done_free_stub(stub);
			mpl_module_unlock();
			return ;
		}
		if (stub != stub_scribble)
		  printk(KERN_ERR "##mpl_scsi_timesout 3 stub %p stub_scribble %p \n",
			stub, stub_scribble);

		if (stub->cmdp->eh_timeout.function != NULL ){
			if(del_timer(&stub->cmdp->eh_timeout)){
				/*
				** Timer has not fired so move the command to the
				** deferral_q. 
				*/
				stub->has_timer = 1 ;
				mpl_remove_stub_from_lpfcq(stub);
				stub->reason = MPL_DO_INT_SCSI_TMO;
				mpl_add_stub_to_eh_deferral(stub);
			} else
				mpl_print(0, MPL_ERR "mpl_scsi_timesout: Command is handled by mid layer timer cmdp %p stub %p \n",
				 stub->cmdp, stub);
		}else 
		  printk(KERN_ERR "mpl_scsi_timesout: Why timer is null stubp %p  cmdp %p \n",
			stub, stub->cmdp );
			

		spin_unlock_irqrestore(&stub_queue_lock, flags);

		mpl_module_unlock();
		mpl_wake_eh_thread();
	}
		return ;

}


int
mpl_abort_handler(struct scsi_cmnd *cmd)
{
	int ret, i;
	struct scsi_cmnd *scsi_cmd;
	struct io_stub   *stub;
	struct mpl_path  *path;
	struct Scsi_Host *shost;
	struct Scsi_Host *abort_shost;
	unsigned long flags;
	struct mpl_host *host;

	/* lock is held when called by SCSI layer */
	shost = cmd->device->host;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_unlock_irq(shost->host_lock);
#endif

	mpl_module_lock();
	mpl_print( 0, MPL_ERR "mpl_abort_handler:%p, %d/%d/%d \n",
		  cmd, mpl_host_no_from_cmd(cmd),
		  mpl_target_no_from_cmd(cmd),
		  mpl_lun_no_from_cmd(cmd));
	
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub = mpl_match_cmnd_remove_stub(cmd, MPL_ABORT_EXACT);

	mpl_print(MPL_TMO, MPL_INFO "mpl_abort_handler:%p, %d/%d/%d \n",
		  cmd, mpl_host_no_from_cmd(cmd),
		  mpl_target_no_from_cmd(cmd),
		  mpl_lun_no_from_cmd(cmd));
	if (stub) {
		spin_unlock_irqrestore(&stub_queue_lock, flags);

		/* Switch to active cmnd */
		cmd = &stub->cmd;
		cmd->result = ScsiResult(DID_ERROR, 0);
		scsi_cmd = stub->cmdp;

		mpl_print(MPL_IOPATH, MPL_ERR
			"ABORT pending IO %p, %d/%d/%d stub %p\n",
			cmd, mpl_host_no_from_cmd(cmd),
			mpl_target_no_from_cmd(cmd),
			mpl_lun_no_from_cmd(cmd), stub);
#ifdef MPL_STUB_SANITY
		if ((stub->flag & MPL_INTERNAL_CMD_STUB) || !scsi_cmd) {
			mpl_print(0, MPL_ERR "Abort Internal Cmd from SCSI %p, "
		  			"%s %s, cmd = %02x %02x %02x %02x\n",
       			stub,
       			mpl_print_device(stub->path->device),
       			mpl_print_path(stub->path),
       			stub->cmd.cmnd[0], stub->cmd.cmnd[1],
       			stub->cmd.cmnd[2], stub->cmd.cmnd[3]);
		}
#endif
		mpl_call_done_free_stub(stub);
		mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(shost->host_lock);
#endif
		ret = SUCCESS;
	} else {
		mpl_print(MPL_IOPATH, MPL_ERR
			"ABORT outstanding IO %p, %d/%d/%d stub %p\n",
			cmd, mpl_host_no_from_cmd(cmd),
			mpl_target_no_from_cmd(cmd),
			mpl_lun_no_from_cmd(cmd), stub);

		/* Wait for the deferral thread to complete all the aborted
		 * commands. We're holding the module_lock, which blocks irqs.
		 * So stub_queue_lock needs to either leave irq settings alone
		 * or do lock_irqsave, unlock_irq_restore.
		 */
		i = 0;
		while (!list_empty(&deferral_queue)) {
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_module_unlock();
			set_current_state(TASK_UNINTERRUPTIBLE);
			schedule_timeout(HZ);
			mpl_module_lock();
			spin_lock_irqsave(&stub_queue_lock, flags);
			if (++i >= 30)
				break;
		}

		/* Command is owned by lpfc driver. We need to issue abort to
		 *  the lpfc driver and then restore the original sdev
		 *  structure. mpl_done can be called while abort is
		 *  pending. However, mpl_done is not guaranteed. We need to
		 *  protect against this. */

		stub = mpl_find_scsi_cmd_in_lpfcq(cmd, MPL_SCSI_CMND);
		if (!stub) {
			/* Check to see if its a SCSI scan Inquiry P0
			 * with a delayed response.
			 */
			stub = mpl_find_scsi_delayed_inqcmd(cmd);
			if (!stub) {
				spin_unlock_irqrestore(&stub_queue_lock, flags);
				/* command got completed while abort pending. Do
				 * nothing.  return SUCCESS */
			
				host  = mpl_find_host_by_scsi_host(mpl_scsi_host_from_cmd(cmd));
				if (!host) {
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort_handler:Unable to find host \n");
					goto out ;
				}
				path = mpl_find_path(host, mpl_target_no_from_cmd(cmd),
				      		 mpl_lun_no_from_cmd(cmd));
				if (!path) {
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort command failed to find path %d/%d/%d\n",
			  		mpl_host_no_from_cmd(cmd),
			  		mpl_target_no_from_cmd(cmd),
			 		 mpl_lun_no_from_cmd(cmd));
					goto out ;
				}  
				if (path->sdev  && (path->sdev != cmd->device)) {
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort:Change SCSI device for %s %s: "
		  			"cmd sdev = %d/%d/%d\n",
		 			 mpl_print_device(path->device),
		  			mpl_print_path(path),
		  			cmd->device->host->host_no,
		  			cmd->device->id, 
		  			cmd->device->lun);

					if (!cmd->device) {
						mpl_print(MPL_TMO, MPL_INFO  "mpl_abort:cmd device is null \n");
						goto out;
					}
				}

				path->sdev = cmd->device;
#ifdef MPL_DBL_FREE
				BUG_ON(cmd->host_scribble == (void *)0x8585 );
#endif
				if (!cmd->host_scribble) {
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort: SCSI cmnd already completed: %s\n",
			  			mpl_print_path(path));
					goto out;
				}
				/* If path is exported from MultiPulse, go directly to lpfc driver */
				if (path->configured == CONFIG_UNAVAILABLE) {
					mpl_module_unlock();
					abort_shost = cmd->device->host;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
					spin_lock_irq(abort_shost->host_lock);
					ret = mpl_base_abort_handler(cmd);
					spin_unlock_irq(abort_shost->host_lock);
#else
					ret = mpl_base_abort_handler(cmd);
#endif
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
						spin_lock_irq(shost->host_lock);
#endif
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort: issue abort on UNAVAILABLE PATH %s ret %d \n",
						mpl_print_path(path), ret );
					return ret;
				 } else
					mpl_print(MPL_TMO, MPL_INFO "mpl_abort_handler : unable to find command on Path %s \n",
					  mpl_print_path(path));
					
			 out :
				mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
				spin_lock_irq(shost->host_lock);
#endif
				mpl_print(MPL_TMO, MPL_ERR "mpl_abort_handler unable to find cmd %p, %d/%d/%d \n",
		  			cmd, mpl_host_no_from_cmd(cmd),
		  			mpl_target_no_from_cmd(cmd),
		 			 mpl_lun_no_from_cmd(cmd));
				return SUCCESS;
			}
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			stub->path->delayed_cmd = NULL;
			stub->cmd.result = ScsiResult(DID_ERROR, 0);
			mpl_call_done_free_stub(stub);
			mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
			spin_lock_irq(shost->host_lock);
#endif
			return SUCCESS;
		}
		scsi_cmd = stub->cmdp;
		printk(KERN_ERR "mpl_abort_handler : command in lpfcq going to abort \n");
		/* Going to abort the command mpl_done should not handle this
		 * command's completion */
		BUG_ON(stub->reason == MPL_DO_INT_SCSI_TMO);
		stub->reason = MPL_DO_ABORT;
		path = stub->path;

		/* Switch to active cmnd */
		cmd = &stub->cmd;
		spin_unlock_irqrestore(&stub_queue_lock, flags);
#ifdef MPL_DBL_FREE
		BUG_ON(cmd->host_scribble == (void*)0x8585 );
#endif
		if (!cmd->host_scribble) {
			ret = SUCCESS;
			mpl_print(0, MPL_ERR "Abort: SCSI cmnd already completed: %s\n",
			  		mpl_print_path(path));
			goto skip_it;
		}
		mpl_module_unlock();
		abort_shost = cmd->device->host;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(abort_shost->host_lock);
		ret = mpl_base_abort_handler(cmd);
		spin_unlock_irq(abort_shost->host_lock);
#else
		ret = mpl_base_abort_handler(cmd);
#endif
		if (ret == FAILED) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
			spin_lock_irq(shost->host_lock);
#endif
			return FAILED;
		}

		mpl_module_lock();
skip_it:
		spin_lock_irqsave(&stub_queue_lock, flags);
		if ((stub = mpl_find_scsi_cmd_in_anyq(scsi_cmd, MPL_SCSI_CMND))) {
			if (stub->cmdp)
				stub->cmdp->host_scribble = NULL;
			switch (stub->state) {
			case STUB_LPFCQ:	/* On HBA lpfc queue */
				mpl_remove_stub_from_lpfcq(stub);
				mpl_stub_free(stub);
				if (!(atomic_read(&path->io_count))) {
					/* If there are no outstanding IOs on this path,
				 	* zero the avg time.
				 	*/
					atomic_set(&path->avg_cnt, 0);
					path->avg_time = 0;
				}
				break;
			case STUB_PENDQ:	/* On pending queue for device */
				mpl_remove_stub_from_pendq(stub);
				mpl_stub_free(stub);
				break;
			case STUB_DEFERQ:	/* Background processing for cmpl */
				mpl_remove_stub_from_deferral(stub);
				mpl_stub_free(stub);
				break;
			case STUB_EH_DEFERQ:	/* Background processing for timeout */
				mpl_print(0, MPL_ERR "Abort: Invalid SCSI cmnd on eh_deferral queue: %s\n",
			  		mpl_print_path(path));
				mpl_remove_stub_from_eh_deferral(stub);
				mpl_stub_free(stub);
				break;
			default:
				mpl_print(0, MPL_ERR "Abort: SCSI cmnd on unknown queue: %s\n",
			  		mpl_print_path(path));
				break;
			}
		}
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(shost->host_lock);
#endif
	}
	return ret;
}

int
mpl_device_reset_handler(struct scsi_cmnd *cmd)
{
	int ret, i, cnt;
	struct io_stub *stub;
	struct io_stub *defer_stub;
	struct mpl_path *path=NULL, *xmit_path;
	struct mpl_host *host=NULL;
	struct scsi_device *tmp_sdev;
	unsigned long flags;

	/* lock is held when called by SCSI layer */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_unlock_irq(cmd->device->host->host_lock);
#endif
	scsi_block_requests_wrapped(cmd->device->host);
 	set_current_state(TASK_UNINTERRUPTIBLE);
 	schedule_timeout(1);

	mpl_module_lock();

	host = mpl_find_host_by_scsi_host(cmd->device->host);

	if (!host) {
		mpl_module_unlock();
		scsi_unblock_requests_wrapped(cmd->device->host);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(cmd->device->host->host_lock);
#endif
		return FAILED;
	}

	path = mpl_find_path(host, cmd->device->id, cmd->device->lun);
	if (!path) {
		mpl_module_unlock();
		scsi_unblock_requests_wrapped(cmd->device->host);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(cmd->device->host->host_lock);
#endif
		return FAILED;
	}

	if (mpl_is_path_configured(path))
		xmit_path = path->device->last_path;
	else
		xmit_path = path;

	spin_lock_irqsave(&stub_queue_lock, flags);
	stub = mpl_match_cmnd_remove_stub(cmd, 
				MPL_ABORT_DEVICE | MPL_ABORT_IGNORE_ABORTED);
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	while (stub) {
		BUG_ON(stub->reason == MPL_DO_INT_SCSI_TMO) ;	 
		mpl_print(MPL_IOPATH, MPL_INFO "DEVICE RESET %p, %d/%d/%d stub %p\n",
		  cmd, mpl_host_no_from_cmd(cmd),
		  mpl_target_no_from_cmd(cmd),
		  mpl_lun_no_from_cmd(cmd), stub);
		if( !del_timer(&stub->cmd.eh_timeout)) {
			// Cmnd will handle by timer function
			mpl_print(MPL_TMO, MPL_INFO "mpl_device_reset: timer expires"
			"cmd_ptr x%p cmd x%x path %s \n",
			&stub->cmd, stub->cmd.cmnd[0],
			mpl_print_path(stub->path));
		} else
		{
			stub->cmd.result = ScsiResult(DID_ERROR, 0);
			mpl_call_done_free_stub(stub);
		}
		spin_lock_irqsave(&stub_queue_lock, flags);
		stub = mpl_match_cmnd_remove_stub(cmd,
			MPL_ABORT_DEVICE | MPL_ABORT_IGNORE_ABORTED);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	}
	tmp_sdev = cmd->device;
	cmd->device = xmit_path->sdev;
	mpl_module_unlock();

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(cmd->device->host->host_lock);
	ret = mpl_base_device_reset_handler(cmd);
	spin_unlock_irq(cmd->device->host->host_lock);
#else
	ret = mpl_base_device_reset_handler(cmd);
#endif

	mpl_module_lock();
	/* Restore the command fields */
	cmd->device = tmp_sdev;

	/* Wait for ALL IOs on this device to complete */
	i = 0;
	while(atomic_read(&xmit_path->device->io_count)) {
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		if (++i >= 60){
		  	mpl_print(MPL_IOPATH, MPL_WARNING  "mpl_device_reset: i/o count queue is not empty \n");
			break;
		}
	}

	/* Wait for the deferral thread to complete all the aborted commands.
	 * We're holding the module_lock, which blocks irqs.  So
	 * stub_queue_lock needs to either leave irq settings alone or do
	 * lock_irqsave, unlock_irq_restore.  We opt for the former. */
	i = 0;
	cnt = 0;
	spin_lock_irqsave(&stub_queue_lock, flags);
	while (!list_empty(&deferral_queue)) {
		list_for_each_entry(defer_stub, &deferral_queue, stub_queue) {
			if (defer_stub->flag &  MPL_INTERNAL_CMD_STUB)
				continue;
			cnt++;
		}
		if (!cnt)
			break;
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		spin_lock_irqsave(&stub_queue_lock, flags);
		if (++i >= 60){
		  mpl_print(MPL_IOPATH, MPL_WARNING  "mpl_device_reset: deferral_queue is not empty \n");
			break;
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	/* Wait for the eh_deferral thread to complete all the timed out
	 * commands.  We're holding the module_lock, which blocks irqs.  So
	 * stub_queue_lock needs to either leave irq settings alone or
	 * do lock_irqsave, unlock_irq_restore.  We opt for the former.*/
	i = 0;
	cnt = 0;
	spin_lock_irqsave(&stub_queue_lock, flags);
	while (!list_empty(&eh_deferral_queue)) {
		list_for_each_entry(defer_stub, &eh_deferral_queue, stub_queue) {
			if (defer_stub->flag &  MPL_INTERNAL_CMD_STUB)
				continue;
			cnt++;
		}
		if (!cnt)
			break;
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		spin_lock_irqsave(&stub_queue_lock, flags);
		if (++i >= 60){
		  mpl_print(MPL_IOPATH, MPL_WARNING "mpl_device_reset: eh_deferral_queue is not empty \n");
			break;
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	mpl_module_unlock();
	scsi_unblock_requests_wrapped(cmd->device->host);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(cmd->device->host->host_lock);
#endif
	return ret;
}

int
mpl_bus_reset_handler(struct scsi_cmnd *cmd)
{
	int ret, retcnt, i;
	struct io_stub *stub;
	struct scsi_device *tmp_sdev;
	struct mpl_host *host=NULL;
	unsigned long flags;
	
	if (driver_unloading) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		ret = mpl_base_bus_reset_handler(cmd);
#else
		ret = mpl_base_bus_reset_handler(cmd);
#endif
		return ret;
        }

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	/* lock is held when called by SCSI layer */
	spin_unlock_irq(cmd->device->host->host_lock);
#endif
	scsi_block_requests_wrapped(cmd->device->host);
	set_current_state(TASK_UNINTERRUPTIBLE);
	schedule_timeout(HZ);


	mpl_module_lock();

	host  = mpl_find_host_by_scsi_host(cmd->device->host);
	if (!host) {
		mpl_module_unlock();
		scsi_unblock_requests_wrapped(cmd->device->host);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		spin_lock_irq(cmd->device->host->host_lock);
#endif
		return FAILED;
	}

	tmp_sdev = cmd->device;

	retcnt = 0;
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub = mpl_match_cmnd_remove_stub(cmd,
				    MPL_ABORT_HOST | MPL_ABORT_IGNORE_ABORTED);
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	while (stub) {
		BUG_ON(stub->reason == MPL_DO_INT_SCSI_TMO) ;	 
		mpl_print(MPL_IOPATH, MPL_INFO "BUS RESET %p, %d/%d/%d stub %p\n",
		  cmd, mpl_host_no_from_cmd(cmd),
		  mpl_target_no_from_cmd(cmd),
		  mpl_lun_no_from_cmd(cmd), stub);
		if( !del_timer(&stub->cmd.eh_timeout)) {
			// Cmnd will handle by timer function
			mpl_print(MPL_TMO, MPL_INFO "mpl_bus_reset: timer expires"
			"cmd_ptr x%p cmd x%x path %s \n",
			&stub->cmd, stub->cmd.cmnd[0],
			mpl_print_path(stub->path));
		}
		else
		{
			// mpl_add_deferral(stub, MPL_ABORTED_CMD);
			stub->cmd.result = ScsiResult(DID_ERROR, 0);
			mpl_call_done_free_stub(stub);
		}
		spin_lock_irqsave(&stub_queue_lock, flags);
		stub = mpl_match_cmnd_remove_stub(cmd, 
				    MPL_ABORT_HOST | MPL_ABORT_IGNORE_ABORTED);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	}

#define MPL_NEW_BUS_RESET_METHOD 1
#ifdef MPL_NEW_BUS_RESET_METHOD
	{
	struct mpl_device *device;
	struct mpl_path *xmit_path;

	/* Loop thru all devices on this host */
	list_for_each_entry(device, &host->devices, devices) {
		xmit_path = device->last_path;
		if (xmit_path && xmit_path->sdev && xmit_path->host != host) {
			/* We need to issue a device reset */
			cmd->device = xmit_path->sdev;
			mpl_module_unlock();

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
			spin_lock_irq(xmit_path->sdev->host->host_lock);
			ret = mpl_device_reset_handler(cmd);
			spin_unlock_irq(xmit_path->sdev->host->host_lock);
#else
			ret = mpl_device_reset_handler(cmd);
#endif
			if (ret == SUCCESS)
				retcnt++;

			mpl_module_lock();
		}
	}

	/* Restore the command fields */
	cmd->device = tmp_sdev;
	}
#endif /* MPL_NEW_BUS_RESET_METHOD */

	mpl_module_unlock();

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(cmd->device->host->host_lock);
	ret = mpl_base_bus_reset_handler(cmd);
	spin_unlock_irq(cmd->device->host->host_lock);
#else
	ret = mpl_base_bus_reset_handler(cmd);
#endif
	mpl_module_lock();


	if (ret == SUCCESS)
		retcnt++;

	/* Wait for ALL IOs on this host to complete */
	i = 0;
	while (atomic_read(&host->io_count)) {
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		if (++i >= 60) {
			mpl_print(MPL_IOPATH, MPL_WARNING "IO queue is not empty \n");
			break;
		}
	}
	

 	/* Wait for the deferral thread to complete all the aborted commands.
	 * We're holding the module_lock, which blocks irqs.  So
	 * stub_queue_lock needs to either leave irq settings alone or do
	 * lock_irqsave, unlock_irq_restore.  We opt for the former. */
	i = 0;
	spin_lock_irqsave(&stub_queue_lock, flags);
	while (!list_empty(&deferral_queue)) {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		spin_lock_irqsave(&stub_queue_lock, flags);
		if (++i >= 60){
			mpl_print(MPL_IOPATH, MPL_WARNING "defereal queue is not empty \n");
			break;
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

 	/* Wait for the eh_deferral thread to complete all the timedout
	 * commands We're holding the module_lock, which blocks irqs.  So
	 * stub_queue_lock needs to either leave irq settings alone or
	 * do lock_irqsave, unlock_irq_restore.  We opt for the former. */
	i = 0;
	spin_lock_irqsave(&stub_queue_lock, flags);
	while (!list_empty(&eh_deferral_queue)) {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(HZ);
		mpl_module_lock();
		spin_lock_irqsave(&stub_queue_lock, flags);
		if (++i >= 60){
			mpl_print(MPL_IOPATH, MPL_WARNING "eh_defereal queue is not empty \n");
			break;
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	mpl_module_unlock();
	scsi_unblock_requests_wrapped(cmd->device->host);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	spin_lock_irq(cmd->device->host->host_lock);
#endif
	if (retcnt)
		return SUCCESS;
	else
		return FAILED;
}

/* 
 *  This function is called when all the low level hosts are detached from
 *  the multipulse driver. This function cleans up all the hosts, devices,
 *  and paths when a low level driver is detached.
 */
static void
mpl_destroy_all_hosts(void) 
{
	unsigned long flags;
	struct list_head *l, *n;
	struct mpl_host *host;
	struct mpl_path *path;
	struct mpl_target *tgtp;
	struct mpl_device *device, *tmp;
	struct io_stub *stub, *next_stub;
	int  i;

	/* First stop all hbeats */
	mpl_module_lock();
	list_for_each_entry(host, &mpl_hosts, host_list) {
		list_for_each_entry(device, &host->devices, devices) {
			list_for_each_entry(path, &device->paths, path_list) {
				mpl_hbeat_state_machine(path, HB_STOP);
			}
		}
	}
	list_for_each_entry(host, &mpl_hosts, host_list) {
		mpl_module_unlock();
		del_timer_sync(&host->host_hbeat);
		mpl_module_lock();
	}

	/* Cleanup any unprocessed mpl_thread entries */
	spin_lock_irqsave(&stub_queue_lock, flags);
	while((stub = mpl_get_first_deferral())) {
		mpl_stub_free(stub);
	}
	while((stub = mpl_get_first_eh_deferral())) {
		mpl_stub_free(stub);
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	mpl_module_unlock();

        /* Wait for internal IOs to cmpl / timeout if nesessary */
	for (i = 0; i < 600; ++i) {  /* 60 second max */
                if ((atomic_read(&total_internal_count)) == 0)
        		break;
		set_current_state(TASK_UNINTERRUPTIBLE);
		schedule_timeout(100);
	} 

	mpl_module_lock();

	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry(host, &mpl_hosts, host_list) {
		list_for_each_entry_safe(stub, next_stub, &host->lpfc_queue, stub_queue) {
			list_del(&stub->stub_queue);
			stub->state = STUB_DQ;
			/* free resources associated with stub */
			mpl_stub_free(stub);
		}
	}
	list_for_each_entry(host, &mpl_hosts, host_list) {
		list_for_each_entry_safe(stub, next_stub, &host->hbeat_deferral_queue, stub_queue) {
			list_del(&stub->stub_queue);
			stub->state = STUB_DQ;
			/* free resources associated with stub */
			mpl_stub_free(stub);
		}
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	
	if (device_hash) {
		spin_lock_irqsave(&device_hash_table_lock, flags);
		for (i = 0; i < device_hash_size; ++i) {
			struct mpl_path *path;
			struct mpl_device *next_dev;
			
			device = device_hash[i];
			spin_unlock_irqrestore(&device_hash_table_lock, flags);

			while (device) {
				mpl_device_state_machine(device, NULL, DEV_DISABLE);
				list_for_each_safe(l, n, &device->paths) {
					path = list_entry(l, struct mpl_path,
							  path_list);
					/* We don't really need to delink the
					 * path since we're going to free the
					 * list header when we
					 * kfree(device) 
					 */
					mpl_free_path(path);
				}
				next_dev = device->next;
				mpl_set_device_configured(device, CONFIG_UNCONFIGURED);
				list_del(&device->devices);
				mpl_module_unlock();
				del_timer_sync(&device->nopath);
				mpl_module_lock();
				kfree(device);
				device = next_dev;
			}
			spin_lock_irqsave(&device_hash_table_lock, flags);
			device_hash[i] = 0;
		}
		spin_unlock_irqrestore(&device_hash_table_lock, flags);
	}

	/* Double check to ensure reaper queue is empty */
	list_for_each_entry_safe(device, tmp, &reaper_queue, devices) {
		mpl_device_state_machine(device, NULL, DEV_DISABLE);
		list_for_each_safe(l, n, &device->paths) {
			struct mpl_path *path;
			path = list_entry(l, struct mpl_path, path_list);
			mpl_free_path(path);
		}
		mpl_set_device_configured(device, CONFIG_UNCONFIGURED);
		list_del(&device->devices);
		mpl_module_unlock();
		del_timer_sync(&device->nopath);
		mpl_module_lock();
		kfree(device);
	}

	list_for_each_safe(l, n, &mpl_targets) {
		tgtp = list_entry(l, struct mpl_target, target_list);
		list_del(&tgtp->target_list);
		mpl_module_unlock();
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
		del_timer_sync(&tgtp->dev_loss);
#endif
		mpl_module_lock();
		kfree(tgtp);
	}

	list_for_each_safe(l, n, &mpl_hosts) {
		host = list_entry(l, struct mpl_host, host_list);
		list_del(&host->host_list);
		mpl_module_unlock();
		del_timer_sync(&host->host_block);
		del_timer_sync(&host->host_scan);
		del_timer_sync(&host->host_hbeat);
		mpl_module_lock();
		kfree(host);
	}

	/* Cleanup any unprocessed mpl_thread entries */
	spin_lock_irqsave(&stub_queue_lock, flags);
	while((stub = mpl_get_first_deferral())) {
		mpl_stub_free(stub);
	}
	while((stub = mpl_get_first_eh_deferral())) {
		mpl_stub_free(stub);
	}
	while((stub = mpl_get_first_scsitmo_deferral())) {
		mpl_stub_free(stub);
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

	mpl_module_unlock();

	if (atomic_read(&total_internal_count)) {
		mpl_print(0, MPL_ERR "Internal I/O Pending on exit: %d\n",
			atomic_read(&total_internal_count));
	}
}

void 
mpl_timeout_blocked_host(unsigned long data)
{
	struct mpl_host *host = (struct mpl_host *)data;
	struct mpl_path *path;
	uint32_t hash;
	unsigned long flags;

	mpl_print(MPL_RECOVERY, MPL_INFO "mpl_timeout_blocked_host %d\n",
		  host->host_no);

	if (mpl_host_is_offline(host->hba)) {
		mpl_module_lock();
		host->blocked = 0;
		spin_lock_irqsave(&path_hash_table_lock, flags);
		for(hash=0; hash<path_hash_size; hash++) {
			for (path = path_hash[hash]; path; path = path->next) {
				if (path->host == host) {
					spin_unlock_irqrestore(&path_hash_table_lock, flags);
					mpl_path_state_machine(path, PATH_DOWN);
					spin_lock_irqsave(&path_hash_table_lock, flags);
				}
			}
		}
		spin_unlock_irqrestore(&path_hash_table_lock, flags);
		mpl_module_unlock();
		scsi_unblock_requests(host->scsi_host);
		mpl_wake_thread();
		return;
	}
	mpl_module_lock();
	mpl_module_unlock();
	mod_timer(&host->host_block, jiffies + HZ);
	return;
}

void
scsi_block_requests_wrapped(struct Scsi_Host * sh)
{
	struct mpl_host *host;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sh);
	mpl_module_unlock();
	if (!host) {
		return;
	}
	if (!host->blocked) {
		/* Wait 8 seconds to allow a HBAnyware offline request, followed
		 * immediately by an online request, time to complete.
		 */
		mod_timer(&host->host_block, jiffies + (8 * HZ));
		scsi_block_requests(sh);
	}
	host->blocked++;
	return;
}

void
scsi_unblock_requests_wrapped(struct Scsi_Host * sh)
{
	struct mpl_host *host;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sh);
	mpl_module_unlock();
	if (!host) {
		return;
	}
	if (host->blocked) {
		host->blocked--;
		if (!host->blocked) {
			del_timer_sync(&host->host_block);
			scsi_unblock_requests(sh);
			mpl_wake_thread();
		}
	}
	return;
}

/* Intercept 2.6 KERNEL calls */

struct Scsi_Host *
scsi_host_alloc_wrapped(struct scsi_host_template *tpnt, int sz)
{
	struct mpl_host *host;

	if (mpl_base_queuecommand == NULL) {
		driver_unloading = 0;
		mpl_template = tpnt;
		mpl_base_release = tpnt->release;
		tpnt->release = mpl_release;
		mpl_base_queuecommand = tpnt->queuecommand;
		tpnt->queuecommand = mpl_queuecommand;
		mpl_base_abort_handler = tpnt->eh_abort_handler;
		tpnt->eh_abort_handler = mpl_abort_handler;
		mpl_base_device_reset_handler = tpnt->eh_device_reset_handler;
		tpnt->eh_device_reset_handler = mpl_device_reset_handler;
		mpl_base_bus_reset_handler = tpnt->eh_bus_reset_handler;
		tpnt->eh_bus_reset_handler = mpl_bus_reset_handler;
		mpl_base_slave_alloc = tpnt->slave_alloc;
		tpnt->slave_alloc = mpl_slave_alloc;
		mpl_base_slave_config = tpnt->slave_configure;
		tpnt->slave_configure = mpl_slave_config;
		mpl_base_slave_destroy = tpnt->slave_destroy;
		tpnt->slave_destroy = mpl_slave_destroy;
	}

	mpl_module_lock();
	host = mpl_find_or_create_host(host_cnt);
	mpl_module_unlock();
	if (!host) {
		return NULL;
	}
	host->scsi_host = scsi_host_alloc(tpnt, sz);
	host->ref_count = 0;
	host_cnt++;
	return host->scsi_host;
}

int 
scsi_add_host_wrapped(struct Scsi_Host *shost, 
		      struct device *dev)
{
	struct mpl_host  *host;

	mpl_module_lock();
	host = mpl_find_host_by_scsi_host(shost);
	if (!host) {
		mpl_print(0, MPL_ERR "scsi_add_host_wrapped cannot find host for" 
			  "shost %p host_no %d \n",
			  shost, shost->host_no);
	}

	if (host) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
		host->hba = (void *)shost->hostdata[0];
#else
		host->hba = (void *)shost->hostdata;
#endif
	}
	mpl_module_unlock();
	return scsi_add_host(shost, dev);
}

void
scsi_remove_host_wrapped(struct Scsi_Host *sh)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	struct mpl_host *host;
	unsigned long flags;
	struct Scsi_Host *shost;
	struct scsi_device *sdev, *tmp;
	struct mpl_path *prsn_path;
	static int run_once = 0;
	/* 
	   The lpfc driver is going away. Remove all the scsi_devices exported 
	   by mpulse driver now. Removing scsi_devices after some of hosts
	   are dettached can cause  SCSI_CACHE_SYNC command to fail if
	   presentation path is not available.
	*/	
	mpl_module_lock();
	if (!driver_unloading && !run_once ) {
		run_once++ ;
		list_for_each_entry(host, &mpl_hosts, host_list) {
			shost = host->scsi_host;
			mpl_module_unlock();
			spin_lock_irqsave(shost->host_lock, flags);
			list_for_each_entry_safe(sdev, tmp, &shost->__devices, siblings) {
				spin_unlock_irqrestore(shost->host_lock, flags);
				if (sdev->sdev_state == SDEV_BLOCK) {
					mpl_module_lock();
					prsn_path = mpl_find_path(host, sdev->id,
								  sdev->lun);
					mpl_module_unlock();
					if (prsn_path)
						del_timer_sync(&prsn_path->sdev_block);
					scsi_internal_device_unblock(sdev);
				}
				scsi_remove_device(sdev);
				spin_lock_irqsave(shost->host_lock, flags);
			}
			spin_unlock_irqrestore(shost->host_lock, flags);
			mpl_module_lock();
		}
	}
	mpl_module_unlock();
#endif
	scsi_remove_host(sh);
	return;
}

void
scsi_host_get_wrapped(struct Scsi_Host *sh)
{
	struct mpl_host  *host;

	mpl_module_lock();
	host = mpl_find_host_by_scsi_host(sh);
	if (!host) {
		mpl_print(0, MPL_ERR "scsi_host_get_wrapped cannot find host for" 
			  " shost %p host_no %d \n",
			  sh, sh->host_no);
		goto out;
	}
	host->ref_count++;
out:
	mpl_module_unlock();
	scsi_host_get(sh);
	return;
}

void
scsi_host_put_wrapped(struct Scsi_Host *sh)
{
	struct mpl_host *host;
	struct mpl_device *device;
	struct mpl_path *path;

	mpl_module_lock();
	host = mpl_find_host_by_scsi_host(sh);
	if (!host) {
		mpl_print(0, MPL_ERR "scsi_host_put_wrapped cannot find host for" 
			  " shost %p host_no %d \n",
			  sh, sh->host_no);
		mpl_module_unlock();	
		goto out;
	}
	if(host->ref_count >= 1) {
		host->ref_count--;
		mpl_module_unlock();	
		scsi_host_put(sh);
		return;
	}
	/* Lets cancel all our timers before freeing scsi_host */
	list_for_each_entry(device, &host->devices, devices) {
		mpl_device_state_machine(device, NULL, DEV_DISABLE);
		list_for_each_entry(path, &device->paths, path_list) {
			mpl_path_state_machine(path, PATH_DISABLE);
			mpl_set_path_configured(path, CONFIG_UNCONFIGURED);
		}
		mpl_set_device_configured(device, CONFIG_UNCONFIGURED);
		mpl_quiesce_device(device, 30);
	}
	mpl_module_unlock();	
	del_timer_sync(&host->host_hbeat);

out:
	scsi_host_put(sh);
	mpl_module_lock();
	host_cnt--;
	/* 
	   If all the hosts are unregistered the lpfc driver is unloaded.
	   Delete all the hosts/paths/devices and reset mpl_base_queuecommand
	*/
	if (host_cnt == 0) {
		mpl_module_unlock();
		mpl_base_queuecommand = NULL;
		mpl_destroy_all_hosts();
	} else {
		mpl_module_unlock();
	}
	return;
}

static int
mpl_get_path_by_sdev(struct mpl_path **pathp, struct scsi_device *sdev)
{
	struct mpl_host *host;
	struct mpl_path *path;
	int configured = 0;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sdev->host);
	if (host) {
		path = mpl_find_path(host, sdev->id, sdev->lun);
		if (!path) {
			mpl_module_unlock();
			return configured;
		}
		*pathp = path;
		configured = mpl_is_path_configured(path);

		if (path->sdev  && (path->sdev != sdev))
			mpl_print(MPL_IOPATH, MPL_INFO
			  "Change SCSI device for %s %s: "
			  "path sdev = %d/%d/%d\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  sdev->host->host_no,
			  sdev->id, 
			  sdev->lun);
	}
	mpl_module_unlock();
	return configured;
}

int
scsi_track_queue_full_wrapped(struct scsi_device *sdev, int cnt)
{
	struct mpl_path *path = NULL;
	int configured = 0;
	int ret = cnt;

	configured = mpl_get_path_by_sdev(&path, sdev);
	if  (!configured) {
		ret = scsi_track_queue_full(sdev, cnt);
	} else {
		struct mpl_path *presentation = mpl_first_path(path->device);

		if (presentation && presentation->sdev) {
			ret = scsi_track_queue_full(presentation->sdev, cnt);
		}
		else {
			ret = scsi_track_queue_full(sdev, cnt);
		}
	}
	return ret;
}

void
scsi_adjust_queue_depth_wrapped(struct scsi_device *sdev, int tag, int cnt)
{
	struct mpl_path *path = NULL;
	int configured = 0;

	configured = mpl_get_path_by_sdev(&path, sdev);
	if  (!configured) {
		scsi_adjust_queue_depth(sdev, tag, cnt);
	} else {
		struct mpl_path *presentation = mpl_first_path(path->device);

		if (presentation && presentation->sdev) {
			scsi_adjust_queue_depth(presentation->sdev, tag, cnt);
		}
		else {
			scsi_adjust_queue_depth(sdev, tag, cnt);
		}
	}
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
static void 
mpl_timeout_blocked_sdev(unsigned long data)
{
	struct scsi_device *sdev = (struct scsi_device *)data;
	mpl_print(0, MPL_ERR "mpl_timeout_blocked_sdev : blocked target timed out <%d/%d/%d> \n",
		  sdev->host->host_no, sdev->id, sdev->lun);
	scsi_internal_device_unblock(sdev);
}

static int 
mpl_path_block(struct mpl_path *path, int tmo)
{
	struct mpl_path    *prsn_path;
	struct scsi_device *prsn_sdev=NULL, *tmp_sdev;
	struct timer_list *timer;
	enum mpl_device_state prev_state;

	mpl_module_lock();
	prev_state = path->device->state;
	path->lld_blocked = 1;
	mpl_expedite_hbeat(path); 
	mpl_module_unlock();

	/* Find the presentation path of this device */
	prsn_path = list_entry(list_first(&path->device->paths),
			       struct mpl_path,
			       path_list);

	/* Find the SCSI device for the presentation path */
	shost_for_each_device(tmp_sdev, prsn_path->host->scsi_host) {
		if ((tmp_sdev->id == prsn_path->target) &&
		    (tmp_sdev->lun == prsn_path->lun)) {
			prsn_sdev = tmp_sdev;
			scsi_device_put(prsn_sdev);
			break;
		}
	}

	if (!prsn_sdev) {
		mpl_print(0, MPL_ERR "mpl_device_block cannot find scsi device for path <%d/%d/%d>\n",
			  prsn_path->host->scsi_host->host_no, prsn_path->target, prsn_path->lun);
		return 0;
	}
	/* 
	   If multipulse driver is able to find another path do not
	   block. Else block the queue.
	*/
	if ((path->device->state == DEV_FAILED) &&
	    (prev_state != DEV_FAILED) &&
	    (prsn_sdev != NULL) &&
	    (prsn_sdev->sdev_state != SDEV_BLOCK)) {
		/* The lpfc driver blocks this target for the timeout period */
		scsi_internal_device_block(prsn_sdev);
		timer = &prsn_path->sdev_block;
		timer->data = (unsigned long)prsn_sdev;
		mod_timer(timer, jiffies + tmo * HZ);
	}
	return 0;
}

static int
mpl_path_unblock(struct mpl_path *path)
{
	mpl_module_lock();
	path->lld_blocked = 0;

	mpl_expedite_hbeat(path);
	mpl_module_unlock();
	return 0;
}

void
mpl_scsi_device_unblock(struct mpl_device *device)
{
	struct mpl_path    *prsn_path;
	struct scsi_device *prsn_sdev=NULL, *tmp_sdev;

	mpl_module_unlock();
	/* Find the presentation path of this device */
	prsn_path = list_entry(list_first(&device->paths),
			       struct mpl_path,
			       path_list);

	/* Find the SCSI device for the presentation path */
	shost_for_each_device(tmp_sdev, prsn_path->host->scsi_host) {
		if ((tmp_sdev->id == prsn_path->target) &&
		    (tmp_sdev->lun == prsn_path->lun)) {
			prsn_sdev = tmp_sdev;
			scsi_device_put(prsn_sdev);
			break;
		}
	}

	if (!prsn_sdev) {
		mpl_print(0, MPL_ERR "mpl_device_unblock cannot find scsi device for path <%d/%d/%d>\n",
			  prsn_path->host->scsi_host->host_no, prsn_path->target, prsn_path->lun);
		mpl_module_lock();
		return ;
	}

	if (prsn_sdev->sdev_state == SDEV_BLOCK) {
		mpl_module_lock();
		mpl_module_unlock();
		del_timer_sync(&prsn_path->sdev_block);
		scsi_internal_device_unblock(prsn_sdev);
	}
	mpl_module_lock();
	return ;
}

#if defined(RHEL_FC)
static int 
mpl_device_block(struct device *dev, void *data)
{
	struct scsi_device *sdev = to_scsi_device(dev);
	struct scsi_target *st  = (struct scsi_target *) data;
	int timeout = fc_starget_dev_loss_tmo(st);
	struct mpl_path *path;
	struct mpl_host *host;
	uint32_t hash;
	unsigned long flags;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sdev->host);
	if (!host) {
		mpl_print(0, MPL_ERR "mpl_device_block cannot find host <%d/%d/%d> \n",
			  sdev->host->host_no, sdev->id, sdev->lun);
		mpl_module_unlock();
		return 1;
	}

	spin_lock_irqsave(&path_hash_table_lock, flags);
	for(hash=0; hash<path_hash_size; hash++) {
		for (path = path_hash[hash]; path; path = path->next) {
			if (path->host == host &&
			    path->target  == sdev->id)
			{
				spin_unlock_irqrestore(&path_hash_table_lock, 
							flags);
				mpl_module_unlock();
				mpl_path_block(path, timeout);
				mpl_module_lock();
				spin_lock_irqsave(&path_hash_table_lock, 
						   flags);
			}
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	mpl_module_unlock();

	return 1;
}

void
fc_target_block_wrapped(struct scsi_target *st)
{
	device_for_each_child(&st->dev, st, mpl_device_block);
	return;
}
#else /* not RHEL_FC -> is SLES_FC */
void
fc_target_block_wrapped(struct Scsi_Host *sh,  uint16_t id,
			struct timer_list *tmr, uint32_t tmo)
{
	struct mpl_host *host;
	struct mpl_path *path;
	uint32_t hash;
	unsigned long flags;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sh);
	if (!host) {
		mpl_print(0, MPL_ERR "fc_target_block_wrapped cannot find host <%d/%d> \n",
			  sh->host_no, id);
		mpl_module_unlock();
		return ;
	}
	spin_lock_irqsave(&path_hash_table_lock, flags);
	for(hash=0; hash<path_hash_size; hash++) {
		for (path = path_hash[hash]; path; path = path->next) {
			if (path->host == host &&
			    path->target  == id)
			{
				spin_unlock_irqrestore(&path_hash_table_lock, 
							flags);
				mpl_module_unlock();
				mpl_path_block(path, tmo);
				mpl_module_lock();
				spin_lock_irqsave(&path_hash_table_lock, 
						   flags);
			}
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	mpl_module_unlock();

	return;
}

#endif

#if defined(RHEL_FC)

static int
mpl_device_unblock(struct device *dev, void *data)
{
	struct scsi_device *sdev = to_scsi_device(dev);
	struct mpl_path *path;
	struct mpl_host *host;
	uint32_t hash;
	unsigned long flags;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sdev->host);

	if (!host) {
		mpl_print(0, MPL_ERR "mpl_device_unblock cannot find host <%d/%d/%d> \n",
			sdev->host->host_no, sdev->id, sdev->lun);
		mpl_module_unlock();
		return 1;
	}

	spin_lock_irqsave(&path_hash_table_lock, flags);
	for(hash=0; hash<path_hash_size; hash++) {
		for (path = path_hash[hash]; path; path = path->next) {
			if (path->host == host &&
			    path->target  == sdev->id)
			{
				spin_unlock_irqrestore(&path_hash_table_lock, 
							flags);
				mpl_module_unlock();
				mpl_path_unblock(path);
				mpl_module_lock();
				spin_lock_irqsave(&path_hash_table_lock, 
						   flags);
			}
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	mpl_module_unlock();
        return 1;
}

void
fc_target_unblock_wrapped(struct scsi_target *st)
{
	mpl_module_lock();
	if (!driver_unloading) {
		mpl_module_unlock();
		device_for_each_child(&st->dev, st, mpl_device_unblock);
		mpl_wake_thread();
	} else
		mpl_module_unlock();
	return;
}
#else /* not RHEL_FC -> is SLES_FC */

void
fc_target_unblock_wrapped(struct Scsi_Host *sh, uint16_t id,
	struct timer_list *tmr)
{
	struct mpl_host *host;
	struct mpl_path *path;
	uint32_t hash;
	unsigned long flags;

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sh);

	if (!host) {
		mpl_print(0, MPL_ERR "fc_target_unblock_wrapped cannot find host <%d/%d> \n",
			sh->host_no, id);
		mpl_module_unlock();
		return ;
	}

	spin_lock_irqsave(&path_hash_table_lock, flags);
	for(hash=0; hash<path_hash_size; hash++) {
		for (path = path_hash[hash]; path; path = path->next) {
			if (path->host == host &&
			    path->target  == id)
			{
				spin_unlock_irqrestore(&path_hash_table_lock, 
							flags);
				mpl_module_unlock();
				mpl_path_unblock(path);
				mpl_module_lock();
				spin_lock_irqsave(&path_hash_table_lock, 
						   flags);
			}
		}
	}
	spin_unlock_irqrestore(&path_hash_table_lock, flags);
	mpl_module_unlock();

	return;
}

#endif
#else /* 2.6.16 or better */

struct fc_rport *
fc_remote_port_add_wrapped(struct Scsi_Host *sh,  int chan,
	struct fc_rport_identifiers *rpid)
{
	struct mpl_host   *host;
	struct mpl_target *tgtp;
	struct fc_rport   *rport;
	struct mpl_path   *pathp;
	struct io_stub    *stub, *devloss_stub;
	unsigned long flags;
	int rescan;
	uint8_t wwpn[8];
	memset( wwpn, 0, (sizeof( uint8_t) * 8));

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(sh);
	tgtp = mpl_find_target_wwpn(host, rpid->port_name);

	if (tgtp == 0) {
		mpl_module_unlock();
		rport = fc_remote_port_add(sh, chan, rpid);
		u64_to_wwn(rpid->port_name, wwpn) ;
		mpl_print(MPL_IOPATH, MPL_INFO "Rport add event for target %x:%x:%x:%x:%x:%x:"
			"%x:%x\n", wwpn[0], wwpn[1],wwpn[2],wwpn[3],wwpn[4],wwpn[5],wwpn[6],
			wwpn[7] );
	}
	else {
		mpl_print(MPL_IOPATH, MPL_INFO "Pseudo rport add for %s\n",
	  		mpl_print_target(tgtp));

		mpl_module_unlock();
		del_timer_sync(&tgtp->dev_loss);
		mpl_module_lock();
		/* 
		 * Since Link up event happened, delete devloss timeout  stub if exists in 
		 * eh_deferral queue.
		 */
		spin_lock_irqsave(&stub_queue_lock, flags);
		devloss_stub = mpl_find_devloss_tmo_in_eh_deferral(rpid->port_name) ;
		if (devloss_stub) {
			mpl_remove_stub_from_eh_deferral(devloss_stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_stub_free(devloss_stub);
		} else
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			
		rport =  tgtp->rport;
		if (rport) {
			/* Now, restore the old values */
			if (tgtp->roles)
				rport->roles = tgtp->roles;
			if (tgtp->port_state)
				rport->port_state = tgtp->port_state;
			rport->port_id = rpid->port_id;
			mpl_do_set_ndlp(host->hba, tgtp->id, rport->dd_data);
			tgtp->roles = 0;
			tgtp->port_state = 0;
		}

		rescan = 0;
		list_for_each_entry(pathp, &tgtp->paths, tgt_path_list) {
			if (!pathp->sdev) {

				/* If the sdev disappeared, we need to
				 * get one back.
				 */
				rescan++;
				if((stub = mpl_stub_alloc(0, pathp, 0))) {
					stub->state = STUB_DQ;
					stub->flag |= MPL_INTERNAL_CMD_STUB;
					spin_lock_irqsave(&stub_queue_lock,
									flags);
					stub->reason = MPL_DO_DEV_SCAN;
					mpl_add_stub_to_eh_deferral(stub);
					spin_unlock_irqrestore(&stub_queue_lock,
									flags);
				}
			}
			if (mpl_is_path_configured(pathp)) {
				pathp->lld_blocked = 0;
				mpl_path_state_machine(pathp, PATH_UP);
			}
		}
		mpl_module_unlock();
		if (rescan) {
			mpl_wake_eh_thread();
		}
	}
	return rport;
}

void
fc_remote_port_delete_wrapped(struct fc_rport *rport)
{
	struct Scsi_Host  *shost = rport_to_shost(rport); 
	struct mpl_host   *host;
	struct mpl_target *tgtp;
	struct mpl_path   *pathp;
	uint8_t wwpn[8];

	memset( wwpn, 0, (sizeof( uint8_t) * 8));

	mpl_module_lock();
	host  = mpl_find_host_by_scsi_host(shost);

	tgtp = mpl_find_target_wwpn(host, rport->port_name);

	if (tgtp == 0) {
		mpl_module_unlock();
		u64_to_wwn(rport->port_name, wwpn) ;
		mpl_print(MPL_IOPATH, MPL_INFO "fc_remote_port_delete target  %x:%x:%x:%x:%x:"
			"%x:%x:%x\n", wwpn[0], wwpn[1],wwpn[2],wwpn[3],wwpn[4],wwpn[5],wwpn[6],
			wwpn[7] );
		fc_remote_port_delete(rport);
	}
	else {
		mpl_print(MPL_IOPATH, MPL_INFO "Pseudo rport delete for  target %s\n",
	  		mpl_print_target(tgtp));

		/* First, save the old values */
		tgtp->roles = rport->roles;
		tgtp->port_state = rport->port_state;
		mpl_module_unlock();
		spin_lock_irq(shost->host_lock);
		rport->port_state = FC_PORTSTATE_BLOCKED;
		spin_unlock_irq(shost->host_lock);

	 	mod_timer(&tgtp->dev_loss,
 			jiffies + (mpl_get_hba_nodev_tmo(host->hba) * HZ));
		mpl_module_lock();

		list_for_each_entry(pathp, &tgtp->paths, tgt_path_list) {
			if (mpl_is_path_configured(pathp)) {
				pathp->lld_blocked = 1;
				mpl_expedite_hbeat(pathp); 
			}
		}
		mpl_module_unlock();
	}
	return;
}
#endif

struct scsi_transport_template *
fc_attach_transport_wrapped(struct fc_function_template *fcfunc)
{
	struct scsi_transport_template *tmp;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	mpl_base_devloss_tmo = fcfunc->dev_loss_tmo_callbk;
#endif
	tmp = fc_attach_transport(fcfunc);
	return tmp;
}

static int __init
mpl_init(void)
{
	int  rv = -ENOMEM;

	printk(KERN_ERR "Emulex MultiPulse module version %s\n",
	       MPL_MODULE_VERSION);
	printk(KERN_ERR MPL_COPYRIGHT "\n");
	mpl_base_queuecommand = NULL;
	mpl_base_release = NULL;

	memset(&mpl_internal_req, 0, sizeof(mpl_internal_req));
	/* Initialize spinlocks */
	spin_lock_init(&mpl_module_spin_lock);
	spin_lock_init(&device_hash_table_lock);
	spin_lock_init(&path_hash_table_lock);
	spin_lock_init(&stub_queue_lock);

	/* Verify configurable hbeat parameters */
	if ((mpl_hbeat_interval == 0) ||
	    (mpl_hbeat_interval > MPL_MAX_HBEAT_INTERVAL)) {
		mpl_print(0, MPL_ERR "mpl_hbeat_interval (%d)"
		   "is out of range [1-%d], resetting to default (%d)\n", 
		   mpl_hbeat_interval,
		   MPL_MAX_HBEAT_INTERVAL,  MPL_DFT_HBEAT_INTERVAL);
		mpl_hbeat_interval = MPL_DFT_HBEAT_INTERVAL;
	}
	if ((mpl_hbeat_fail_interval == 0) ||
	    (mpl_hbeat_fail_interval > MPL_MAX_HBEATFAIL_INTERVAL)) {
		mpl_print(0, MPL_ERR "mpl_hbeat_fail_interval (%d)"
		   "is out of range [1-%d], resetting to default (%d)\n", 
		   mpl_hbeat_fail_interval,
		   MPL_MAX_HBEATFAIL_INTERVAL,  MPL_DFT_HBEATFAIL_INTERVAL);
		mpl_hbeat_fail_interval = MPL_DFT_HBEATFAIL_INTERVAL;
	}
	if ((mpl_max_hbeat_retry == 0) ||
	    (mpl_max_hbeat_retry > MPL_MAX_HBEAT_RETRY)) {
		mpl_print(0, MPL_ERR "mpl_max_hbeat_retry (%d)"
		   "is out of range [1-%d], resetting to default (%d)\n", 
		   mpl_max_hbeat_retry,
		   MPL_MAX_HBEAT_RETRY,  MPL_DFT_HBEAT_RETRY);
		mpl_max_hbeat_retry = MPL_DFT_HBEAT_RETRY;
	}
	if(mpl_hbeat_fail_interval > mpl_hbeat_interval) {
		mpl_print(0, MPL_ERR "mpl_hbeat_fail_interval (%d)"
		   "is > mpl_hbeat_interval (%d), resetting to default (%d)\n",
		    mpl_hbeat_fail_interval, mpl_hbeat_interval,
		    MPL_DFT_HBEATFAIL_INTERVAL);
		mpl_hbeat_fail_interval = MPL_DFT_HBEATFAIL_INTERVAL;
	}

				/* truncate hash_sizes to power of 2 */
	while (device_hash_size & (device_hash_size - 1)) {
		device_hash_size = device_hash_size & (device_hash_size - 1);
	}
	while (path_hash_size & (path_hash_size - 1)) {
		path_hash_size = path_hash_size & (path_hash_size - 1);
	}
	device_hash = kmalloc(device_hash_size * sizeof(*device_hash),
			      GFP_KERNEL);
	if (!device_hash)
		goto out;
	memset(device_hash, 0, device_hash_size * sizeof(*device_hash));
	path_hash = kmalloc(path_hash_size * sizeof(*path_hash),
			      GFP_KERNEL);
	if (!path_hash)
		goto out_free_device_hash;
	memset(path_hash, 0, path_hash_size * sizeof(*path_hash));
	rv = mpl_create_mem_pool();
	if (rv)
		goto out_free_path_hash;

	rv = mpl_create_proc();
	if (rv)
		goto out_free_cache;

	mpl_setup_multipulse();
	kernel_thread(mpl_thread, NULL, 0);
	kernel_thread(mpl_eh_thread, NULL, 0);

	goto out;

out_free_cache:
	mpl_destroy_mem_pool();
out_free_path_hash:
	kfree(path_hash);
	path_hash = NULL;
out_free_device_hash:
	kfree(device_hash);
	device_hash = NULL;
out:
	return rv;
}

static void __exit
mpl_exit(void)
{
	struct io_stub *stub;
	unsigned long flags;

	mpl_thread_quit = 1;
	mpl_wake_thread();
	down(&mpl_thread_gone);

	mpl_eh_thread_quit = 1;
	mpl_wake_eh_thread();
	down(&mpl_eh_thread_gone);

	driver_unloading = 1;
        mpl_destroy_all_hosts();

	mpl_module_lock();

	if (device_hash) {
                kfree(device_hash);
                kfree(path_hash);
		device_hash = NULL;
	}

	mpl_remove_proc(); 

	/* Cleanup any unprocessed mpl_thread entries */
	spin_lock_irqsave(&stub_queue_lock, flags);
	while((stub = mpl_get_first_deferral())) {
		mpl_stub_free(stub);
	}
	while((stub = mpl_get_first_eh_deferral())) {
		mpl_stub_free(stub);
	}
	while((stub = mpl_get_first_scsitmo_deferral())) {
		mpl_stub_free(stub);
	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	mpl_module_unlock();

	mpl_destroy_mem_pool();
	mpl_free_initial_data();
	return;
}

#ifdef MPL_SCSICMD_SANITY
int is_mid_layer_timer_pending( struct io_stub *stub )
{
	int ret = 5 ;
	if (stub && (stub->cmdp->eh_timeout.function != NULL ))   
		return timer_pending(&stub->cmdp->eh_timeout);
	else {
		if (stub )
		 printk (KERN_ERR "is_mid_layer: scsi cmd x%x cmd x%x stub reason %d stub _flag %x state x%x \n",
			stub->cmdp->cmnd[0],stub->cmd.cmnd[0], stub->reason, stub->flag,stub->state );
		else
		 printk (KERN_ERR "is_mid_layer: stub is null \n");

		return ret ;
	}
}
void 
mpl_dump_all_scsi_cmd(void )
{
	struct io_stub *stub;
	unsigned long flags;
	struct mpl_host *host;
	struct mpl_device *device ;
	struct Scsi_Host *scsi_host ;
	static int x = 0 ;
	int i ;
	if (x < 10 ) {
	  printk(KERN_ERR "mpl_dump_all_scsi_cmd  \n");
	  x++ ;
	}
	list_for_each_entry(stub, &scsiglobal_queue, global_queue) {
		printk(KERN_ERR "GLOBAL QUEUE stubp %p state x%x reason x%x \n",
			stub, stub->state, stub->reason);
		for (i = 0 ; i < 20 ; i ++ ) {
		  printk(KERN_ERR "mpl_dump history reason x%x state x%x \n",
			stub->history[i], stub->history[i+1] );
			i++ ;	
		}
	}	
	spin_lock_irqsave(&stub_queue_lock, flags);
	list_for_each_entry(stub, &deferral_queue, stub_queue ){
		if (stub->flag & MPL_INTERNAL_CMD_STUB)
			continue ;
	 	else 
	  		 printk(KERN_ERR "DEFERALQ:scsi_cmd x%p Cmnd x%x Path %s timer %d reason %d \n",
			stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path),is_mid_layer_timer_pending(stub),
			 stub->reason );
 	
	}
	list_for_each_entry(stub, &eh_deferral_queue, stub_queue ){
	 if (stub->flag & MPL_INTERNAL_CMD_STUB)
		continue ;
	 else 
	   printk(KERN_ERR "EH_DEFERALQ:scsi_cmd x%p Cmnd x%x Path %s timer %d  reason %d \n",
		stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path),is_mid_layer_timer_pending(stub),
		 stub->reason );
 	
	}
	list_for_each_entry(stub, &int_scsi_tmo_queue, stub_queue ){
	 if (stub->flag & MPL_INTERNAL_CMD_STUB) {
	   printk(KERN_ERR "SCSI_TMO_Q:NEED TO FIND MORE ## Cmnd x%x Path %s reason %d \n",
		stub->cmd.cmnd[0],mpl_print_path(stub->path),stub->reason );
		
		continue ;
	 }
	 else 
	   printk(KERN_ERR "SCSI_TMO_Q:scsi_cmd x%p Cmnd x%x Path %s timer %d reason %d \n",
		stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path),is_mid_layer_timer_pending(stub),
		 stub->reason );
	}
	list_for_each_entry (host, &mpl_hosts, host_list) {
	        scsi_host = host->scsi_host ;
		printk(KERN_ERR "Scsi host no %d scsi _host busy %u host_failed %u"
		"host_self_blocked %u host_blocked %u host_state %lu \n",
		scsi_host->host_no, scsi_host->host_busy, scsi_host->host_failed, 
		scsi_host->host_self_blocked, scsi_host->host_blocked, scsi_host->shost_state );
		/* Getting every scsi cmd from lpfc_queue */
		list_for_each_entry(stub,&host->lpfc_queue,stub_queue) {
			if (stub->flag & MPL_INTERNAL_CMD_STUB)
			  continue;
			else
	   			printk(KERN_ERR "lpfc_Q:scsi_cmnd x%p Cmnd x%x Path %s timer %d  reason %d \n",
				stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path),is_mid_layer_timer_pending(stub),
				 stub->reason );
		}	
		/* Getting every thing from pending queus */
		list_for_each_entry(device ,&host->devices, devices) {
			list_for_each_entry(stub,&device->pending_ios, stub_queue) {
				if (stub->flag & MPL_INTERNAL_CMD_STUB){
	   				printk(KERN_ERR "Pend_Q:scsi_cmnd INTERNAL## x%p Cmnd x%x Path %s  reason %d flag%d \n",
					stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path), stub->reason, stub->flag );
			 		 continue;
				}
				else
	   				printk(KERN_ERR "Pend_Q:scsi_cmnd x%p Cmnd x%x Path %s timer %d reason %d flag%d \n",
					stub->cmdp,stub->cmd.cmnd[0],mpl_print_path(stub->path),is_mid_layer_timer_pending(stub),
					 stub->reason, stub->flag );
			}
				
		}

	}
	spin_unlock_irqrestore(&stub_queue_lock, flags);

}
#endif
int debug_cnt = 0;

static void
mpl_hb_done(struct scsi_cmnd *cmd)
{
	struct io_stub *stub, *deferral_stub;
	struct mpl_path *path;
	struct mpl_device *device;
	unsigned long flags;

	/* If timer fired, this command will be handled by timeout handler */
        if (!del_timer(&cmd->eh_timeout)) {
		return;
	}

	mpl_module_lock();
	
	spin_lock_irqsave(&stub_queue_lock, flags);
	/* Map cmnd to corresponding stub */
	stub = mpl_locate_stub(cmd);
	stub = mpl_find_stub_in_lpfcq(stub);
	if (stub) {
		mpl_remove_stub_from_lpfcq(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
	}
	else {
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_print(0, MPL_ERR "HBeat cmpl: NO stub\n");
		mpl_module_unlock();
		return;
	}

	path = stub->path;
	device = path->device;
	if ((stub->flag & MPL_ABORT_FAILED) != 0) {
		atomic_dec(&path->host->inflight_hbeat_count);
		mpl_print(0, MPL_ERR "Heartbeat cmd 0x%x completed after "
			  "abort failed for %s %s, result = %08x\n",
			  cmd->cmnd[0],
			  mpl_print_device(path->device),
			  mpl_print_path(path), cmd->result);
		mpl_stub_free(stub);
		mpl_module_unlock();
		goto out ;
	}
	if (!(stub->flag & MPL_INTERNAL_CMD_STUB)) {
		mpl_print(0, MPL_ERR "HBeat cmpl: Bad stub %p: %d %s\n",
		stub, stub->flag, mpl_print_path(path));
		mpl_stub_free(stub);
		mpl_module_unlock();
		return;
	}

	mpl_print(MPL_INTERNAL, MPL_INFO "Cmpl cmd %x %x for %s %s "
		  "result %x sk %x asc %x ascq %x\n",
		  cmd->cmnd[0], cmd->cmnd[2],
		  mpl_print_device(device), mpl_print_path(path),
		  cmd->result, cmd->sense_buffer[2],
		  cmd->sense_buffer[12], cmd->sense_buffer[13]); 

	if ((path->state == PATH_DISABLED) || (path->hbstate == HB_STOPPED)) {
		atomic_dec(&path->host->inflight_hbeat_count);
		mpl_print(MPL_INTERNAL, MPL_INFO
			"HBeat cmpl: %s Invalid state: %d %d\n",
			mpl_print_path(path), path->state, path->hbstate);
		mpl_stub_free(stub);
		mpl_module_unlock();
		goto out ;
	}

	if (mpl_is_power_on_reset(cmd->result, cmd->sense_buffer) ||
	   (path->tpgs && mpl_is_sas_changed(cmd->result, cmd->sense_buffer)) ||
	   ((mpl_is_lu_not_ready(cmd->result, cmd->sense_buffer)) &&
		((path->hbstate == HB_TUR) && path->retries < 5))) {
		/* Set the time out handler */
		if ( stub->cmd.cmnd[0] == INQUIRY ) {
			mpl_set_cmd_timer(stub, MPL_INQUIRY_CMD_TMO);
		} else if ( stub->cmd.cmnd[0] == TEST_UNIT_READY ) {
			mpl_set_cmd_timer(stub, MPL_TUR_CMD_TMO);
		} else {
			mpl_set_cmd_timer(stub, MPL_INTERNAL_CMD_TMO);
		}
		/* Just retry 29/00, QFULL and 04/01 errors on TUR */
		memset(&cmd->sense_buffer[0], 0, SCSI_SENSE_BUFFERSIZE);
		atomic_dec(&path->host->inflight_hbeat_count);
		path->retries++;
		spin_lock_irqsave(&stub_queue_lock, flags);
		stub->reason = MPL_HEARTBEAT_CMD;
		mpl_add_stub_to_deferral(stub);
		spin_unlock_irqrestore(&stub_queue_lock, flags);
		mpl_module_unlock();
		mpl_wake_thread();
		return;
	}

	if (cmd->cmnd[0] == REPORT_TGT_PORT_GROUP)  {
		device->tpg_cmd_inp = 0;
	}
	spin_lock_irqsave(&stub_queue_lock, flags);
	stub->reason = MPL_HB_CMD;
	mpl_add_stub_to_deferral(stub);
	spin_unlock_irqrestore(&stub_queue_lock, flags);
	mpl_module_unlock();
	mpl_wake_thread();
	return;

out:
	mpl_module_lock();
	 if( ((atomic_read(&path->host->inflight_hbeat_count)) < path->host->hbeat_canqueue) 
		&& ( !list_empty(&path->host->hbeat_deferral_queue) )) {
		spin_lock_irqsave(&stub_queue_lock, flags) ;
		mpl_print( MPL_HBEATDEFQ, MPL_INFO "mpl_hb_done: Putting hbeat from hbeat"
			" deferrel queue to deferral queue \n" );
		deferral_stub = mpl_get_first_hbeat_deferral(path->host) ;
		if (deferral_stub) {
			/* Added timer to heart beat stub because we deleted the timer when we put 
			 *into hbeat * deferral queue. I also want to wakeup thread from here 
			 *because we just added one hbeat to deferral.
			 */
			mod_timer(&deferral_stub->cmd.eh_timeout, jiffies + 3600 * HZ ) ;
			mpl_add_stub_to_deferral(deferral_stub);
			spin_unlock_irqrestore(&stub_queue_lock,flags);
			mpl_module_unlock();
			mpl_wake_thread();
			return ;
			
		}
		else {
			mpl_print(MPL_HBEATDEFQ, MPL_WARNING "mpl_hb_done: Locking issue \n");
		}
		 spin_unlock_irqrestore(&stub_queue_lock,flags);
	 }
	 else {
		 mpl_print( MPL_HBEATDEFQ, MPL_INFO "mpl_hb_done: Not moving because "
		" either no bandwidth or queue empty \n" ) ;
	}
	mpl_module_unlock();
	return ;
}

#define hb_transition(state, event) ((state << 3) | event)

void
mpl_hbeat_state_machine(struct mpl_path *path, enum mpl_hb_event event)
{
	static char *states[] = {
		"HB_STOPPED",
		"HB_TUR",
		"HB_RTPG",
		"HB_INQSN",
		"HB_WAITING",
	};
	static char *events[] = {
		"HB_START",
		"HB_STOP",
		"HB_SUCCEED",
		"HB_FAIL",
		"HB_TIMEOUT",
		"HB_BUSY",
	};
	struct io_stub *stub;
	enum mpl_hb_state orig_hbstate = path->hbstate;
	unsigned long flags;
	int  expires = 0;
	int  wake_it = 0;

	if ((unsigned) path->hbstate > NUM_ENTRIES(states)) {
		mpl_print(0, MPL_ERR "Invalid path heartbeat state: %d\n",
			  path->hbstate);
		return;
	}
	if ((unsigned) event > NUM_ENTRIES(events)) {
		mpl_print(0, MPL_ERR "Invalid heartbeat event: %d\n", event);
		return;
	}

	mpl_print(MPL_HBSM, MPL_INFO "HbeatSM IN state %s, event %s "
		  "for %s %s blk %d (%p)\n",
		  states[path->hbstate], events[event],
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  (path->host->blocked || path->lld_blocked),
		  (&path)[-1]);

	switch (hb_transition(path->hbstate, event)) {
	/* HB_START event */
	case hb_transition(HB_TUR, HB_START):
	case hb_transition(HB_RTPG, HB_START):
	case hb_transition(HB_INQSN, HB_START):
		mpl_print(MPL_HEARTBEAT,
			  MPL_INFO "Heartbeat already started: %s, %s, "
			  "state %s, event %s from (%p)\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  states[path->hbstate],
			  events[event],
			  (&path)[-1]);
		break;

	case hb_transition(HB_WAITING, HB_START):
		mpl_module_unlock();
		del_timer_sync(&path->hbeat);
		mpl_module_lock();
		/* Drop thru to initiate TUR */

	case hb_transition(HB_STOPPED, HB_START):
		/* Drop thru to initiate TUR */

	/* HB_TIMEOUT event */
	case hb_transition(HB_WAITING, HB_TIMEOUT):
		if(!path->got_wwpn_data && path->host->hba) {
			/* Get the WWPN for this path */
			path->got_wwpn_data = 
				mpl_get_tgt_wwinfo(path->host->hba,
				path->target, path->tgt);
                }
		if (path->host->blocked || path->lld_blocked ||
		    !path->sdev) {
			if (!path->sdev) {
				mpl_path_state_machine(path, PATH_DOWN);
			}
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_fail_interval;
			path->flag |= PATH_FAST_POLL;
			break;
		}
		if (!path->host->scsi_host ||
		    !path->host->scsi_host->hostdata[0]) {
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_interval;
			break;
		}
		stub = mpl_setup_tur(path);
		if (!stub) {
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_interval;
		} else {
			path->hbstate = HB_TUR;
				/* Don't issue new commands in timer
				 * context.
				 */
			path->retries = 0;
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub->reason = MPL_HEARTBEAT_CMD;
			mpl_add_stub_to_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_module_unlock();
			mpl_wake_thread();
			mpl_module_lock();
		}
		break;

	case hb_transition(HB_STOPPED, HB_TIMEOUT):
	case hb_transition(HB_TUR, HB_TIMEOUT):
	case hb_transition(HB_RTPG, HB_TIMEOUT):
	case hb_transition(HB_INQSN, HB_TIMEOUT):
	case hb_transition(HB_WAITING, HB_BUSY):
		break;

	case hb_transition(HB_TUR, HB_BUSY):
	case hb_transition(HB_RTPG, HB_BUSY):
	case hb_transition(HB_INQSN, HB_BUSY):
		path->hbstate = HB_WAITING;
		expires = mpl_hbeat_fail_interval;
		path->flag |= PATH_FAST_POLL;
		break;

	/* HB_SUCCEED event */
	case hb_transition(HB_TUR, HB_SUCCEED):
		path->failures = 0;
		if (path->device->alua_support && path->tpgs && 
			!path->device->tpg_cmd_inp &&
			!mpl_is_out_lun_own_chg(path->device)) {
			stub = mpl_setup_report_tgt_port(path);
			path->device->tpg_cmd_inp = 1;
			path->hbstate = HB_RTPG;
		}
		else {
			stub = mpl_setup_inquiry(path, (unsigned char)0x83);
			path->hbstate = HB_INQSN;
		}
		if (!stub) {
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_interval;
		} else {
			path->retries = 0;
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub->reason = MPL_HEARTBEAT_CMD;
			mpl_add_stub_to_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			mpl_wake_thread();
		}
		break;

	case hb_transition(HB_RTPG, HB_SUCCEED):
		path->failures = 0;
		stub = mpl_setup_inquiry(path, (unsigned char)0x83);
		if (!stub) {
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_interval;
		} else {
			path->hbstate = HB_INQSN;
			path->retries = 0;
			spin_lock_irqsave(&stub_queue_lock, flags);
			stub->reason = MPL_HEARTBEAT_CMD;
			mpl_add_stub_to_deferral(stub);
			spin_unlock_irqrestore(&stub_queue_lock, flags);
			wake_it = 1;
		}
		break;
		
	case hb_transition(HB_INQSN, HB_SUCCEED):
		path->failures = 0;
		if (mpl_is_path_configured(path)) {
			if ((!path->device->plugin) ||
				path->device->plugin->path_up(path)) {

				/* path_up plugin will return TRUE if path
				 * is really up. If extra steps need to be
				 * taken, PSM for PATH_UP will be called by
				 * plugin subsystem for device.
				 */
				mpl_path_state_machine(path, PATH_UP);

				/* If we were DEV_FAILED, PATH_UP will
				 * only bring us to DEV_FAILING.
				 */
				if (path->device->state == DEV_FAILING) {
					expires = mpl_hbeat_fail_interval;
					path->flag |= PATH_FAST_POLL;
				}
			}
			if (!expires)
				expires = mpl_hbeat_interval;
			path->hbstate = HB_WAITING;
		}
		else {
			path->hbstate = HB_STOPPED;
			mpl_flush_hbeat(path);
			mpl_path_state_machine(path, PATH_DOWN);
			mpl_path_state_machine(path, PATH_DISABLE);
		}
		break;

	case hb_transition(HB_STOPPED, HB_SUCCEED):
	case hb_transition(HB_WAITING, HB_SUCCEED):
		break;

	/* HB_FAIL event */
	case hb_transition(HB_TUR, HB_FAIL):
	case hb_transition(HB_RTPG, HB_FAIL):
	case hb_transition(HB_INQSN, HB_FAIL):
		if (mpl_is_path_configured(path)) {
			/* If a HBeat command times out, retry it once
			 * with a faster timeout to avoid long failover
			 * times.
			 */
			if (!path->failures && (path->flag & PATH_HBEAT_TMO)) {
				path->hbstate = HB_WAITING;
				expires = mpl_hbeat_fail_interval;
				path->flag |= PATH_FAST_POLL;
				path->failures++; /* also causes shorter tmo */
				break;
			}
			path->failures = 0;
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_fail_interval;
			path->flag |= PATH_FAST_POLL;
			mpl_path_state_machine(path, PATH_DOWN);
		}
		else {
			/* TUR, RTPG or INQ while unconfigured only happens
			 * during initial Inquiry for a new path. In this case
			 * keep sending till we are successful or the internal
			 * retry exceeded.
			 */
			if (path->failures > mpl_max_hbeat_retry) {
				path->hbstate = HB_STOPPED;
				mpl_flush_hbeat(path);

				mpl_print(0, MPL_ERR "%s %s Heart beat failure "
					"exceeded max retry :%d \n",
					mpl_print_device(path->device),
					mpl_print_path(path),
					mpl_max_hbeat_retry);

				mpl_path_state_machine(path, PATH_DOWN);
				break;
			}
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_fail_interval;
			path->flag |= PATH_FAST_POLL;
			path->failures++;
		}
		break;

	case hb_transition(HB_WAITING, HB_FAIL):
		if (mpl_is_path_configured(path)) {
			path->hbstate = HB_WAITING;
			expires = mpl_hbeat_fail_interval;
			path->flag |= PATH_FAST_POLL;
			mpl_path_state_machine(path, PATH_DOWN);
		}
		else {
			path->hbstate = HB_STOPPED;
			mpl_flush_hbeat(path);
			mpl_path_state_machine(path, PATH_DOWN);
		}
		break;

	case hb_transition(HB_STOPPED, HB_FAIL):
		break;

	/* HB_STOP event */
	case hb_transition(HB_RTPG, HB_STOP):
	case hb_transition(HB_WAITING, HB_STOP):
	case hb_transition(HB_TUR, HB_STOP):
	case hb_transition(HB_INQSN, HB_STOP):
				/* We're called from path state machine to
				 * stop the heartbeat.  So, in the future, be
				 * carfeful of adding calls back into the path
				 * state machine here. */
		mpl_module_unlock();
		del_timer_sync(&path->hbeat);
		mpl_module_lock();

		path->hbstate = HB_STOPPED;
		mpl_flush_hbeat(path);

		mpl_path_state_machine(path, PATH_DOWN);
		break;

	case hb_transition(HB_STOPPED, HB_STOP):
		break;

	default:
		mpl_print(0, MPL_ERR "Odd heartbeat transition: %s, %s, "
			  "state %s, event %s from (%p)\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  states[path->hbstate],
			  events[event],
			  (&path)[-1]);
	}
	if (path->hbstate == HB_WAITING) {
		if (path->flag & PATH_FAST_POLL) {
			mpl_module_unlock();
			mod_timer(&path->hbeat, jiffies + expires * HZ);
			mpl_module_lock();
		}
		else {
			path->hbsm_io_success = 0;
			path->hbsm_io_error = 0;
		}
	} else if ((event != HB_TIMEOUT) && timer_pending(&path->hbeat)) {
				/* Don't del_timer_sync.  That can deadlocks
				 * us if this I/O completion interrupted
				 * __run_timers(). */
		mpl_module_unlock();
		del_timer(&path->hbeat);
		mpl_module_lock();
		mpl_print(MPL_HEARTBEAT, MPL_INFO "State %s w/ active tmr on %s %s.  "
			  "Deleted..\n",
			  states[path->hbstate],
			  mpl_print_device(path->device),
			  mpl_print_path(path));
	}

	mpl_print(MPL_HBSM, MPL_INFO "HbeatSM state %s, event %s "
		  "result %s for %s %s (%p)\n",
		  states[orig_hbstate], events[event], states[path->hbstate],
		  mpl_print_device(path->device),
		  mpl_print_path(path),
		  (&path)[-1]);

	if (wake_it) {
		mpl_module_unlock();
		mpl_wake_thread();
		mpl_module_lock();
	}
	return;
}

#define path_transition(state, event) ((state << 3) | event)

void
mpl_path_state_machine(struct mpl_path *path, enum mpl_path_event event)
{
	static char *events[] = {
		"DOWN",
		"UP",
		"DISABLE",
		"ENABLE",
		"FAILBACK",
		"FAILOVER",
	};
	enum mpl_path_state orig_state = path->state;
	struct mpl_path *newpath;
	int log = 0;

	if ((unsigned) path->state > NUM_ENTRIES(mpl_pstates)) {
		mpl_print(0, MPL_ERR "Invalid path state: %d\n", path->state);
		return;
	}
	if ((unsigned) event > NUM_ENTRIES(events)) {
		mpl_print(0, MPL_ERR "Invalid path event: %d\n", event);
		return;
	}

	path->refcnt++;

	/* Filter out no action transitions */
	if ((event != PATH_UP) ||
		((path->state != PATH_ONLINE) && (path->state != PATH_STANDBY))) {
		log = 1;
		mpl_print(MPL_PSM, MPL_INFO "PSM IN %s %s state %s event %s (%p)\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  mpl_pstates[orig_state],
			  events[event],
			  (&path)[-1]);
	}

	switch (path_transition(path->state, event)) {

	/* PATH_DOWN event */
	case path_transition(PATH_ONLINE, PATH_DOWN):
	case path_transition(PATH_STANDBY, PATH_DOWN):
				/* Must set state before plugin is called */
		path->state = PATH_FAILED;
		mpl_device_state_machine(path->device, path, DEV_PATH_DOWN);
		if (path->device->plugin) {
			path->device->plugin->path_down(path);
		}
		break;

	case path_transition(PATH_DISABLED, PATH_DOWN):
	case path_transition(PATH_FAILED, PATH_DOWN):
		path->flag &= ~PATH_JUST_ENABLED;
		break;


	/* PATH_DISABLE event */
	case path_transition(PATH_ONLINE, PATH_DISABLE):
	case path_transition(PATH_STANDBY, PATH_DISABLE):
		path->state = PATH_DISABLED;
		mpl_hbeat_state_machine(path, HB_STOP);
		mpl_device_state_machine(path->device, path, DEV_PATH_DOWN);

		mpl_module_unlock();
		del_timer_sync(&path->sched_cmd);
		mpl_module_lock();
		path->flag &= ~PATH_NEEDS_ALL_CMDS;

		path->flag = 0;
		if (path->device->plugin) {
			path->device->plugin->path_down(path);
		}
		break;

	case path_transition(PATH_FAILED, PATH_DISABLE):
		path->state = PATH_DISABLED;
		path->flag &= ~PATH_JUST_ENABLED;
		mpl_hbeat_state_machine(path, HB_STOP);

		mpl_module_unlock();
		del_timer_sync(&path->sched_cmd);
		mpl_module_lock();
		path->flag &= ~PATH_NEEDS_ALL_CMDS;
		break;

	case path_transition(PATH_DISABLED, PATH_DISABLE):
		break;

	/* PATH_UP event */
	case path_transition(PATH_FAILED, PATH_UP):
		if (mpl_is_path_primary(path) &&
			!(path->flag & PATH_JUST_ENABLED) &&
			!mpl_is_auto_failback(path->device)) {
			path->state = PATH_DISABLED;
			mpl_hbeat_state_machine(path, HB_STOP);

			mpl_module_unlock();
			del_timer_sync(&path->sched_cmd);
			mpl_module_lock();
			path->flag &= ~PATH_NEEDS_ALL_CMDS;
			break;
		}

		mpl_device_state_machine(path->device, path, DEV_PATH_UP);
		path->state = (path == path->device->last_path
			       ? PATH_ONLINE
			       : PATH_STANDBY);

		mpl_hbeat_state_machine(path, HB_START);

		/* If we just enabled the primary path, try to fail it back */
		if (mpl_is_path_primary(path) &&
			(path->flag & PATH_JUST_ENABLED)) {
			path->flag &= ~PATH_JUST_ENABLED;
			goto pfback;
		}
		path->flag &= ~PATH_JUST_ENABLED;
		break;

	case path_transition(PATH_ONLINE, PATH_UP):
	case path_transition(PATH_STANDBY, PATH_UP):
				/* Notify device we're still up. */
		mpl_device_state_machine(path->device, path, DEV_PATH_UP);
		path->state = (path == path->device->last_path
			       ? PATH_ONLINE
			       : PATH_STANDBY);
		break;

	case path_transition(PATH_DISABLED, PATH_UP):
		break;

	/* PATH_ENABLE event */
	case path_transition(PATH_DISABLED, PATH_ENABLE):
		path->state = PATH_FAILED;
		mpl_hbeat_state_machine(path, HB_START);
		path->flag |= PATH_JUST_ENABLED;
		break;

	case path_transition(PATH_FAILED, PATH_ENABLE):
	case path_transition(PATH_ONLINE, PATH_ENABLE):
		break;

	case path_transition(PATH_STANDBY, PATH_ENABLE):
		if (!mpl_is_path_primary(path)) {
			break;
		}
pfback:
		if(path->device->last_path != path)
			path->device->failing_path = path->device->last_path;
		path->device->last_path = path;
		/* Drop thru to FAILBACK */

	/* PATH_FAILBACK event */
	case path_transition(PATH_FAILED, PATH_FAILBACK):
	case path_transition(PATH_STANDBY, PATH_FAILBACK):
		/* Only one path can be online */
		list_for_each_entry(newpath, &path->device->paths, path_list) {
			if ((newpath != path) && (newpath->state == PATH_ONLINE)) {
				newpath->state = PATH_STANDBY;
			}
		}
		path->flag |= PATH_FAIL_BACK_EVT;
		path->state = PATH_STANDBY;
		if ((!path->device->plugin) ||
			path->device->plugin->path_up(path)) {

			/* path_up plugin will return TRUE if path
			 * is really up. If extra steps need to be
			 * taken, PSM for PATH_UP will be called by
			 * plugin subsystem for device.
			 */
			mpl_path_state_machine(path, PATH_UP);
		}
		break;

	case path_transition(PATH_ONLINE, PATH_FAILBACK):
	case path_transition(PATH_DISABLED, PATH_FAILBACK):
		break;

	/* PATH_FAILOVER event */
	case path_transition(PATH_STANDBY, PATH_FAILOVER):

		path->flag |= PATH_FAIL_OVER_EVT;
		path->state = PATH_ONLINE;
		/* Only one path can be online */
		list_for_each_entry(newpath, &path->device->paths, path_list) {
			if ((newpath != path) && (newpath->state == PATH_ONLINE)) {
				newpath->state = PATH_STANDBY;
			}
		}
		/* Restart Hbeat to force TUR */
		mpl_hbeat_state_machine(path, HB_START);
		break;

	case path_transition(PATH_FAILED, PATH_FAILOVER):
	case path_transition(PATH_ONLINE, PATH_FAILOVER):
	case path_transition(PATH_DISABLED, PATH_FAILOVER):
		break;
		
	default:
		mpl_print(0, MPL_ERR "Odd path state transition: %s, %s, "
			  "state %s, event %s from (%p)\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  mpl_pstates[path->state],
			  events[event],
			  (&path)[-1]);
	}

	if (log) {
		mpl_print(MPL_PSM, MPL_INFO "PSM %s %s state %s event %s result %s (%p)\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  mpl_pstates[orig_state],
			  events[event],
			  mpl_pstates[path->state],
			  (&path)[-1]);
	}

	path->refcnt--;
	if ((orig_state != path->state) &&
		!driver_unloading &&
		(mpl_device_path_cnt(path->device) > 1) &&
		(path->refcnt <= 1)) {
		if ((event == PATH_FAILOVER) || (event == PATH_FAILBACK)) {
			mpl_print(MPL_DFLT, MPL_INFO "DevSM  %s %s%s: event %s\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  (mpl_is_path_primary(path)
				     ? " primary"
				     : ""),
			  events[event]);
		}
		else {
			mpl_print(MPL_DFLT, MPL_INFO "PathSM %s %s%s: event %s: From %s to %s\n",
			  mpl_print_device(path->device),
			  mpl_print_path(path),
			  (mpl_is_path_primary(path)
				     ? " primary"
				     : " "),
			  events[event],
			  mpl_pstates[orig_state],
			  mpl_pstates[path->state]);
		}
	}
	return;
}

static int
mpl_select_next_path(struct mpl_device *device, struct mpl_path *first_path)
{
	struct mpl_path *path = first_path;
	int  found = 0;

				/* Find path on active controller if
				 * possible, preferring default path. */
	if (path->device->alua_support) {
		do {
			path = mpl_get_next_path(path);

			if (mpl_is_path_primary(path) &&
			   (path->tpgs && (path->sas == MPL_ALUA_ACTIVE_OPT)) &&
			   (path->state == PATH_ONLINE ||
			    path->state == PATH_STANDBY)) {
				found = 1;
				device->last_path = path;
				goto done;
			}
		} while (path != first_path);
		do {
			path = mpl_get_next_path(path);

			if ((path->tpgs && (path->sas == MPL_ALUA_ACTIVE_OPT)) &&
			    (path->state == PATH_ONLINE ||
			     path->state == PATH_STANDBY)) {
				found = 1;
				device->last_path = path;
				goto done;
			}
		} while (path != first_path);
	}
	else {
		do {
			path = mpl_get_next_path(path);

			if (mpl_is_path_primary(path) &&
			    path->active_controller &&
			    (path->state == PATH_ONLINE ||
			     path->state == PATH_STANDBY)) {
				found = 1;
				device->last_path = path;
				goto done;
			}
		} while (path != first_path);
		do {
			path = mpl_get_next_path(path);

			if (path->active_controller &&
			    (path->state == PATH_ONLINE ||
			     path->state == PATH_STANDBY)) {
				found = 1;
				device->last_path = path;
				goto done;
			}
		} while (path != first_path);
	}
	do {
		path = mpl_get_next_path(path);
		if (path->state == PATH_ONLINE ||
		    path->state == PATH_STANDBY) {
			found = 1;
				/* send start/stop on MSA */
			device->last_path = path;
			goto done;
		}
	} while (path != first_path);

done:
	return found;
}

#define device_action(state, event) ((state << 3) | event)

void
mpl_device_state_machine(struct mpl_device *device,
			 struct mpl_path *path,
			 enum mpl_device_event event)
{
	static char *events[] = {
		"DEV_PATH_UP",
		"DEV_PATH_DOWN",
		"DEV_DISABLE",
	};
	struct mpl_path *last_path = device->last_path;
	enum mpl_device_state orig_state = device->state;
	int log = 0;

	if ((unsigned) device->state > NUM_ENTRIES(mpl_dstates)) {
		mpl_print(0, MPL_ERR "Invalid device state: %d\n",
			  device->state);
		return;
	}
	if ((unsigned) event > NUM_ENTRIES(events)) {
		mpl_print(0, MPL_ERR "Invalid device event: %d\n", event);
		return;
	}

	/* Filter out no action transitions */
	if ((event != DEV_PATH_UP) || (device->state != DEV_ONLINE) ||
		((path->state != PATH_ONLINE) && (path->state != PATH_STANDBY))) {
		log = 1;

		mpl_print(MPL_DSM, MPL_INFO "DSM IN %s state %s event %s (%p)\n",
			  mpl_print_device(device),
			  mpl_dstates[orig_state],
			  events[event],
			  (&device)[-1]);
	}

	switch (device_action(device->state, event)) {

	/* DEV_PATH_UP event */
	case device_action(DEV_ONLINE, DEV_PATH_UP):
		if (path == last_path) {
			mpl_device_process_pendq(device);
			break;
		}
		if (mpl_is_auto_failback(device) &&
		     mpl_is_path_primary(path)) {
			if(last_path != path)
				device->failing_path = last_path;
			device->last_path = path;
			/* If we changed to a new IO path */
			if (last_path != path) {
				mpl_path_state_machine(path, PATH_FAILBACK);
			}
		}
		break;

	case device_action(DEV_FAILED, DEV_PATH_UP):
		if(last_path != path)
			device->failing_path = last_path;
		device->last_path = path;
		device->state = DEV_FAILING;
		break;

	case device_action(DEV_FAILING, DEV_PATH_UP):
		if (path == last_path) {
			device->state = DEV_ONLINE;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
			mpl_scsi_device_unblock(device);
#endif
			if (timer_pending(&device->nopath)) {
				mpl_module_unlock();
				del_timer_sync(&device->nopath);
				mpl_module_lock();
			}
			mpl_device_process_pendq(device);
		}
		break;

	case device_action(DEV_DISABLED, DEV_PATH_UP):
		break;

	/* DEV_PATH_DOWN event */
	case device_action(DEV_ONLINE, DEV_PATH_DOWN):
		if (path != last_path) {
			break;
		}
		if (!mpl_select_next_path(device, path)) {
			device->state = DEV_FAILED;
			if (device->nopath_tmo) {
				mpl_module_unlock();
				mod_timer(&device->nopath,
					jiffies + device->nopath_tmo * HZ);
				mpl_module_lock();
			}
			else {
				mpl_device_process_pendq(device);
			}
		} else {
			device->failing_path = last_path;
			device->state = DEV_FAILING;
			/* If we changed to a new IO path */
			if (last_path != device->last_path) {
				mpl_path_state_machine(device->last_path, PATH_FAILOVER);
			}
		}

		break;

	case device_action(DEV_FAILING, DEV_PATH_DOWN):
		if (path != last_path) {
			break;
		}
		if (!mpl_select_next_path(device, path)) {
			device->state = DEV_FAILED;
			if (device->nopath_tmo &&
				mpl_is_path_configured(path)) {
				mpl_module_unlock();
				mod_timer(&device->nopath,
					jiffies + device->nopath_tmo * HZ);
				mpl_module_lock();
			}
			else {
				mpl_device_process_pendq(device);
			}
		} else {
			device->failing_path = last_path;
			/* If we changed to a new IO path */
			if (last_path != device->last_path) {
				mpl_path_state_machine(device->last_path, PATH_FAILOVER);
			}
		}
		break;

	case device_action(DEV_DISABLED, DEV_PATH_DOWN):
	case device_action(DEV_FAILED, DEV_PATH_DOWN):
		break;

	/* DEV_DISABLE event */
	case device_action(DEV_ONLINE, DEV_DISABLE):
	case device_action(DEV_FAILING, DEV_DISABLE):
	case device_action(DEV_FAILED, DEV_DISABLE):
		device->state = DEV_DISABLED;
		mpl_device_process_pendq(device);
		break;

	case device_action(DEV_DISABLED, DEV_DISABLE):
		break;

	default:
		mpl_print(0, MPL_ERR "Odd device state/event: %s, "
			  "state %s, event %s from (%p)\n",
			  mpl_print_device(device),
			  mpl_dstates[device->state],
			  events[event],
			  (&path)[-1]);
	}
	
	if (log) {
		mpl_print(MPL_DSM, MPL_INFO "DSM %s state %s event %s "
			  "result %s newIO(%d)\n",
			  mpl_print_device(device),
			  mpl_dstates[orig_state],
			  events[event],
			  mpl_dstates[device->state],
			  (last_path != device->last_path));
	}
	return;
}

EXPORT_SYMBOL(scsi_host_alloc_wrapped);
EXPORT_SYMBOL(scsi_add_host_wrapped);
EXPORT_SYMBOL(scsi_host_put_wrapped);
EXPORT_SYMBOL(scsi_host_get_wrapped);
EXPORT_SYMBOL(scsi_remove_host_wrapped);
EXPORT_SYMBOL(scsi_track_queue_full_wrapped);
EXPORT_SYMBOL(scsi_adjust_queue_depth_wrapped);
EXPORT_SYMBOL(scsi_block_requests_wrapped);
EXPORT_SYMBOL(scsi_unblock_requests_wrapped);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
EXPORT_SYMBOL(fc_target_block_wrapped);
EXPORT_SYMBOL(fc_target_unblock_wrapped);
#else
EXPORT_SYMBOL(fc_remote_port_add_wrapped);
EXPORT_SYMBOL(fc_remote_port_delete_wrapped);
#endif
EXPORT_SYMBOL(fc_attach_transport_wrapped);

module_init(mpl_init);
module_exit(mpl_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Emulex Corporation - tech.support@emulex.com");
MODULE_DESCRIPTION("Emulex MultiPulse for Linux");
MODULE_VERSION(MPL_MODULE_VERSION);

